# -*- coding: utf-8 -*-
import datetime

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.db.models import Sum

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from product.models import Product, Vat
from django.contrib.auth.models import User

class BasketManager(models.Manager):
    def quantity(self, user):
        return self.get_query_set().filter(user=user).aggregate(quantity=Sum('quantity'))['quantity']
    def for_user(self, user):
        return self.get_query_set().filter(user=user)

class Basket(DateModel, UserModel):
    objects = BasketManager()

    user = models.ForeignKey(
        User,
        related_name='basket',
        verbose_name=_('User'),
    )
    product = models.ForeignKey(
        Product,
        related_name='basket',
        verbose_name=_('product'),
    )
    vat = models.ForeignKey(
        Vat,
        verbose_name=_("vat"),
    )
    quantity = models.IntegerField(
        default=0,
        verbose_name=_("quantity"),
    )
    price = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("price"),
    )
    fee = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("fee"),
    )
    discount = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("discount"),
    )

    class Meta:
        verbose_name = _('Basket')
        verbose_name_plural = _('Baskets')

    def __unicode__(self):
        return u'%s - %s' % (self.user.contact, self.product.name)

    def is_bad_basket(self):
        ## verifico quantita minima
        if self.product.shop.min_quantity > self.quantity:
            return "ATTENTION!!! Product error minimum quantity."

        ## verifico quantita massima
        if self.product.shop.max_quantity > 0 and self.product.shop.max_quantity < self.quantity:
            return "ATTENTION!!! Product error maximum quantity."

        ## verifico quantita multipla
        if self.product.shop.mult_quantity > 0 and (self.quantity % self.product.shop.mult_quantity) != 0:
            return "ATTENTION!!! Product error multiple quantity."

        ## verifico carrello pieno
        if self.product.shop.fill_basket:
            pass

        ## verifico richiesta form aggiuntiva e completezza eventuale
        if self.product.shop.request_form:
            pass

        ## verifico orario di ufficio
        if self.product.shop.office_time:
            now = datetime.datetime.now()
            weekday = now.weekday()
            office_time = settings.OFFICE_TIME
            day_time = office_time.get(weekday, None)
            if day_time:
                for interval in day_time:
                    if now.time() <= interval[0] or now.time() >= interval[1]:
                        return "ATTENTION!!! Product Time OUT of Range TIME."

        return False
    is_bad_basket = property(is_bad_basket)

    def get_total_income(self):
        return self.quantity * self.price
    total_income = property(get_total_income)

    def get_total_fee(self):
        return self.quantity * self.fee
    total_fee = property(get_total_fee)

    def get_total_discount(self):
        return self.quantity * self.discount
    total_discount = property(get_total_discount)

    def get_total(self):
        return self.total_income + self.total_fee - self.total_discount
    total = property(get_total)

