# -*- coding: utf-8 -*-
import datetime, decimal

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.conf.urls.defaults import *
from django import forms
from django.conf import settings
from django.contrib import messages
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.template.defaultfilters import floatformat

from jumbo.forms.models import AdminModelForm
from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin, SettingsAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.forms import DateWidget
from jumbo.forms.fields import ChainedChoiceField
from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete import AutocompleteModelChoiceField, AutocompleteInlineCharField
from jumbo.views.generic.create_oo_file import create_file
#from jumbo.utils.csv import render_csv

from ddt.models import Ddt, DdtRow, Settings
from procedure.models import ProcedureRow
from organization.models import Organization, Contact, Vat, Bank, Currency, Agent
from organization.models import OrganizationAddress, Delivery, M2MtoVector

### FORM
from store.models import Store

class DdtAdminForm(AdminModelForm):
    class Meta:
        model = Ddt

    def __init__(self, *args, **kws):
        super(DdtAdminForm, self).__init__(*args, **kws)
        self.fields['owner'] = AutocompleteModelChoiceField('procedure_owner', label=_("Owner"))
        self.fields['recipient'] = AutocompleteModelChoiceField('procedure_recipient', label=_("Recipient"), required=False)

class DdtRowAdminForm(AdminModelForm):
    class Meta:
        model = DdtRow

    def __init__(self, *args, **kws):
        super(DdtRowAdminForm, self).__init__(*args, **kws)
        self.fields['product'] = AutocompleteInlineCharField('procedure_product', label=_("Product"))

    def clean(self):
        if self.cleaned_data.has_key('procedurerow') and self.cleaned_data['procedurerow'] and\
           self.cleaned_data['quantity'] > self.cleaned_data['procedurerow'].quantity:
            raise forms.ValidationError("DDT Quantity greater than Procedure Row Quantity.")
        return self.cleaned_data


### INLINE
class DdtRowInline(admin.TabularInline):
    model = DdtRow
    form = DdtRowAdminForm
    extra = 0
    template = 'admin/ddt/ddtrow/tabular_ddt_row.html'
    fields = ('product', 'vat', 'store_cause', 'quantity', 'price', 'fee', 'discount')

### SEARCH
class DdtRowSearchForm(forms.ModelForm):
    class Meta:
        model = DdtRow
        fields = ('store_cause', 'vat')
        display_fields =    (('product__code__icontains','product__code',),
                             ('store_cause__store', 'store_cause', 'vat'),
                             ('ddt__date_d__year', 'search_in_all_years'))

    def __init__(self, *args, **kws):
        super(DdtRowSearchForm, self).__init__(*args, **kws)

        self.fields['product__code__icontains'] = forms.CharField(label=_("Product Code contains"), required=False)
        self.fields['product__code'] = forms.CharField(label=_("Product Code exact"), required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['ddt__date_d__year'] = forms.CharField(label=_("Year"), required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['store_cause__store'] = forms.ModelChoiceField(queryset=Store.objects.all(), label=_("Store"), required=False)

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('ddt__date_d__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(ddt__date_d__year = datetime.date.today().year)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs

class DdtSearchForm(forms.ModelForm):
    class Meta:
        model = Ddt
        fields = ('payment_form', 'store_cause')
        display_fields =    (('owner__name__istartswith', 'recipient__name_istartswith'),
                            ('payment_form', 'store_cause', 'store_cause__category'),
                            ('number__gte', 'number__lte'),
                            ('date_d__year', 'search_in_all_years'),
                            ('date_d__gte', 'date_d__lte'),
                             'procedure',)

    def __init__(self, *args, **kws):
        super(DdtSearchForm, self).__init__(*args, **kws)
        BOOLEAN = (('', '---'), ('1', _('Yes')), ('0', _('No')),)
        ORGANIZATIONTYPE_DEF = (("", _("-----"),), ("cus", _("customer"),), ("sup", _("supplier"),))

        self.fields['owner__name__istartswith'] = forms.CharField(label=_("Owner"), required=False)
        self.fields['recipient__name_istartswith'] = forms.CharField(label=_("Recipient"), required=False)
        self.fields['date_d__gte'] = forms.DateField(label=_("Date Begin"), widget= DateWidget, required=False)
        self.fields['date_d__lte'] = forms.DateField(label=_("Date End"), widget= DateWidget, required=False)
        self.fields['number__gte'] = forms.IntegerField(label=_("From Number"), required=False)
        self.fields['number__lte'] = forms.IntegerField(label=_("To Number"), required=False)
        self.fields['procedure'] = forms.ChoiceField(label=_("Procedure Complete"), choices=BOOLEAN, required=False)
        self.fields['store_cause__category'] = forms.ChoiceField(label=_("Category"), choices=ORGANIZATIONTYPE_DEF, required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['date_d__year'] = forms.CharField(label=_("Year"), required=False)

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('date_d__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(date_d__year = datetime.date.today().year)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs


### GENERAL
########################################
# DDT ROW                              #
########################################
class DdtRowAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("ddt/css/admin_ddt.css",)}

    form = DdtRowAdminForm
    search_form = DdtRowSearchForm
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'ddt', 'product', 'vat', 'store_cause', 'quantity', 'price', 'fee', 'discount')
        }),
    )
    search_fields = ('ddt', 'product', 'store_cause')

    list_display = ['get_date_d', 'get_number', 'get_detail', 'get_product_detail',
                    'quantity', 'price', 'fee', 'discount', 'invoice', 'str_total_income',
                    'str_total_fee', 'str_total_discount', 'str_total',
                    'get_cause_code', 'get_vat_code']

    list_display_csv = ['get_date_d', 'get_number', 'get_detail', 'get_product_detail',
                        'quantity', 'price', 'fee', 'discount', 'invoice', 'str_total_income',
                        'str_total_fee', 'str_total_discount', 'str_total',
                        'get_cause_code', 'get_vat_code']
    actions = ["create_invoice"]

    def get_actions(self, request):
        actions = super(DdtRowAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions

    def set_invoice_quantity(self, obj):
        return """<INPUT type="text" name="quantity_%s" size="3" maxlength="3">""" % obj.id
    set_invoice_quantity.short_description = _('INV Qty')
    set_invoice_quantity.allow_tags = True

    def get_date_d(self, obj):
        return obj.ddt.date_d.strftime("%d/%m/%Y")
    get_date_d.short_description = _('Date')
    get_date_d.admin_order_field = 'ddt__date_d'

    def get_number(self, obj):
        return obj.ddt.number or ""
    get_number.short_description = _('Number')
    get_number.admin_order_field = 'ddt__number'

    def get_detail(self, obj):
        owner = obj.ddt.owner
        recipient = obj.ddt.recipient or ""
        contact = obj.ddt.contact or ""
        if recipient == owner:
            recipient = ""
        return "%s<br/>%s<br/>%s" % (owner, recipient, contact)
    get_detail.short_description = _("Bill to")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'ddt__owner'

    def get_product_detail(self, obj):
        product = obj.get_description()
        return """<b><a href="%s/">%s</a></b>""" % (obj.id, product)
    get_product_detail.short_description = _("Product")
    get_product_detail.allow_tags = True
    get_product_detail.admin_order_field = 'description'

    def get_cause_code(sel, obj):
        return obj.store_cause.code
    get_cause_code.short_description = _('SC')

    def get_vat_code(self, obj):
        return obj.vat.code
    get_vat_code.short_description = _('VC')

    def str_total_income(self, obj):
        return obj.get_total_income()
    str_total_income.short_description = _('Total Income')

    def str_total_fee(self, obj):
        return obj.get_total_fee()
    str_total_fee.short_description = _('Total Fee')

    def str_total_discount(self, obj):
        return obj.get_total_discount()
    str_total_discount.short_description = _('Total Discount')

    def str_total(self, obj):
        return obj.get_total()
    str_total.short_description = _('Total')

    def create_invoice(self, request, queryset):
        """
        Crea Fatture e Righe Fatture da Righe DDT selezionate
        """
        change = ()
        invs_created = 0
        invs_not_created = 0
        rows_updated = 0
        rows_not_updated = 0
        post = request.POST

        ## ordino il queryset
        queryset = queryset.order_by(
            'ddt__owner',
            'ddt__bank', 'ddt__payment_form',
            'ddt__date_d',
            'ddt__number', 'ddt__id'
        )
        for obj in queryset:
            ddt_qty = post.get('quantity_%s' % obj.id, None)
            try:
                ddt_qty = decimal.Decimal(ddt_qty.replace(',','.'))
            except Exception, e:
                messages.error(request, _("Wrong Quantity"))
                rows_not_updated += 1
                continue

            ## cambio dati intestazione -> nuova Fattura
            if (
                obj.ddt.owner,
                obj.ddt.bank, obj.ddt.payment_form
            ) != change:
                try:
                    invoice, error = obj.create_invoice(quantity=ddt_qty)

                    if not ddt and error:
                        messages.error(request, error)
                        rows_not_updated += 1
                        invs_not_created += 1
                        continue
                    else:
                        rows_updated += 1
                        invs_created += 1
                        change = (
                            obj.procedure.owner,
                            obj.ddt.bank, obj.procedure.payment_form
                        )

                except Exception, e:
                    messages.error(request, e)
                    invs_not_created += 1
                    rows_not_updated += 1
                    continue

            ## stessi dati intestazione -> vecchio DDT
            else:
                try:
                    res, error = obj.add_to_invoice(invoice=invoice, quantity=ddt_qty)

                    if not res and error:
                        messages.error(request, error)
                        rows_not_updated += 1
                        continue

                    rows_updated += 1
                except Exception, e:
                    messages.error(request, e)
                    invs_not_updated += 1
                    continue

        if rows_updated == 0 and rows_not_updated == 0:
            messages.error(request, _("No Invoice Row was created"))
        if invs_created == 0 and invs_not_created == 0:
            messages.error(request, _("No Invoice was created"))

        if rows_not_updated > 0:
            if rows_not_updated == 1:
                message_bit = _("1 Invoice Row was")
            else:
                message_bit = _("%s Invoice Rows were") % rows_not_updated
            messages.error(request, _("%s not created.") % message_bit)

        if invs_not_created > 0:
            if invs_not_created == 1:
                message_bit = _("1 Invoice was")
            else:
                message_bit = _("%s Invoice were") % invs_not_created
            messages.error(request, _("%s not created.") % message_bit)

        if rows_updated > 0:
            if rows_updated == 1:
                message_bit = _("1 Invoice Row was")
            else:
                message_bit = _("%s Invoice Rows were") % rows_updated
            messages.info(request, _("%s successfully created.") % message_bit)

        if invs_created > 0:
            if invs_created == 1:
                message_bit = _("1 Invoice was")
            else:
                message_bit = _("%s Invoice were") % invs_created
            messages.info(request, _("%s successfully created.") % message_bit)

        return HttpResponseRedirect(request.META['HTTP_REFERER'])
    create_invoice.short_description = _("Create Invoice")

######################################
# DDT                                #
######################################
class DdtAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("ddt/css/admin_ddt.css",)}

    form = DdtAdminForm
    search_form = DdtSearchForm
    inlines = [DdtRowInline]
    actions = ["create_invoice"]
    date_hierarchy = 'date_d'
    fieldsets = (
        (_('logs admin'), {'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',), 'classes': ('collapse',)}),
        (None, {'fields': (
            'status',
            ('date_d', 'number',),
            ('owner', 'recipient',),
            'contact',
            'store_cause',
            ('vector', 'delivery',),
            ('date_delivery', 'shipping_address',),
            ('gross_weight', 'net_weight',),
            ('description_of_goods', 'number_of_packages',),
            'note'
        )}),
    )
    search_fields = ('owner__code', 'owner__name', 'recipient__code', 'recipient__name', 'number', 'date_d')
    list_display = (
        'get_date_d', 'number', 'get_detail', 'get_payment_form',
        'get_store_cause', 'get_income', 'get_fee', 'get_discount', 'get_vat',
        'get_total', 'get_invoice', 'create_odt', 'create_pdf', 'get_ddt_rows'
    )
    list_display_csv = (
        'get_date_d', 'number', 'owner', 'contact', 'payment_form', 'store_cause',
        'get_income', 'get_fee', 'get_discount', 'get_vat', 'get_total',
    )
    list_display_links = ['number']
    default_group_by = 'owner'
    save_as = True
    save_on_top = True

    def get_actions(self, request):
        actions = super(DdtAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions

    def get_invoice(self, obj):
        icon = "icon-no.gif"
        if obj.invoice:
            icon = "icon-yes.gif"
        ddtrows = """
            <a href="%(href)s?ddtrow__ddt__id=%(obj)s" title="%(title)s">
                <img alt="True" src="/media/img/admin/%(icon)s" />
            </a>
        """ % {
            'href'      :   reverse('admin:invoice_invoicerow_changelist'),
            'obj'       :   obj.id,
            'title'     :   ugt("List Invoice Rows"),
            'icon'      :   icon
        }
        return ddtrows
    get_invoice.short_description=_('INV')
    get_invoice.allow_tags = True
    get_invoice.admin_order_field='invoice'

    def get_date_d(self, obj):
        return obj.date_d.strftime("%d/%m/%Y")
    get_date_d.short_description = _('Date')
    get_date_d.admin_order_field = 'date_d'

    def get_detail(self, obj):
        owner = """<a href=%(reverse_url)s>%(owner)s</a>""" % {
            #'reverse_url': reverse('organization-detail', obj.owner.id),
            'reverse_url':  '',
            'owner':       obj.owner.name
        }
        contact = obj.contact or ""
        return "<b>%s</b><br/>%s" % (owner, contact)
    get_detail.short_description = _("Bill to")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'owner'

    def get_payment_form(self, obj):
        return """<span title="%s">%s</span>""" % (obj.payment_form.name, obj.payment_form.code)
    get_payment_form.short_description = _("Payment Form")
    get_payment_form.allow_tags = True
    get_payment_form.admin_order_field = 'payment_form__code'

    def get_store_cause(self, obj):
        return """<span title="%s">%s</span>""" % (obj.store_cause.name, obj.store_cause.code)
    get_store_cause.short_description = _("Store Cause")
    get_store_cause.allow_tags = True
    get_store_cause.admin_order_field = 'store_cause__code'

    def get_income(self, obj):
        return floatformat(obj.totals['income'], -2)
    get_income.short_description = _('Total Income')

    def get_fee(self, obj):
        return floatformat(obj.totals['fee'], -2)
    get_fee.short_description = _('Total Fee')

    def get_discount(self, obj):
        return floatformat(obj.totals['discount'], -2)
    get_discount.short_description = _('Total Discount')

    def get_vat(self, obj):
        return floatformat(obj.totals['vat'], -2)
    get_vat.short_description = _('Total VAT')

    def get_total(self, obj):
        return floatformat(obj.totals['total'], -2)
    get_total.short_description = _('Total')

    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(DdtAdmin, self).get_urls()
        my_urls = patterns('',
            url(
                r'^create_odt/(?P<obj_id>\d+)/$',
                self.admin_site.admin_view(create_file),
                {
                    'model':            Ddt,
                    'file_model':       'admin/ddt/ddt/ddt.odt',
                    'file_type':        'odt',
                },
                name='ddt_ddt_create_odt'
            ),
            url(
                r'^create_pdf/(?P<obj_id>\d+)/$',
                self.admin_site.admin_view(create_file),
                {
                    'model':            Ddt,
                    'file_model':       'admin/ddt/ddt/ddt.odt',
                    'file_type':        'pdf',
                },
                name='ddt_ddt_create_pdf'
            )
        )
        return my_urls + urls

    def create_odt(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/writer.gif' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:ddt_ddt_create_odt', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Create DDT ODT"),
        }
    create_odt.short_description = _("ODT")
    create_odt.allow_tags = True

    def create_pdf(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/pdf_icon.png' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:ddt_ddt_create_pdf', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Create Order PDF")
        }
    create_pdf.short_description = _("PDF")
    create_pdf.allow_tags = True

    def get_ddt_rows(self, obj):
        n_rows = obj.ddtrows.count()
        link = ""
        if n_rows > 0:
            link = """<a href="%(reverse_url)s" class="link_ddtrows" rel="#overlay">%(num_ddt)s</a>""" % {
                #'reverse_url': reverse('ddt-ddt-procedurerows', obj.id),
                'reverse_url': '',
                'num_ddt': n_rows,
            }

        return link
    get_ddt_rows.short_description = _('QR')
    get_ddt_rows.allow_tags = True

    def create_invoice(self, request, queryset):
        """
        Crea Fatture e Righe Fatture da DDT selezionati
        """
        change = ()
        invs_created = 0
        rows_updated = 0
        rows_not_updated = 0

        for obj in queryset.order_by(
            'owner', 'bank', 'payment_form',
            'date_d', 'number', 'id'
        ):
            for row in obj.ddtrows.all():
                ## cambio dati intestazione -> nuova Fattura
                if (obj.owner, obj.bank, obj.payment_form) != change:
                    try:
                        invoice, error = row.create_invoice()
                        if not invoice and error:
                            messages.error(request, error)
                            rows_not_updated += 1
                            continue
                        else:
                            change = (obj.owner, obj.bank, obj.payment_form)
                            invs_created += 1
                            rows_updated += 1
                    except Exception, e:
                        messages.error(request, e)
                        rows_not_updated += 1
                        continue

                ## stessi dati intestazione -> vecchio DDT
                else:
                    try:
                        res, error = row.add_to_invoice(invoice=invoice)
                        if not res and error:
                            messages.error(request, error)
                            rows_not_updated += 1
                            continue

                        rows_updated += 1
                    except Exception, e:
                        messages.error(request, e)
                        rows_not_updated += 1
                        continue

        if rows_updated == 0 and rows_not_updated == 0:
            messages.error(request, _("No Invoice Row was created"))

        if rows_not_updated > 0:
            if rows_not_updated == 1:
                message_bit = _("1 Invoice Row was")
            else:
                message_bit = _("%s Invoice Rows were") % rows_not_updated
            messages.error(request, _("%s not created.") % message_bit)

        if rows_updated > 0:
            if rows_updated == 1:
                message_bit = _("1 Invoice Row was")
            else:
                message_bit = _("%s Invoice Rows were") % rows_updated
            messages.info(request, _("%s successfully created.") % message_bit)

        if invs_created > 0:
            if invs_created == 1:
                message_bit = _("1 Invoice was")
            else:
                message_bit = _("%s Invoice were") % invs_created
            messages.info(request, _("%s successfully created.") % message_bit)

        return HttpResponseRedirect(request.META['HTTP_REFERER'])
    create_invoice.short_description = _("Create Invoice")

admin.site.register(Settings, SettingsAdmin)
admin.site.register(Ddt, DdtAdmin)
admin.site.register(DdtRow, DdtRowAdmin)
