# -*- coding: utf-8 -*-
from datetime import date, timedelta
import decimal

from django.conf import settings
from django.db import models, transaction
from django.contrib.auth.models import User
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.db.models import Settings as JSettings

from address.models import Address
from organization.models import Organization, Office, Contact, Bank, OrganizationAddress, Delivery, M2MtoVector as Vector
from payment.models import PaymentForm

from store.models import StoreCause
from organization.models import Vat
from product.models import Product
from procedure.models import ProcedureRow

###############################
# DDT                         #
###############################
class Ddt (DateModel, UserModel, StatusModel):
    ## ForeignKey
    owner = models.ForeignKey (
        Organization,
        related_name = "owner_ddts",
        db_index = True,
        verbose_name = _("Owner"),
    )
    recipient = models.ForeignKey (
        Organization,
        null = True, blank = True,
        related_name = "recipient_ddts",
        db_index = True,
        verbose_name = _("Recipient"),
    )
    bank = models.ForeignKey (
        Bank,
        blank = True, null = True,
        related_name = "ddts",
        db_index = True,
        verbose_name = _("Bank")
    )
    payment_form = models.ForeignKey (
        PaymentForm,
        blank = True, null = True,
        related_name = "ddts",
        db_index = True,
        verbose_name = _("Payment Form"),
    )
    office = models.ForeignKey (
        Office,
        blank = True, null = True,
        related_name = "ddts",
        db_index = True,
        verbose_name = _("Office"),
    )
    contact = models.ForeignKey (
        Contact,
        blank = True, null = True,
        related_name = "ddts",
        db_index = True,
        verbose_name = _("Contact"),
    )
    delivery = models.ForeignKey (
        Delivery,
        blank = True, null = True,
        related_name = "ddts",
        db_index = True,
        verbose_name = _("Delivery")
    )
    vector = models.ForeignKey (
        Vector,
        blank = True, null = True,
        related_name = "ddts",
        db_index = True,
        verbose_name = _("Vector")
    )
    store_cause = models.ForeignKey (
        StoreCause,
        related_name = "ddts",
        db_index = True,
        verbose_name = _("Store Cause")
    )
    shipping_address = models.ForeignKey (
        OrganizationAddress,
        blank = True, null = True,
        related_name = "ddts",
        db_index = True,
        verbose_name = _("Shipping Address")
    )
    ## Campi
    number = models.IntegerField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Number"),
        db_column = "dnumber"
    )
    date_d = models.DateField (
        db_index = True,
        verbose_name = _("Date DDT")
    )
    invoice = models.BooleanField (
        ## pratica già trasformata in fattura
        db_index = True,
        verbose_name = _("Invoice Complete"),
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    gross_weight = models.DecimalField(
        ## peso lordo in base ai kg
        blank = True, null = True,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Gross Weight in Kg"),
    )
    net_weight = models.DecimalField(
        ## peso netto in base ai kg
        blank = True, null = True,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Net Weight in Kg"),
    )
    description_of_goods = models.CharField(
        ## aspetto esteriore dei beni
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("Descriptions of Goods"),
    )
    number_of_packages = models.IntegerField(
        ## numero dei colli
        blank = True, null = True,
        verbose_name = _("Number of Packages")
    )
    date_delivery = models.DateField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Date Delivery"),
    )

    def save(self, force_insert=False, force_update=False):
        if not self.number:
            from store.utils import create_update_storedocumentnumber
            self.number = create_update_storedocumentnumber(
                'd', curr_date=self.date_d, store_document=self.store_cause.store_document
            )
        super(Ddt, self).save(force_insert=force_insert, force_update=force_update)

    def get_totals_vat(self):
        ## vat, total_income, total_vat, total_fee, total_discount, total (per ogni aliquota iva)
        vats = Vat.objects.all() #filter(status=1)
        rows = self.ddtrows.all()
        totals = {}

        ## azzeramento totali
        for vat in vats:
            totals[vat.id] = {}
            totals[vat.id]['income'] = 0
            totals[vat.id]['fee'] = 0
            totals[vat.id]['discount'] = 0
            totals[vat.id]['vat'] = 0
            totals[vat.id]['total'] = 0

        for row in rows:
            totals[row.vat.id]['income']   +=  row.total_income
            totals[row.vat.id]['fee']      +=  row.total_fee
            totals[row.vat.id]['discount'] +=  row.total_discount
            totals[row.vat.id]['vat']      = (
                (totals[row.vat.id]['income'] + totals[row.vat.id]['fee'] - totals[row.vat.id]['discount']) *
                (row.vat.aliquot_perc / 100)
            )
            totals[row.vat.id]['total']    = (
                totals[row.vat.id]['income'] + totals[row.vat.id]['fee'] - totals[row.vat.id]['discount'] + totals[row.vat.id]['vat']
            )
        return totals
    totals_vat = property(get_totals_vat)


    def get_total_quantity(self):
        quantity = decimal.Decimal(0)
        for row in self.ddtrows.all():
            quantity += row.quantity
        return quantity
    total_quantity = property(get_total_quantity)


    def get_totals(self):
        vats = Vat.objects.filter(status=1)

        totals = {}
        totals['income'] = 0
        totals['fee'] = 0
        totals['discount'] = 0
        totals['vat'] = 0
        totals['total'] = 0
        totals_vat = self.totals_vat

        for vat in vats:
            totals['income'] += totals_vat[vat.id][ 'income']
            totals['fee'] += totals_vat[vat.id]['fee']
            totals['discount'] += totals_vat[vat.id]['discount']
            totals['vat'] += totals_vat[vat.id]['vat']
            totals['total'] += totals_vat[vat.id]['total']
        return totals
    totals = property(get_totals)


    def default_invoice_address(self):
        invoice_addresses = self.owner.get_addresses().filter(address_scopes__is_invoice_address=True)
        if invoice_addresses:
            return invoice_addresses[0]
        return None

    def default_invoice_address1(self):
        addr = self.default_invoice_address()
        return "%s %s, %s" % (addr.get_address_type_display() or "", addr.name, addr.number)

    def default_invoice_address2(self):
        addr = self.default_invoice_address()
        return "%s %s (%s)" % (addr.zip_code, addr.locality, addr.province)

    def default_invoice_address3(self):
        addr = self.default_invoice_address()
        return "%s" % addr.country

    def shipping_address1(self):
        if self.shipping_address:
            atype =  self.shipping_address.get_address_type_display() or ""
            name =  self.shipping_address.name or ""
            number = self.shipping_address.number
            return "%s %s, %s" % (atype, name, number)
        return ""

    def shipping_address2(self):
        if self.shipping_address:
            zip_code = self.shipping_address.zip_code or ""
            locality = self.shipping_address.locality or ""
            province = self.shipping_address.province or ""
            return "%s %s (%s)" % (zip_code, locality, province)
        return ""

    def shipping_address3(self):
        if self.shipping_address:
            country = self.shipping_address.country
            return "%s" % country
        return ""

    if "invoice" in settings.INSTALLED_APPS:
        def is_allinvoiced(self):
            """
            restituisce True se il DDT è stato fatturato o se tutte le Righe DDT sono fatturate
            verificare causale magazzino
            """
            if self.invoice:
                return True
            ### verifica se tutte le righe sono fatturate
            rows = self.ddtrows.filter(invoice=False, store_cause__ddt=True, store_cause__invoice=True)
            if len(rows) != 0:
                return False
            return True

    if "invoice" in settings.INSTALLED_APPS:
        def set_allinvoiced(self):
            """
            Imposta a True se tutte le Righe DDT sono state fatturate
            """
            if self.is_allinvoiced():
                self.invoice = True
                self.save()

    if "invoice" in settings.INSTALLED_APPS:
        @transaction.commit_manually
        def create_or_append_to_invoice(self, invoice = None, date_i = date.today()):
            """
            crea o aggiunge ad una Fattura le Righe Ddt
            """
            ## se il Ddt è già stato fatturato esco
            if self.is_allinvoiced():
                transaction.rollback()
                return
            ddtrows = self.ddtrows.filter(invoice=False, store_cause__ddt=True, store_cause__invoice=True)
            if not invoice:
                from invoice.models import Invoice
                invoice = Invoice(
                    owner=self.owner, recipient=self.recipient,
                    payment_form=self.payment_form,
                    office=self.office, contact=self.contact,
                    store_cause=self.store_cause, date_i=date_i,
                )
                invoice.save()

            for row in ddtrows:
                row.add_to_invoice(invoice)

            if invoice.invoicerows.count() == 0:
                transaction.rollback()
                return False
            self.set_allinvoiced()
            transaction.commit()
            return True

    class Meta:
        verbose_name = _("DDT")
        verbose_name_plural = _("DDTs")
        permissions = (
            ('add_customer_ddt', 'Can ADD Customer DDT'),
            ('view_customer_ddt', 'Can VIEW Customer DDT'),
            ('add_supplier_ddt', 'Can ADD Supplier DDT'),
            ('view_supplier_ddt', 'Can VIEW Supplier DDT'),
        )

    def __unicode__(self):
        return u"%(date)s, %(number)s" % {'date': self.date_d, 'number': self.number}

##################################
# RIGA DDT                       #
##################################
UNIT_TYPE = (
    ("pz", _("Pezzi")),
    ("ml", _("Metro lineare")),
    ("mq", _("Metro quadro")),
    ("mc", _("Metro cubo")),
    ("kg", _("Chili")),
    ("t", _("Tonnellate")),
    ("l", _("Litri")),
    ("cp", _("a corpo")),
    ("hh", _("ore")),
    ("gg", _("giorni")),
)

class DdtRow (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    ddt = models.ForeignKey (
        Ddt,
        related_name = "ddtrows",
        db_index = True,
        verbose_name = _("DDT"),
    )
    ### da aggiungere anche anno
    procedurerow = models.ForeignKey(
        ProcedureRow,
        blank = True, null = True,
        related_name = "ddtrows",
        verbose_name = _("Procedure Row"),
    )
    product = models.ForeignKey (
        Product,
        related_name = "ddtrows",
        db_index = True,
        verbose_name = _("Product"),
    )
    vat = models.ForeignKey (
        Vat,
        related_name = "ddtrows",
        db_index = True,
        verbose_name = _("Vat"),
    )
    store_cause = models.ForeignKey (
        StoreCause,
        related_name = "ddtrows",
        db_index = True,
        verbose_name = _("Store Cause"),
    )
    ## Campi
    quantity = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Quantity"),
    )
    m_unit = models.CharField (
        max_length = 2,
        choices = UNIT_TYPE,
        default = "pz",
        db_index = True,
        verbose_name = _("Unit"),
    )
    price = models.DecimalField (
        max_digits = 9,
        decimal_places = 4,
        default = 0,
        verbose_name = _("Price"),
    )
    fee = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Fee"),
    )
    discount = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Discount"),
    )
    invoice = models.BooleanField (
        db_index = True,
        verbose_name = _("Invoice"),
    )
    description = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Description")
    )

    def delete(self, *args, **kwargs):
        if self.procedurerow:
            self.procedurerow.ddt = False
            self.procedurerow.save()
            self.procedurerow.procedure.ddt = False
            self.procedurerow.procedure.save()

        super(DdtRow, self).delete(*args, **kwargs)


    def save(self, check_quantity=True, *args, **kwargs):
        if check_quantity and self.procedurerow and self.quantity > self.procedurerow.quantity:
            raise Exception("DDT Quantity greater than Procedure Row Quantity.")

        super(DdtRow, self).save(*args, **kwargs)

    def get_description(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or "%s %s" % (self.product.code, self.product.name)

    def get_description_wo_code(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or self.product.name

    def get_total_income(self):
        tot_income = self.quantity * self.price
        return tot_income.quantize(decimal.Decimal('0.01'))
    total_income = property(get_total_income)

    def get_total_fee(self):
        tot_fee = self.quantity * self.fee
        return tot_fee.quantize(decimal.Decimal('0.01'))
    total_fee = property(get_total_fee)

    def get_total_discount(self):
        tot_discount = self.quantity * self.discount
        return tot_discount.quantize(decimal.Decimal('0.01'))
    total_discount = property(get_total_discount)

    def get_total(self):
        totale = self.total_income + self.total_fee - self.total_discount
        return totale.quantize(decimal.Decimal('0.01'))
    total = property(get_total)

    if "invoice" in settings.INSTALLED_APPS:
        def get_invoicedquantity(self):
            """
            restituisce la quantita totale fatturata
            """
            invoicerows = self.invoicerows.all()
            quantity = 0
            for row in invoicerows:
                quantity += row.quantity
            return quantity

    if "invoice" in settings.INSTALLED_APPS:
        def get_notinvoicedquantity(self):
            """
            restituisce la quantita ancora da fatturare
            """
            return self.quantity - self.get_invoicedquantity()
        get_notinvoicedquantity.short_description = _("NI Quantity")


    if "invoice" in settings.INSTALLED_APPS:
        def check_header_integrity(self, objmodel):
            if (
                objmodel.owner == self.ddt.owner
                and objmodel.recipient == self.ddt.recipient
                and objmodel.payment_form == self.ddt.payment_form
            ):
                return True
            return False

    if "invoice" in settings.INSTALLED_APPS:
        @transaction.commit_manually
        def create_invoice(self, date_i = None, quantity=None, check_quantity=True):
            """
            crea una Fattura associando Riga DDT
            """
            if not date_i:
                date_i = date.today()

            ## se la 'Riga DDT' è già stata fatturata interamente esco
            if self.ddt.is_allinvoiced():
                transaction.rollback()
                return (False, "DDT already invoiced")

            ## creo fattura nuova
            from invoice.models import Invoice
            invoice = Invoice(
                owner=self.ddt.owner, recipient=self.ddt.recipient,
                bank=self.ddt.bank, payment_form=self.ddt.payment_form,
                office=self.ddt.office, contact=self.ddt.contact,
                store_cause=self.ddt.store_cause, date_i=date_i,
            )
            try:
                invoice.save()
            except Exception, e:
                return (False, e)

            ## appendo nuove righe fattura
            res, error = self.add_to_invoice(invoice, quantity, check_quantity=check_quantity)

            if res:
                transaction.commit()
                return (invoice, "")
            else:
                transaction.rollback()
                return (False, error)

    if "invoice" in settings.INSTALLED_APPS:
        def add_to_invoice(self, invoice, quantity=None, check_quantity=True):
            """
            aggiunge la "Riga DDT" ad una Fattura.
            Solo la quantita' passata viene fatturata.
            Se la quantita' non e' specificata l'intera quantita' da fatturare viene inserita nella Fattura
            """
            notinvoicedquantity = self.get_notinvoicedquantity()

            ## verifica che la 'Riga DDT' non sia già stata fatturata
            ## se già fatturata non è possibile consegnare ed esce
            if self.invoice == True:
                return (False, "DDT Row Already Invoiced")

            ## verifica se la quantita della 'Riga DDT' da inserire nella 'Riga Fattura' sia POSITIVA
            ## se <= 0 esce, non ci sono prodotti da fatturare
            if notinvoicedquantity <= 0:
                return (False, "No product to invoice")

            ## se non viene specificata una 'quantita' viene inserita in 'Riga Fattura' la quantita ancora da fatturare
            if not quantity:
                quantity = notinvoicedquantity
            else:
                quantity = decimal.Decimal(quantity)

            ## se viene passata una 'quantita' viene controllata che non sia maggiore di quella da fatturare
            ## se > allora esce
            if quantity > notinvoicedquantity:
                return (False, "Invoice Row quantity greater then DDT Row quantity")

            if not self.check_header_integrity(invoice):
            ## verifica che i dati delle intestazioni dell'"Ordine" e del "DDT" corrispondono
            ## se non c'è corrispondenza esce
                return (False, "Invoice Header and DDT Header are different.")

            from invoice.models import InvoiceRow
            invoicerow = InvoiceRow(
                invoice=invoice, ddtrow=self, product=self.product, vat=self.vat, store_cause=self.store_cause,
                quantity=quantity, price=self.price, fee=self.fee, discount=self.discount,
            )
            try:
                invoicerow.save()
            except Exception, e:
                return (False, e)
            return (True, "")

    class Meta:
        verbose_name = _("DDT Row")
        verbose_name_plural = _("DDT Rows")
        permissions = (
            ('add_customer_ddtrow', 'Can ADD Customer DDT Row'),
            ('view_customer_ddtrow', 'Can VIEW Customer DDT Row'),
            ('add_supplier_ddtrow', 'Can ADD Supplier DDT Row'),
            ('view_supplier_ddtrow', 'Can VIEW Supplier DDT Row'),
        )
        ordering = ['-ddt__date_d', '-ddt__number', 'ordering', 'id']

    def __unicode__(self):
        return u"%(ddt)s, %(product)s" % {'ddt': self.ddt, 'product': self.product}

class Settings(JSettings):
    pass

from ddt import signals
