# -*- coding: utf-8 -*-
import datetime

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.conf.urls.defaults import *
from django import forms
from django.conf import settings
from django.db.models import Q
from django.contrib import messages

from jumbo.utils.functions import get_settings
from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete import AutocompleteModelChoiceField
from jumbo.apps.project.models import Project
from jumbo.forms.models import AdminModelForm
from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin, SettingsAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.forms import DateWidget
from jumbo.views.generic.create_oo_file import create_file
from jumbo.utils.functions import get_settings

from invoice.models import Invoice, Settings
from invoice.models import InvoiceRow, VatRow
from organization.models import Organization, Bank, Vat
from invoice import views as invoice_views

### FORM
from product.models import Product
from store.models import StoreCause

class InvoiceAdminForm(AdminModelForm):
    class Meta:
        model = Invoice
    class Media:
        js = ('invoice/js/invoice_form.js',)

    def __init__(self, *args, **kws):
        super(InvoiceAdminForm, self).__init__(*args, **kws)
        self.fields['owner'] = AutocompleteModelChoiceField(
            'invoice_owner', label=_("Owner"),
            on_select_token='on_select_owner',
            on_delete_token='on_delete_owner',
        )
        self.fields['recipient'] = AutocompleteModelChoiceField(
            'invoice_recipient', label=_("Recipient")
        )
        self.fields['own_bank'].queryset = Bank.objects.filter(
            organization__organization_type__category='com',
            organization__organization_type__status=1,
            organization__status=1,
            status=1
        )

class InvoiceRowAdminForm(AdminModelForm):
    class Meta:
        model = InvoiceRow
    class Media:
        js = ('invoice/js/invoicerow_form.js',)

    def __init__(self, *args, **kws):
        super(InvoiceRowAdminForm, self).__init__(*args, **kws)

        self.fields['product'] = AutocompleteModelChoiceField(
            'invoice_product', label=_("Product"),
            on_select_token='on_select_product',
            on_delete_token='on_delete_product',
        )

        invoice_id = self.get_object_id('invoice')
        invoice = Invoice.objects.get(pk=invoice_id)
        product_id = self.get_object_id('product')
        projects = Project.objects.filter(status=1)
        vats = Vat.objects.none()
        store_causes = StoreCause.objects.none()

        self.fields['invoice'].queryset = Invoice.objects.filter(pk=invoice_id)
        if invoice.store_cause.category == "cus":
            projects = projects.filter(organization__in=(invoice.owner, invoice.recipient))
        if self.fields.get('project'):
            self.fields['project'].queryset = projects
        if not product_id:
            self.fields['vat'].queryset = vats
            self.fields['store_cause'].queryset = store_causes
            self.fields['m_unit'].choices = []

class VatRowAdminForm(AdminModelForm):
    class Meta:
        model = VatRow

### INLINE
class InvoiceRowInline(admin.TabularInline):
    model = InvoiceRow
    form = InvoiceRowAdminForm
    extra = 1
    fields = ('product', 'project', 'vat', 'store_cause', 'quantity', 'price', 'fee', 'discount', 'description')
    #raw_id_fields = ['product']

### SEARCH
class InvoiceSearchForm(forms.ModelForm):
    class Meta:
        model = Invoice
        fields = ('owner__name__istartswith', 'owner__code', 'recipient__code', 'recipient__name__istartswith', 'payment_form', 'store_cause')
        display_fields =    (('owner__name__istartswith', 'owner__code'),
                             ('recipient__name__istartswith', 'recipient__code'),
                             ('payment_form', 'store_cause', 'store_cause__category'),
                             ('number__gte', 'number__lte'),
                             ('date_i__year', 'search_in_all_years'),
                             ('date_i__gte', 'date_i__lte'))

    def __init__(self, *args, **kws):
        super(InvoiceSearchForm, self).__init__(*args, **kws)
        ORGANIZATIONTYPE_DEF = (("", _("-----"),), ("cus", _("customer"),), ("sup", _("supplier"),))

        self.fields['owner__name__istartswith'] = forms.CharField(label=_("Owner"), required=False)
        self.fields['owner__code'] = forms.CharField(label=_("Owner Code"), required=False)
        self.fields['recipient__name__istartswith'] = forms.CharField(label=_("Recipient"), required=False)
        self.fields['recipient__code'] = forms.CharField(label=_("Recipient Code"), required=False)
        self.fields['date_i__gte'] = forms.DateField(label=_("Date Begin"), widget= DateWidget, required=False)
        self.fields['date_i__lte'] = forms.DateField(label=_("Date End"), widget= DateWidget, required=False)
        self.fields['number__gte'] = forms.IntegerField(label=_("From Number"), required=False)
        self.fields['number__lte'] = forms.IntegerField(label=_("To Number"), required=False)
        self.fields['store_cause__category'] = forms.ChoiceField(label=_("Category"), choices=ORGANIZATIONTYPE_DEF, required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['date_i__year'] = forms.CharField(label=_("Year"), required=False)

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('date_i__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(date_i__year = datetime.date.today().year)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs

class InvoiceRowSearchForm(forms.ModelForm):
    class Meta:
        model = InvoiceRow
        fields = ('description__icontains', 'store_cause', 'vat', 'project')
        display_fields =    ('description__icontains',
                            ('store_cause', 'store_cause__category'),
                            ('invoice__date_i__year','search_in_all_years'),
                             'vat',
                             'project')

    def __init__(self, *args, **kws):
        super(InvoiceRowSearchForm, self).__init__(*args, **kws)
        ORGANIZATIONTYPE_DEF = (("", _("-----"),), ("cus", _("customer"),), ("sup", _("supplier"),))

        self.fields['description__icontains'] = forms.CharField(label=_("Description"), required=False)
        self.fields['store_cause__category'] = forms.ChoiceField(label=_("Category"), choices=ORGANIZATIONTYPE_DEF, required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['invoice__date_i__year'] = forms.CharField(label=_("Year"), required=False)
        self.fields['project'] = forms.ModelChoiceField(label=_("Project"), required=False, queryset=Project.objects.all())

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('invoice__date_i__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(invoice__date_i__year = datetime.date.today().year)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs

########################################
# INVOICE ROW                          #
########################################
class InvoiceRowAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("invoice/css/admin_invoice.css",)}

    use_fancybox = True
    father_foreignkey_fancybox = 'invoice'

    def get_date_i(self, obj):
        return obj.invoice.date_i.strftime("%d/%m/%Y")
    get_date_i.short_description = _('Date')
    get_date_i.admin_order_field = 'invoice__date_i'

    def get_number(self, obj):
        return obj.invoice.number or ""
    get_number.short_description = _('Number')
    get_number.admin_order_field = 'invoice__number'

    def get_detail(self, obj):
        owner = obj.invoice.owner
        recipient = obj.invoice.recipient or ""
        contact = obj.invoice.contact or ""
        if recipient == owner:
            recipient = ""
        return "%s<br/>%s<br/>%s" % (owner, recipient, contact)
    get_detail.short_description = _("Bill to")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'invoice__owner'

    def get_product_detail(self, obj):
        product = obj.get_description()
        project = obj.project or ""
        return """<b><a href="%s/">%s</a></b><br/>%s""" % (obj.id, product, project)
    get_product_detail.short_description = _("Product")
    get_product_detail.allow_tags = True
    get_product_detail.admin_order_field = 'description'

    def get_cause_code(sel, obj):
        return obj.store_cause.code
    get_cause_code.short_description = _('SC')

    def get_vat_code(self, obj):
        return obj.vat.code
    get_vat_code.short_description = _('VC')

    def str_total_income(self, obj):
        return str("%.2f" % obj.get_total_income())
    str_total_income.short_description = _('Total Income')

    def str_total_fee(self, obj):
        return str("%.2f" % obj.get_total_fee())
    str_total_fee.short_description = _('Total Fee')

    def str_total_discount(self, obj):
        return str("%.2f" % obj.get_total_discount())
    str_total_discount.short_description = _('Total Discount')

    def str_total(self, obj):
        return str("%.2f" % obj.get_total())
    str_total.short_description = _('Total')

    form = InvoiceRowAdminForm
    search_form = InvoiceRowSearchForm
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {'fields': (
            'creator', 'date_create','last_modifier', 'date_last_modify',
        ), 'classes': ('collapse',)}),
        (None, {'fields': (
            'status',
            ('invoice', 'project',),
            'product',
            ('store_cause', 'vat',),
            ('quantity', 'price',),
            'm_unit',
            ('fee', 'discount',),
            'description',
            'long_description'
        )}),
    )
    search_fields = ('invoice', 'product', 'store_cause')
    default_group_by = 'invoice'
    list_display = [
        'get_date_i', 'get_number', 'get_detail', 'get_product_detail', 'description', 'long_description', 'quantity', 'price', 'fee', 'discount', 'str_total_income', 'str_total_fee', 'str_total_discount', 'str_total', 'commision', 'get_cause_code', 'get_vat_code'
    ]
    list_display_csv = [
        'get_date_i', 'get_number', 'get_detail', 'get_product_detail', 'description', 'long_description', 'quantity', 'price', 'fee', 'discount', 'str_total_income', 'str_total_fee', 'str_total_discount', 'str_total', 'commision', 'get_cause_code', 'get_vat_code'
    ]

########################################
# INVOICE                              #
########################################
class InvoiceAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("invoice/css/admin_invoice.css",)}

    form = InvoiceAdminForm
    search_form = InvoiceSearchForm
    list_per_page = 30
    date_hierarchy = 'date_i'
    fieldsets = (
        (_('logs admin'), {'fields': (
            'creator', 'date_create','last_modifier', 'date_last_modify',
        ), 'classes': ('collapse',)}),
        (None, {'fields': (
            'date_i',
            ('number', 'owner_number'),
            ('owner', 'recipient'),
            'contact',
            'own_bank',
            ('bank', 'payment_form'),
            'store_cause',
            'invoice_address',
            'paid',
        )}),
        (_('note'), {'fields': ('note',), 'classes': ('collapse',)}),
    )
    search_fields = ['owner__code', 'owner__name', 'recipient__code', 'recipient__name', 'store_cause__category', 'number', 'owner_number']
    list_display = (
        'get_date_i', 'number', 'get_detail', 'get_payment_form', 'get_store_cause',
        'get_total_income', 'paid', 'create_odt', 'create_pdf',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv = (
        'get_date_i', 'number', 'owner', 'contact', 'payment_form',
        'store_cause', 'get_income', 'get_fee', 'get_discount', 'get_vat',
        'get_total',
    )
    list_display_links = ['number']
    default_group_by = 'owner'
    save_as = True
    save_on_top = True

    def invoice_create_file(self, request, obj_id, file_type='odt'):
        obj = Invoice.objects.get(id=obj_id)
        model = Invoice
        file_model = 'admin/invoice/invoice/invoice.odt'
        if obj.store_cause.model_template:
            file_model = 'admin/invoice/invoice/%s.odt' % obj.store_cause.model_template
        return create_file(request, model=model, file_model=file_model, file_type=file_type, obj_id=obj_id)


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(InvoiceAdmin, self).get_urls()
        my_urls = patterns('',
            url(
                r'^send_mail/(?P<invoice_id>\d+)/$',
                self.admin_site.admin_view(invoice_views.invoice_send_mail),
                name = 'invoice_invoice_sendmail'
            ),
            url(
                r'^create_odt/(?P<obj_id>\d+)/$',
                self.admin_site.admin_view(self.invoice_create_file),
                {'file_type': 'odt',},
                name='invoice_invoice_create_odt'
            ),
            url(
                r'^create_pdf/(?P<obj_id>\d+)/$',
                self.admin_site.admin_view(self.invoice_create_file),
                {'file_type': 'pdf',},
                name='invoice_invoice_create_pdf'
            ),
        )
        return my_urls + urls

    def get_date_i(self, obj):
        return obj.date_i.strftime("%d/%m/%Y")
    get_date_i.short_description = _('Date')
    get_date_i.admin_order_field = 'date_i'

    def get_detail(self, obj):
        owner = """<a href=%(reverse_url)s>%(owner)s</a>""" % {
            #'reverse_url': reverse('organization-detail', obj.owner.id),
            'reverse_url':  '',
            'owner':       obj.owner.name
        }
        contact = obj.contact or ""
        return "<b>%s</b><br/>%s" % (owner, contact)
    get_detail.short_description = _("Bill to")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'owner'

    def get_payment_form(self, obj):
        return """<span title="%s">%s</span>""" % (obj.payment_form.name, obj.payment_form.code)
    get_payment_form.short_description = _("Payment Form")
    get_payment_form.allow_tags = True
    get_payment_form.admin_order_field = 'payment_form__code'

    def get_store_cause(self, obj):
        return """<span title="%s">%s</span>""" % (obj.store_cause.name, obj.store_cause.code)
    get_store_cause.short_description = _("Store Cause")
    get_store_cause.allow_tags = True
    get_store_cause.admin_order_field = 'store_cause__code'

    def get_total_income(self, obj):
        return obj.totals['income'] + obj.totals['fee'] - obj.totals['discount']
    get_total_income.short_description = _('Total Income')

    def get_income(self, obj):
        return obj.totals['income']
    get_income.short_description = _('Total Income')

    def get_fee(self, obj):
        return obj.totals['fee']
    get_fee.short_description = _('Total Fee')

    def get_discount(self, obj):
        return obj.totals['discount']
    get_discount.short_description = _('Total Discount')

    def get_vat(self, obj):
        return obj.totals['vat']
    get_vat.short_description = _('Total VAT')

    def get_total(self, obj):
        return obj.totals['total']
    get_total.short_description = _('Total')

    def send_mail(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/mail.png' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:invoice_invoice_sendmail', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Send Mail")
        }
    send_mail.short_description = _("Mail")
    send_mail.allow_tags = True

    def create_odt(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/writer.gif' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:invoice_invoice_create_odt', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Create Invoice ODT"),
        }
    create_odt.short_description = _("ODT")
    create_odt.allow_tags = True


    def create_pdf(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/pdf_icon.png' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:invoice_invoice_create_pdf', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Create Invoice PDF")
        }
    create_pdf.short_description = _("PDF")
    create_pdf.allow_tags = True

    def create_commision(self, obj):
        return "<a href=%(reverse_url)s>x</a>" % {
            #'reverse_url':reverse('invoice-invoice-create_commision', obj.id),
            'reverse_url':'',
            'window_title':_("Create Invoice")
        }
    create_commision.short_description = _("CC")
    create_commision.allow_tags = True

    actions = ['set_as_paid', 'send_by_email']
    def get_actions(self, request):
        actions = super(InvoiceAdmin, self).get_actions(request)
        try:
            del actions['set_active']
            del actions['set_disactive']
            del actions['set_as_completed']
            del actions['set_as_to_complete']
        except:
            pass
        return actions

    def send_by_email(self, request, queryset):
        sended = 0
        limit = get_settings("invoice.settings", "MULTIPLE_INVOICES_MAIL_LIMIT")
        if get_settings("invoice.settings", "SEND_INVOICE_BY_MAIL") == False:
            messages.error(request, _("You should set true 'SEND_INVOICE_BY_MAIL' in your settings"))
            return

        if queryset.count() > limit:
            self.message_user(request, _("You can send %(msg)s max mail at the same time!") % {'msg': limit})
            return

        for invoice in queryset:
            if invoice.send_by_mail():
                sended += 1

        self.message_user(request, _("%(msg)s emails sent.") % {'msg': sended})
    send_by_email.short_description = _("Send invoice by email")

    def set_as_paid(self, request, queryset):
        """
        segna la fattura come pagata e non crea le righe di addebito (commision)
        """
        rows_updated = 0
        for invoice in queryset:
            invoice.paid = True
            invoice.save()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Invoice was"
        else:
            message_bit = "%s Invoices were" % rows_updated
        self.message_user(request, "%s successfully accepted." % message_bit)
    set_as_paid.short_description = _("Set Invoices as paid")

    #def get_invoice_rows(self, obj):
    #    n_rows = obj.invoicerows.count()
    #    link = ""
    #    if n_rows > 0:
    #        link = """<a href="%(reverse_url)s" class="link_invoicerows" rel="#overlay">%(num_invoice)s</a>""" % {
    #            #'reverse_url': reverse('invoice-invoice-invoicerows', obj.id),
    #            'reverse_url': '',
    #            'num_invoice': n_rows,
    #        }
    #
    #    return link
    #get_invoice_rows.short_description = _('PR')
    #get_invoice_rows.allow_tags = True

admin.site.register(Settings, SettingsAdmin)
admin.site.register(InvoiceRow, InvoiceRowAdmin)
admin.site.register(Invoice, InvoiceAdmin)

########################################
# AUTOCOMPLETE REGISTER                #
########################################
def get_invoice_product(request, query):
    return Product.objects.filter(status=1).order_by('code')

def get_autocomplete_invoice_owner(request, query):
    results = []
    try:
        org_exact = Organization.objects.get(code=query)
        results.append([org_exact.id, "%s - %s" % (org_exact.code, org_exact.name)])
    except:
        pass

    q1 = Q(name__istartswith=query)
    q2 = Q(code__istartswith=query)
    for other_org in Organization.objects.exclude(code=query).filter(status=1).filter(q1|q2)[:10]:
        results.append([other_org.id, "%s - %s" % (other_org.code, other_org.name)])
    return results

def label_custom_code(obj):
    return "%s - %s" % (obj.code, obj.name)

autocomplete.register('invoice_owner'    , model = Organization, views = get_autocomplete_invoice_owner)
autocomplete.register('invoice_recipient', model = Organization, views = get_autocomplete_invoice_owner)
autocomplete.register(
    'invoice_product', model=Product, views=get_invoice_product,
    search_fields=('name', 'code__icontains'), limit=10
)
