# -*- coding: utf-8 -*-
import datetime

from django.utils.translation import ugettext_lazy as _

from admin_tools.dashboard import modules
from invoice.models import Invoice
from invoice.statistics import total_invoice_per_year, total_invoice_per_month
from invoice.statistics import total_invoice_per_year_per_organization, total_invoice_per_month_per_organization

CURRENT_YEAR = datetime.date.today().year
LAST_YEARS = [CURRENT_YEAR, CURRENT_YEAR-1, CURRENT_YEAR-2]
LAST_MONTH = datetime.date.today().month

class InvoiceDashboard(modules.Group):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': "Invoices",
            'display': "tabs",
            'children': [
                InvoiceModels(),
                InvoiceSettings(),
            ]
        }
        super(InvoiceDashboard, self).__init__(**kwargs)

class InvoiceModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Models"),
            'models': ('invoice.models.Invoice', 'invoice.models.InvoiceRow'),
            }
        super(InvoiceModels, self).__init__(**kwargs)

class InvoiceSettings(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Settings"),
            'models':['invoice.models.Settings',],
            }
        super(InvoiceSettings, self).__init__(**kwargs)

class CustomerInvoices(modules.Group):
    def __init__(self, **kwargs):
        """
        customer invoice dashboard:
        - last invoices
        """
        kwargs =  {
            'title': _("Customer Invoices"),
            'children': [
                Invoices(title=_('Top'), order_by='-total_income', extra_filters={'store_cause__category': 'cus'}),
                Invoices(title=_('Last'), order_by='-date_i', extra_filters={'store_cause__category': 'cus'}),
                Invoices(title=_('To Pay'), order_by='-date_i', limit=None, extra_filters={'store_cause__category': 'cus', 'paid': False}),
            ],
        }
        super(CustomerInvoices, self).__init__(**kwargs)

class CustomerYearAggregateInvoices(modules.Group):
    def __init__(self, **kwargs):
        """
        customer invoice aggregate dashboard:
        - invoiced per year
        - invoiced to pay per year
        - invoiced paid per year
        """
        kwargs =  {
            'title': _("Customer Year Aggregate Invoices"),
            'children': [
                Invoices(
                    title=_('Invoiced'),
                    children=total_invoice_per_year(),
                    template='admin_tools/dashboard/year_aggregate_invoices.html'
                ),
                Invoices(
                    title=_('Invoiced to pay'),
                    children=total_invoice_per_year(others_where="AND paid=False"),
                    template='admin_tools/dashboard/year_aggregate_invoices.html'
                ),
                Invoices(
                    title=_('Invoiced paid'),
                    children=total_invoice_per_year(others_where="AND paid=True"),
                    template='admin_tools/dashboard/year_aggregate_invoices.html'
                ),
            ],
        }
        super(CustomerYearAggregateInvoices, self).__init__(**kwargs)

class CustomerMonthAggregateInvoices(modules.Group):
    def __init__(self, **kwargs):
        """
        customer invoice aggregate dashboard:
        - invoiced per month
        - invoiced to pay per month
        - invoiced paid per month
        """
        kwargs =  {
            'title': _("Customer Month Aggregate Invoices"),
            'children': [
                Invoices(
                    title=_('Invoiced'),
                    children=total_invoice_per_month(),
                    template='admin_tools/dashboard/month_aggregate_invoices.html'
                ),
                Invoices(
                    title=_('Invoiced to pay'),
                    children=total_invoice_per_month(others_where="AND paid=False"),
                    template='admin_tools/dashboard/month_aggregate_invoices.html'
                ),
                Invoices(
                    title=_('Invoiced paid'),
                    children=total_invoice_per_month(others_where="AND paid=True"),
                    template='admin_tools/dashboard/month_aggregate_invoices.html'
                ),
            ],
        }
        super(CustomerMonthAggregateInvoices, self).__init__(**kwargs)

class CustomerYearTopTenInvoices(modules.Group):
    def __init__(self, **kwargs):
        top_list = []
        for year in LAST_YEARS:
            top_list.append(Invoices(
                title=_('Invoiced %s') % year,
                children=total_invoice_per_year_per_organization(years=year, limit=10),
                template='admin_tools/dashboard/topten_invoices.html'
            ))
        kwargs =  {
            'title': _("Customer Top Ten Per Year"),
            'children': top_list,
        }
        super(CustomerYearTopTenInvoices, self).__init__(**kwargs)

class CustomerYearMonthTopTenInvoices(modules.Group):
    """
    Return dashboard module group by per year and per month
    """
    def __init__(self, **kwargs):
        top_list = []
        for year in LAST_YEARS:
            top_list.append(CustomerMonthTopTenInvoices(
                title=_('Invoiced %s') % year,
                year=year
            ))
        kwargs =  {
            'title': _("Customer Top Ten Per Month"),
            'children': top_list,
        }
        super(CustomerYearMonthTopTenInvoices, self).__init__(**kwargs)

class CustomerMonthTopTenInvoices(modules.Group):
    """
    Return dashboard module group by per month
    This is use from CustomerYearMonthTopTenInvoices
    """
    def __init__(self, **kwargs):
        year = kwargs.get('year', CURRENT_YEAR)
        top_list = []

        for month in range(1,13):
            top_list.append(Invoices(
                title=_('Invoiced Month %s') % month,
                children=total_invoice_per_month_per_organization(years=year, months=month, limit=10),
                template='admin_tools/dashboard/topten_month_invoices.html'
            ))
        kwargs =  {
            'title': _("Top Ten Per Month"),
            'children': top_list,
        }
        super(CustomerMonthTopTenInvoices, self).__init__(**kwargs)

class Invoices(modules.DashboardModule):
    def __init__(
        self, title, template='admin_tools/dashboard/invoices.html',
        order_by=None, limit=10, qs=Invoice.objects.all(), extra_filters={},
        children=None, **kwargs
    ):
        self.order_by = order_by
        self.template = template
        self.qs = qs
        self.extra_filters = extra_filters
        self.limit = limit
        self.custom_children = children

        kwargs =  {
            'title': title,
            'template': self.template,
            'children': self.custom_children
        }
        super(Invoices, self).__init__(**kwargs)

    def init_with_context(self, context):
        request = context['request']
        if self.custom_children is None:
            qs = self.qs
            if self.extra_filters: qs = qs.filter(**self.extra_filters)
            if self.order_by: qs = qs.order_by(self.order_by)
            if self.limit: qs = qs[:self.limit]
            self.children = qs

