# -*- coding: utf-8 -*-
from datetime import date, timedelta
import calendar
import decimal

from django.conf import settings
from django.db import models, transaction
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site, RequestSite
from django.core.mail import EmailMultiAlternatives

from jumbo.shortcuts import render_to_string
from jumbo.utils.functions import get_settings
from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.db.models import Settings as JSettings

from address.models import Address
from organization.models import Organization, Office, Contact, Bank, OrganizationAddress
from payment.models import PaymentForm
from store.models import StoreCause
from product.models import Product
from organization.models import Vat
from procedure.models import ProcedureRow
from ddt.models import DdtRow
from jumbo.apps.project.models import Project

###############################
# FATTURA                     #
###############################
class Invoice (DateModel, UserModel, StatusModel):
    ## ForeignKey
    owner = models.ForeignKey (
        Organization,
        related_name = "owner_invoices",
        db_index = True,
        verbose_name = _("Bill to"),
    )
    recipient = models.ForeignKey (
        Organization,
        null = True, blank = True,
        related_name = "recipient_invoices",
        db_index = True,
        verbose_name = _("Ship to"),
    )
    payment_form = models.ForeignKey (
        PaymentForm,
        blank = True, null = True,
        related_name = "invoices",
        db_index = True,
        verbose_name = _("Payment Form"),
    )
    office = models.ForeignKey (
        Office,
        blank = True, null = True,
        related_name = "invoices",
        db_index = True,
        verbose_name = _("Office"),
    )
    contact = models.ForeignKey (
        Contact,
        blank = True, null = True,
        related_name = "invoices",
        db_index = True,
        verbose_name = _("Contact"),
    )
    own_bank = models.ForeignKey (
        Bank,
        blank = True, null = True,
        related_name = "own_invoices",
        db_index = True,
        verbose_name = _("Own Bank"),
    )
    bank = models.ForeignKey (
        Bank,
        blank = True, null = True,
        related_name = "invoices",
        db_index = True,
        verbose_name = _("Bank"),
    )
    store_cause = models.ForeignKey (
        StoreCause,
        related_name = "invoices",
        db_index = True,
        verbose_name = _("Store Cause")
    )
    invoice_address = models.ForeignKey (
        OrganizationAddress,
        blank = True, null = True,
        related_name = "invoices",
        db_index = True,
        verbose_name = _("Invoice Address")
    )
    ## Campi
    protocol_number = models.IntegerField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Protocol Number"),
        db_column = "pnumber"
    )
    number = models.CharField (
        blank = True, null = True,
        db_index = True,
        max_length = 50,
        verbose_name = _("Number"),
        db_column = "inumber"
    )
    owner_number = models.CharField (
        max_length = 50,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Ref Owner Number"),
    )
    date_i = models.DateField (
        db_index = True,
        verbose_name = _("Date")
    )
    commision = models.BooleanField (
        db_index = True,
        verbose_name = _("Commision Complete"),
    )
    paid = models.BooleanField (
        ## fattura pagata
        default = False,
        db_index = True,
        verbose_name = _("Paid"),
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    total_income = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Income"),
    )
    total_fee = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Fee"),
    )
    total_discount = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Discount"),
    )
    total_vat = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total VAT"),
    )

    def get_total_no_vat(self):
        return self.total_income + self.total_fee - self.total_discount
    total_no_vat = property(get_total_no_vat)

    def default_invoice_address(self):
        invoice_addresses = self.owner.get_addresses().filter(address_scopes__is_invoice_address=True)
        if invoice_addresses:
            return invoice_addresses[0]
        return None

    def default_invoice_address1(self):
        addr = self.default_invoice_address()
        return "%s %s, %s" % (addr.get_address_type_display() or "", addr.name, addr.number)

    def default_invoice_address2(self):
        addr = self.default_invoice_address()
        return "%s %s (%s)" % (addr.zip_code, addr.locality, addr.province)

    def default_invoice_address3(self):
        addr = self.default_invoice_address()
        return "%s" % addr.country

    def get_own_organization(self):
        try:
            return Organization.objects.filter(organization_type__category="com", status=1)[0]
        except:
            return None
    own_organization = property(get_own_organization)

#    def get_own_bank(self):
#        try:
#            organization = self.get_own_organization()
#            return organization.banks.filter(status=1)[0]
#        except:
#            return None
#    own_bank = property(get_own_bank)


    def get_first_procedure(self):
        invoice_rows = self.invoicerows.all()
        if invoice_rows:
            first_invoice_row = invoice_rows[0]
            procedure_row = first_invoice_row.procedurerow
            if procedure_row:
                first_procedure = procedure_row.procedure
                return first_procedure
        return None

    def send_by_mail(
        self, template_subject="admin/invoice/invoice/invoice_email_subject.txt",
        template_body_html="admin/invoice/invoice/invoice_email.html",
        template_body_txt="admin/invoice/invoice/invoice_email.txt", **kwargs
    ):
        subject = render_to_string(template_subject)
        body_msg = render_to_string(template_body_txt)

        if Site._meta.installed:
            site = Site.objects.get_current()
        else:
            site = RequestSite(request)

        kwargs.update({'site':site})

        html_body_msg = render_to_string(template_body_html, invoice = self, **kwargs)
        from_mail = get_settings('invoice.settings', 'INVOICE_MAIL_FROM')

        emails = set()
        if (
            self.contact and self.contact.email and
            self.contact.send_invoice_by_email and self.contact.status == 1
            and get_settings('invoice.settings', 'SEND_INVOICE_BY_MAIL')
        ):
            msg = EmailMultiAlternatives(
                subject, body_msg, from_mail, [self.contact.email]
            )
            msg.attach_alternative(html_body_msg, "text/html")
            msg.send()
            return True
        return False

    def save(self, force_insert=False, force_update=False):
        if not self.number:
            from store.utils import create_update_storedocumentnumber
            self.number = create_update_storedocumentnumber(
                'i', curr_date=self.date_i, store_document=self.store_cause.store_document
            )

        super(Invoice, self).save(force_insert=force_insert, force_update=force_update)

    def update_totals(self):
        try:
            first_procedurerow=self.invoicerows.all()[0]
            first_procedurerow.update_totals()
        except:
            pass

    def get_totals(self):
        totals = {}
        totals['income'] = self.total_income
        totals['fee'] = self.total_fee
        totals['discount'] = self.total_discount
        totals['vat'] = self.total_vat
        totals['total_no_vat'] = decimal.Decimal(self.total_income) + \
                                 decimal.Decimal(self.total_fee) - \
                                 decimal.Decimal(self.total_discount)
        totals['total'] = totals['total_no_vat'] + decimal.Decimal(self.total_vat)
        return totals
    totals = property(get_totals)

    if "jumbo.apps.commision" in settings.INSTALLED_APPS:
        def is_allcommisioned(self):
            """
            restituisce True se la Fattura è chiusa
            """
            if self.commision:
                return True

    if "jumbo.apps.commision" in settings.INSTALLED_APPS:
        def get_commisionedamount(self):
            """
            restituisce la somma degli estratto conto relativi alla fattura
            """
            commisions = self.commisions.all()
            amount = 0
            for commision in commisions:
                amount += commision.amount
            return amount

    if "jumbo.apps.commision" in settings.INSTALLED_APPS:
        def get_notcommisionedamount(self):
            return self.totals['total'] - self.get_commisionedamount()

    if "jumbo.apps.commision" in settings.INSTALLED_APPS:
        @transaction.commit_manually
        def create_commision(self, operation_date = date.today(), currency_date = date.today(), amount = None):
            import calendar
            from jumbo.apps.commision.models import Commision
            try:
                amount=self.totals['total']
                installments = self.payment_form.installments.all()
                inst_number = len(installments)
                if inst_number != 0:
                    for installment in installments:
                        inst_curr_date = self.date_i + timedelta(days=installment.days)
                        if installment.end_month:
                            last_day_month = calendar.monthrange(inst_curr_date.year, inst_curr_date.month)[1]
                            inst_curr_date = date(inst_curr_date.year, inst_curr_date.month, last_day_month)
                        inst_description = installment.description
                        inst_amount = amount * installment.amount / 100
                        commision = Commision(
                            organization=self.owner, invoice=self, operation_date=operation_date,
                            currency_date=inst_curr_date, description=inst_description, amount=inst_amount
                        )
                        commision.save()
                else:
                    commision = Commision(
                        organization=self.owner, invoice=self, operation_date=operation_date,
                        currency_date=currency_date, description="NULL", amount=amount
                    )
                    commision.save()

                self.commision=True
                self.save()
            except Exception, e:
                transaction.rollback()
            else:
                transaction.commit()

    def get_payment_installments(self):
        if not self.payment_form:
            return []

        if not self.payment_form.installments.all():
            return []

        installments = []
        total_amount = 0
        for inst in self.payment_form.installments.all():
            installment = {}
            installment['form_name'] = inst.payment_form.name
            installment['name'] = inst.name
            installment['full_name'] = inst.full_name
            installment['maturity'] = self.date_i + timedelta(days=inst.days)
            if inst.end_month:
                maturity = installment['maturity']
                range = calendar.monthrange(
                    maturity.year,
                    maturity.month
                )
                installment['maturity'] = (
                    date(maturity.year, maturity.month, range[1]) +
                    timedelta(days=inst.days_end_month)
                )
            ## Arrotondo la divisione e lascio il resto per l'ultima rata
            installment['amount'] = int(
                self.totals['total'] * inst.amount / 100
            )
            ## controllo l'arrotondamento
            total_amount += installment['amount']
            installments.append(installment)
        ## controllo l'arrotondamento
        if total_amount != self.totals['total']:
            installments[-1]['amount'] = installments[-1]['amount'] - total_amount + self.totals['total']
        return installments

    def how_many_installments(self):
        return len(self.get_payment_installments())

    class Meta:
        verbose_name = _("Invoice")
        verbose_name_plural = _("Invoices")
        ordering = ("-date_i", "-number")
        unique_together = ("owner", "number", "date_i")
        permissions = (
            ('add_customer_invoice',          'Can ADD Customer Invoice'),
            ('add_supplier_invoice',          'Can ADD Supplier Invoice'),

            ('change_customer_invoice',       'Can CHANGE Customer Invoice'),
            ('change_supplier_invoice',       'Can CHANGE Supplier Invoice'),

            ('view_invoice',                  'Can VIEW Invoice'),
            ('view_customer_invoice',         'Can VIEW Customer Invoice'),
            ('view_supplier_invoice',         'Can VIEW Supplier Invoice'),

            ('list_all_invoices',             'Can LIST ALL Invoices'),
            ('list_group_invoices',           'Can LIST Group Invoices'),
            ('list_own_invoices',             'Can LIST Own Invoices'),
            ('list_contact_invoices',         'Can LIST Contact Invoices'),
            ('list_organization_invoices',    'Can LIST Organization Invoices'),
            ('list_customer_invoices',        'Can LIST Customer Invoices'),
            ('list_supplier_invoices',        'Can LIST Supplier Invoices'),
            ('list_accepted_invoices',        'Can LIST Accepted Invoices'),
        )

    def __unicode__(self):
        return u"%(date)s, %(number)s" % {'date': self.date_i, 'number': self.number}

##################################
# RIGA FATTURA                   #
##################################
UNIT_TYPE = (
    ("pz", _("Pezzi")),
    ("ml", _("Metro lineare")),
    ("mq", _("Metro quadro")),
    ("mc", _("Metro cubo")),
    ("kg", _("Chili")),
    ("t", _("Tonnellate")),
    ("l", _("Litri")),
    ("cp", _("a corpo")),
    ("hh", _("ore")),
    ("gg", _("giorni")),
)

class InvoiceRow (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    invoice = models.ForeignKey (
        Invoice,
        related_name = "invoicerows",
        db_index = True,
        verbose_name = _("Invoice"),
    )
    procedurerow = models.ForeignKey(
        ProcedureRow,
        blank = True, null = True,
        related_name = "invoicerows",
        verbose_name = _("Procedure Row"),
    )
    ddtrow = models.ForeignKey(
        DdtRow,
        blank = True, null = True,
        related_name = "invoicerows",
        verbose_name = _("DDT Row"),
    )
    product = models.ForeignKey (
        Product,
        related_name = "invoicerows",
        db_index = True,
        verbose_name = _("Product"),
    )
    vat = models.ForeignKey(
        Vat,
        related_name = "invoicerows",
        db_index = True,
        verbose_name = _("Vat"),
    )
    store_cause = models.ForeignKey (
        StoreCause,
        related_name = "invoicerows",
        db_index = True,
        verbose_name = _("Store Cause"),
    )
    project = models.ForeignKey(
        Project,
        null = True, blank = True,
        db_index = True,
        related_name = "invoicerows",
        verbose_name = _("Project"),
    )
    ## Campi
    quantity = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Quantity"),
    )
    m_unit = models.CharField (
        max_length = 2,
        choices = UNIT_TYPE,
        default = "pz",
        db_index = True,
        verbose_name = _("Unit"),
    )
    price = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0.00,
        verbose_name = _("Price"),
    )
    fee = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0.00,
        verbose_name = _("Fee"),
    )
    discount = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0.00,
        verbose_name = _("Discount"),
    )
    commision = models.BooleanField (
        db_index = True,
        verbose_name = _("Commision Complete"),
    )
    description = models.CharField(
        max_length = 200,
        blank = True, null = True,
        verbose_name = _("Description"),
    )
    long_description = models.TextField (
        blank = True, null = True,
        verbose_name = _("Long Description"),
    )

    def save(self, force_insert=False, force_update=False):
        super(InvoiceRow, self).save(force_insert=force_insert, force_update=force_update)
        self.update_totals()

    def delete(self, *args, **kwargs):
        super(InvoiceRow, self).delete(*args, **kwargs)
        self.update_totals()

    def save_not_update(self, force_insert=False, force_update=False):
        super(InvoiceRow, self).save(force_insert=force_insert, force_update=force_update)

    def get_description(self):
        return self.description or self.long_description or self.product.name

    def get_net_price(self):
        return self.price + self.fee - self.discount
    get_net_price.short_description = _('Net Price')
    net_price = property(get_net_price)

    def get_total_income(self):
        income = self.quantity * self.price
        return income.quantize(decimal.Decimal('0.01'))
    get_total_income.short_description = _('Total Income')
    total_income = property(get_total_income)

    def get_total_fee(self):
        fee = self.quantity * self.fee
        return fee.quantize(decimal.Decimal('0.01'))
    get_total_fee.short_description = _('Total Fee')
    total_fee = property(get_total_fee)

    def get_total_discount(self):
        discount = self.quantity * self.discount
        return discount.quantize(decimal.Decimal('0.01'))
    get_total_discount.short_description = _('Total Discount')
    total_discount = property(get_total_discount)

    def get_total(self):
        totale = self.total_income + self.total_fee - self.total_discount
        return totale.quantize(decimal.Decimal('0.01'))
    get_total.short_description = _('Total')
    total = property(get_total)

    def update_totals(self):
        invoice = self.invoice
        invoicerows=self.invoice.invoicerows.all()
        VatRow.objects.filter(invoice=invoice).delete()
        total = {'income': 0, 'fee': 0, 'discount': 0, 'total': 0, 'vat': 0}
        subtotal = {'income': {}, 'fee': {}, 'discount': {}, 'vat': {}, 'total': {}}
        for row in invoicerows:
            subtotal['income'][row.vat]=subtotal['income'].get(row.vat, 0)
            subtotal['income'][row.vat]+=row.total_income

            subtotal['fee'][row.vat]=subtotal['fee'].get(row.vat, 0)
            subtotal['fee'][row.vat]+=row.total_fee

            subtotal['discount'][row.vat]=subtotal['discount'].get(row.vat, 0)
            subtotal['discount'][row.vat]+=row.total_discount

            subtotal['total'][row.vat]=subtotal['total'].get(row.vat, 0)
            subtotal['total'][row.vat]+=row.total

        for vat in subtotal['income'].keys():
            ## calcolo Totali Procedura
            total['income']+=subtotal['income'][vat]
            total['fee']+=subtotal['fee'][vat]
            total['discount']+=subtotal['discount'][vat]
            total['total']+=subtotal['total'][vat]
            ## calcolo Subtotale IVA per ogni aliquota
            subtotal['vat'][vat]=subtotal['total'][vat]*vat.aliquot_perc/100
            total['vat']+=subtotal['vat'][vat]
            ## creo nuovo oggetto VatRow
            vatrow = VatRow()
            vatrow.invoice = invoice
            vatrow.vat = vat
            vatrow.total_income = str(subtotal['income'][vat])
            vatrow.total_fee = str(subtotal['fee'][vat])
            vatrow.total_discount = str(subtotal['discount'][vat])
            vatrow.total_vat = str(subtotal['vat'][vat])
            vatrow.save()

        total['totals']=total['total']+total['vat']

        invoice.total_income = str(total['income'])
        invoice.total_fee = str(total['fee'])
        invoice.total_discount = str(total['discount'])
        invoice.total_vat = str(total['vat'])
        invoice.save()

    class Meta:
        verbose_name = _("Invoice Row")
        verbose_name_plural = _("Invoice Rows")
        ordering = ['-invoice__date_i', '-invoice__number', 'ordering', 'id']
        permissions = (
            ('view_customer_invoicerow',         'Can VIEW Customer Invoice Row'),
            ('view_supplier_invoicerow',         'Can VIEW Supplier Invoice ROw'),)

    def __unicode__(self):
        return u"%(invoice)s, %(product)s" % {'invoice': self.invoice, 'product': self.product}

##################################
# RIGA IVA                       #
##################################
class VatRow (DateModel, UserModel, StatusModel):
    ## ForeignKey
    invoice = models.ForeignKey (
        Invoice,
        related_name = "vattotalinvoicerows",
        db_index = True,
        verbose_name = _("Procedure"),
    )
    vat = models.ForeignKey (
        Vat,
        related_name = "vattotalinvoicerows",
        db_index = True,
        verbose_name = _("Vat"),
    )
    total_income = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Income"),
    )
    total_fee = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Fee"),
    )
    total_discount = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Discount"),
    )
    total_vat = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Vat"),
    )

    def total(self):
        return self.total_income + self.total_fee - self.total_discount + self.total_vat
    total = property(total)

    def total_no_vat(self):
        return self.total_income + self.total_fee - self.total_discount
    total_no_vat = property(total_no_vat)

    def update_totals(self):
        invoice = self.invoice
        try:
            first_invoicerow=invoice.invoicerows.all()[0]
            first_invoicerow.update_totals()
        except:
            pass

    class Meta:
        verbose_name = _("vat row")
        verbose_name_plural = _("vat rows")
        ordering = ('vat__ordering', 'vat__code')

    def __unicode__(self):
        return u"%s %s %s %s" % (self.invoice, self.vat, self.total_income, self.total_vat)

class Settings(JSettings):
    pass

from invoice import signals
