from django.utils.translation import ugettext_lazy as _

from admin_tools.dashboard import modules
from procedure.models import Procedure
from procedure.statistics import total_procedure_per_year, total_procedure_per_month, total_procedure_per_organization

class ProcedureDashboard(modules.Group):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': "Procedures",
            'display': "tabs",
            'children': [
                ProcedureModels(),
                ProcedureSettings(),
            ]
        }
        super(ProcedureDashboard, self).__init__(**kwargs)

class ProcedureModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Models"),
            'models': ('procedure.models.Procedure', 'procedure.models.ProcedureRow'),
            }
        super(ProcedureModels, self).__init__(**kwargs)

class ProcedureSettings(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Settings"),
            'models':['organization.models.Settings',],
            }
        super(ProcedureSettings, self).__init__(**kwargs)

class ProcedureModelList(modules.ModelList):
    """
    generic dashboard diplays procedure menu
    """
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Procedure"),
            'models':['procedure.models.Procedure',],
            'exclude':['procedure.models.ProcedureRow',]
        }
        super(ProcedureModelList, self).__init__(**kwargs)

class CustomerProcedures(modules.Group):
    """
    customer procedure dashboard
    """
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Customer Procedures"),
            'children': [
                Procedures(title=_('Top'), order_by='-total_income', extra_filters={'store_cause__category': 'cus'}),
                Procedures(title=_('Last'), order_by='-date_p', extra_filters={'store_cause__category': 'cus'}),
                Procedures(
                    title=_('Last to delivery'), ddt=False, order_by='-date_p',
                    extra_filters={
                        'store_cause__category': 'cus',
                        'ddt': False
                    }),
                Procedures(
                    title=_('Last to invoice'), order_by='-date_p',
                    extra_filters={
                        'store_cause__category': 'cus',
                        'invoice': False
                    }),
            ],
        }
        super(CustomerProcedures, self).__init__(**kwargs)

class CustomerYearAggregateProcedures(modules.Group):
    def __init__(self, **kwargs):
        """
        customer procedure aggregate dashboard:
        - to delivery per year
        - delivered per year
        - invoiced per year
        """
        kwargs =  {
            'title': _("Customer Year Aggregate Procedures"),
            'children': [
                Procedures(
                    title=_('Total'),
                    children=total_procedure_per_year(),
                    template='admin_tools/dashboard/year_aggregate_procedures.html'
                ),
                Procedures(
                    title=_('To Delivery'),
                    children=total_procedure_per_year(others_where="AND ddt=False"),
                    template='admin_tools/dashboard/year_aggregate_procedures.html'
                ),
                Procedures(
                    title=_('Delivered'),
                    children=total_procedure_per_year(others_where="AND ddt=True"),
                    template='admin_tools/dashboard/year_aggregate_procedures.html'
                ),
                Procedures(
                    title=_('Invoiced'),
                    children=total_procedure_per_year(others_where="AND invoice=True"),
                    template='admin_tools/dashboard/year_aggregate_procedures.html'
                ),
            ],
        }
        super(CustomerYearAggregateProcedures, self).__init__(**kwargs)

class CustomerMonthAggregateProcedures(modules.Group):
    def __init__(self, **kwargs):
        """
        customer procedure aggregate dashboard:
        - to delivery per month
        - delivered per month
        - invoiced per month
        """
        kwargs =  {
            'title': _("Customer Month Aggregate Procedures"),
            'children': [
                Procedures(
                    title=_('To Delivery'),
                    children=total_procedure_per_month(others_where="AND ddt=False"),
                    template='admin_tools/dashboard/month_aggregate_procedures.html'
                ),
                Procedures(
                    title=_('Delivered'),
                    children=total_procedure_per_month(others_where="AND ddt=True"),
                    template='admin_tools/dashboard/month_aggregate_procedures.html'
                ),
                Procedures(
                    title=_('Invoiced'),
                    children=total_procedure_per_month(others_where="AND invoice=True"),
                    template='admin_tools/dashboard/month_aggregate_procedures.html'
                ),
            ],
        }
        super(CustomerMonthAggregateProcedures, self).__init__(**kwargs)

class SupplierProcedures(modules.Group):
    def __init__(self, **kwargs):
        """
        supplier procedure dashboard
        """
        kwargs =  {
            'title': _("Supplier Procedures"),
            'children': [
                Procedures(title='Top',order_by='-total_income', extra_filters={'store_cause__category': 'sup'}),
                Procedures(title='Last', order_by='-date_p', extra_filters={'store_cause__category': 'sup'}),
            ],
            'template': 'admin_tools/dashboard/modules/group.html',
        }
        super(SupplierProcedures, self).__init__(**kwargs)

class Procedures(modules.DashboardModule):
    def __init__(self, title, template='admin_tools/dashboard/procedures.html',
        order_by=None, limit=10, qs=Procedure.objects.all(), extra_filters={},
        children=None, **kwargs
    ):
        self.order_by = order_by
        self.template = template
        self.qs = qs
        self.extra_filters = extra_filters
        self.limit = limit
        self.custom_children = children

        kwargs =  {
            'title': title,
            'template': self.template,
            'children': self.custom_children
        }
        super(Procedures, self).__init__(**kwargs)

    def init_with_context(self, context):
        request = context['request']
        if self.custom_children is None:
            qs = self.qs
            if self.extra_filters: qs = qs.filter(**self.extra_filters)
            if self.order_by: qs = qs.order_by(self.order_by)
            if self.limit: qs = qs[:self.limit]
            self.children = qs

