# -*- coding: utf-8 -*-
from datetime import date, timedelta
import decimal
import calendar

from django.conf import settings
from django.db import models, transaction
from django.db.models import Q
from django.core.mail import EmailMultiAlternatives
from django.utils.translation import ugettext_lazy as _
from django.utils.formats import number_format
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse

from jumbo.shortcuts import render_to_string
from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.db.models import Settings as JSettings
from jumbo.apps.project.models import Project
from jumbo.utils.functions import get_settings

from organization.models import Organization, Office, Contact, Bank, OrganizationAddress
from organization.models import Currency, Delivery, Vat, Agent, Commision, M2MtoVector as Vector
from payment.models import PaymentForm
from store.models import StoreCause
from product.models import Product, OrganizationPriceList


###############################
# PRATICA                     #
###############################
class ProcedureManager(models.Manager):
    def get_query_set(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        groups = user.groups.all()
        try:
            contact = user.contact
            organization = contact.organization
        except Exception:
            contact = None
            organization = None

        qs = super(ProcedureManager, self).get_query_set()

        ## se può vedere tutti i documenti ritorno il QS non filtrato
        if user.has_perm("procedure.list_all_procedures"):
            return qs

        ## creo un Q iniziale vuoto
        q = Q(id=-1)
        ## se può vedere gli Ordini creati da Utenti appartenenti ai propri GRUPPI
        if user.has_perm("procedure.list_group_procedures") and groups:
            q1 = Q(creator__groups__in=groups)
            q = q | q1

        ## se può vedere gli Ordini propri
        if user.has_perm("procedure.list_own_procedures"):
            q1 = Q(creator=user)
            q = q | q1

        ## se può vedere gli Ordini a lui associati (tramite Contact)
        if user.has_perm("procedure.list_contact_procedures") and contact:
            q1 = Q(contact=contact)
            q = q | q1

        ## se può vedere gli Ordini a lui associati (tramite Contact)
        if user.has_perm("procedure.list_organization_procedures") and organization:
            q1 = Q(organization=organization)
            q = q | q1

        ## se può vedere gli Ordini Cienti
        if user.has_perm("procedure.list_customer_procedures"):
            q1 = Q(store_cause__category='cus')
            q = q | q1

        ## se può vedere gli Ordini Fornitore
        if user.has_perm("procedure.list_supplier_procedures"):
            q1 = Q(store_cause__category='sup')
            q = q | q1

        ## se può vedere gli Oedini Accettati
        if user.has_perm("procedure.list_accepted_procedures"):
            q1 = Q(accepted=True)
            q = q | q1

        return qs.filter(q)

class Procedure(DateModel, UserModel):
    objects = models.Manager()

    status = models.CharField(
        choices=get_settings("procedure.settings", "PROCEDURE_STATUS"),
        max_length=3, default=0,
        verbose_name=_('Status'),
    )
    ## ForeignKey
    owner = models.ForeignKey (
        Organization,
        related_name = "owner_procedures",
        db_index = True,
        verbose_name = _("Bill to"),
    )
    recipient = models.ForeignKey (
        Organization,
        null = True, blank = True,
        related_name = "recipient_procedures",
        db_index = True,
        verbose_name = _("Ship to"),
    )
    project = models.ForeignKey (
        Project,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Project"),
    )
    payment_form = models.ForeignKey (
        PaymentForm,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Payment Form"),
    )
    office = models.ForeignKey (
        Office,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Office"),
    )
    contact = models.ForeignKey (
        Contact,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Contact"),
    )
    bank = models.ForeignKey (
        Bank,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Bank")
    )
    currency = models.ForeignKey (
        Currency,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Currency")
    )
    delivery = models.ForeignKey (
        Delivery,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Delivery")
    )
    vat = models.ForeignKey (
        Vat,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Vat")
    )
    agent = models.ForeignKey (
        Agent,
        null = True, blank = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Agent"),
    )
    commision = models.ForeignKey (
        Commision,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Agent & Commision")
    )
    price_list = models.ForeignKey (
        OrganizationPriceList,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Prices List & Discount")
    )
    vector = models.ForeignKey (
        Vector,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Vector")
    )
    store_cause = models.ForeignKey (
        StoreCause,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Store Cause")
    )
    shipping_address = models.ForeignKey (
        OrganizationAddress,
        blank = True, null = True,
        related_name = "procedures",
        db_index = True,
        verbose_name = _("Shipping Address")
    )
    ## Campi
    description = models.CharField(
        max_length = 255,
        db_index = True,
        verbose_name = _("Description")
    )
    number = models.CharField (
        max_length = 50,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Number"),
        db_column = "pnumber"
    )
    owner_number = models.CharField (
        max_length = 50,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Ref Owner Number"),
    )
    date_p = models.DateField (
        db_index = True,
        verbose_name = _("Date")
    )
    ddt = models.BooleanField (
        ## pratica già trasformata in ddt
        default = False,
        db_index = True,
        verbose_name = _("DDT Complete"),
    )
    invoice = models.BooleanField (
        ## pratica già trasformata in fattura
        default = False,
        db_index = True,
        verbose_name = _("Invoice Complete"),
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    internal_note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Internal Note"),
    )
    accepted = models.BooleanField (
        ## ordine accettato
        default = False,
        db_index = True,
        verbose_name = _("Accepted"),
    )
    total_income = models.DecimalField (
        default = 0,
        max_digits = 11,
        decimal_places = 4,
        verbose_name = _("Total Income"),
    )
    total_fee = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Fee"),
    )
    total_discount = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Discount"),
    )
    total_vat = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total VAT"),
    )
    # Altro indirizzo di spedizione (con ragione sociale)
    other_shipping_address_owner = models.CharField (
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("Other shipping address owner"),
    )
    other_shipping_address = models.CharField (
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("Other shipping address"),
    )
    # data prevista consegna
    date_delivery = models.DateField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Date Delivery"),
    )
    ## campi condizioni generali
    # validita offerta
    quote_firm_for = models.CharField (
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("Quote Firm For"),
    )
    # tempi di consegna
    delivery_window = models.CharField (
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("Delivery Window"),
    )

    def save(self, force_insert=False, force_update=False):
        if not self.number:
            ## solo se l'ordine è nuovo e non ha ancora un numero assegnato.
            from store.utils import create_update_storedocumentnumber
            self.number = create_update_storedocumentnumber(
                'p', curr_date=self.date_p, store_document=self.store_cause.store_document
            )
        if self.pk:
            old_obj = Procedure.objects.get(pk=self.pk)

            old_accepted = old_obj.accepted
            old_invoice = old_obj.invoice
            old_ddt = old_obj.ddt

            change = False

            if old_accepted != self.accepted:
                change = True
            if old_ddt != self.ddt:
                change = True
            if old_invoice != self.invoice:
                change = True

            if change and get_settings('procedure.settings', 'SEND_PROCEDURE_PROCESS_BY_MAIL'):
                self.send_mail_procedure_process_status()
        super(Procedure, self).save(force_insert=force_insert, force_update=force_update)


    def store_rows(self):
        return ProcedureRow.objects.filter(procedure=self, ddt=False).order_by('product__positions__position').distinct()

    #def default_address(self):
    #    address = []
    #    addr = self.owner.get_addresses()[0]
    #    address.append("%s %s, %s" % (addr.get_address_type_display() or "", addr.name, addr.number))
    #    address.append("%s %s (%s)" % (addr.zip_code, addr.locality, addr.province))
    #    address.append("%s" % addr.country)
    #    return address

    def get_payment_installments(self):
        if not self.payment_form:
            return []

        if not self.payment_form.installments.all():
            return []

        installments = []
        total_amount = 0
        for inst in self.payment_form.installments.all():
            installment = {'name': inst.name, 'full_name': inst.full_name,
                           'maturity': self.date_p + timedelta(days=inst.days)}
            if inst.end_month:
                maturity = installment['maturity']
                range = calendar.monthrange(
                    maturity.year,
                    maturity.month
                )
                installment['maturity'] = (
                    date(maturity.year, maturity.month, range[1]) +
                    timedelta(days=inst.days_end_month)
                )
            ## Arrotondo la divisione e lascio il resto per l'ultima rata
            installment['amount'] = int(
                self.totals['total'] * inst.amount / 100
            )
            ## controllo l'arrotondamento
            total_amount += installment['amount']
            installments.append(installment)
        ## controllo l'arrotondamento
        if total_amount != self.totals['total']:
            installments[-1]['amount'] = installments[-1]['amount'] - total_amount + self.totals['total']
        return installments

    def default_invoice_address(self):
        invoice_addresses = self.owner.get_addresses().filter(address_scopes__is_invoice_address=True)
        if invoice_addresses:
            return invoice_addresses[0]
        return None

    def default_invoice_address1(self):
        addr = self.default_invoice_address()
        return "%s %s, %s" % (addr.get_address_type_display() or "", addr.name, addr.number)

    def default_invoice_address2(self):
        addr = self.default_invoice_address()
        return "%s %s (%s)" % (addr.zip_code, addr.locality, addr.province)

    def default_invoice_address3(self):
        addr = self.default_invoice_address()
        return "%s" % addr.country

    def shipping_address1(self):
        return "%s %s, %s" % (self.shipping_address.get_address_type_display() or "", self.shipping_address.name, self.shipping_address.number)

    def shipping_address2(self):
        return "%s %s (%s)" % (self.shipping_address.zip_code, self.shipping_address.locality, self.shipping_address.province)

    def shipping_address3(self):
        return "%s" % self.shipping_address.country


    def update_totals(self):
        try:
            first_procedurerow=self.rows.all()[0]
            first_procedurerow.update_totals()
        except Exception:
            pass

    def get_deliveredquantity(self):
        qty_delivered = 0
        for row in self.rows.all():
            qty_delivered += row.get_deliveredquantity()
        return qty_delivered

    def get_notdeliveredquantity(self):
        qty_not_delivered = 0
        for row in self.rows.all():
            qty_not_delivered += row.get_notdeliveredquantity()
        return qty_not_delivered

    def get_deliveredvalue(self):
        if self.ddt:
            return self.total_income

        qty_delivered = 0
        for row in self.rows.all():
            qty_delivered += row.get_deliveredvalue()
        return qty_delivered

    def get_notdeliveredvalue(self):
        return self.total_income - self.get_deliveredvalue()

    def get_backorder_quantity_and_value(self):
        qty_not_delivered = 0
        for row in self.rows:
            qty_not_delivered += row.get_notdeliveredquantity
        return qty_not_delivered

    def get_totals(self):
        # Ho dovuto mettere la forzatura a Decimal solamente perche'
        # questo metodo viene chiamato anche nella funzione __unicode__.
        # Non so perche' ma quando viene chiamato da quella specifica funzione
        # i valori self.totale_income, self.total_fee, etc sono delle stringhe,
        # mentre normalemente (e giustamente) sono dei decimal.
        #
        totals = {}
        totals['income'] = decimal.Decimal(self.total_income)
        totals['fee'] = decimal.Decimal(self.total_fee)
        totals['discount'] = decimal.Decimal(self.total_discount)
        totals['vat'] = decimal.Decimal(self.total_vat)
        totals['total_no_vat'] = decimal.Decimal(self.total_income) + \
                                 decimal.Decimal(self.total_fee) - \
                                 decimal.Decimal(self.total_discount)
        totals['total'] = totals['total_no_vat'] + decimal.Decimal(self.total_vat)
        return totals
    totals = property(get_totals)

    def send_mail_procedure_process_status(self, template_subject="admin/procedure/procedure/email/process_status_email_subject.txt",
                                   template_body_html="admin/procedure/procedure/email/process_status_email.html",
                                   template_body_txt="admin/procedure/procedure/email/process_status_email.txt", **kwargs):
        subject = render_to_string(template_subject)
        body_msg = render_to_string(template_body_txt)

        site = Site.objects.get_current()
        kwargs.update({'site':site})

        html_body_msg = render_to_string(template_body_html, procedure = self, **kwargs)
        from_mail = get_settings('procedure.settings', 'PROCESS_STATUS_FROM_MAIL')

        emails = set()
        if self.contact and self.contact.email:
            emails.add(self.contact.email)

        if emails:
            msg = EmailMultiAlternatives(subject, body_msg,
                                        from_mail, emails)
            msg.attach_alternative(html_body_msg, "text/html")
            msg.send()



    if "suspension" in settings.INSTALLED_APPS:
        def get_stops(self, date_s = date.today()):
            """
            restituisce tutte le sospensioni della Pratica alla data specificata
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)
            return stop


    if "suspension" in settings.INSTALLED_APPS:
        def is_stopped(self, date_s = date.today()):
            """
            restituisce True o False se la Pratica e' sospesa
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)[0]
            if stop: return True
            return False


    if "suspension" in settings.INSTALLED_APPS:
        def is_stopped_in(self, date_begin = date.today(), date_end = date.today()):
            """
            restituisce True o False se eiste già una sospensione della Pratica nel periodo specificato.
            """
            stop = self.suspensions.filter(date_begin__lte=date_end, date_end__gte=date_begin)[0]
            if stop: return True
            return False


    if "suspension" in settings.INSTALLED_APPS:
        def is_forever_stopped(self, date_s = date.today()):
            """
            restituisce True o False se la Pratica e' sospesa definitivamente:
            e' sospesa definitivamente se non esiste la data di fine
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__isnull=True)[0]
            if stop: return True
            return False


    if "ddt" in settings.INSTALLED_APPS:
        def is_alldelivered(self):
            """
            restituisce True se la Pratica è stata consegnata o se tutte le Righe Pratica sono consegnate
            """
            if self.ddt:
                return True
            ### verifica se ci sono righe da consegnare
            frows = self.rows.filter(ddt=False)
            trows = self.rows.filter(ddt=True)
            if len(trows) == 0:
                return False
            if len(frows) == 0:
                return True


    if "ddt" in settings.INSTALLED_APPS:
        def set_alldelivered(self):
            """
            Imposta a True se tutte le Righe Pratica sono state consegnate
            """
            if self.is_alldelivered():
                self.ddt = True
                self.save()

    if "ddt" in settings.INSTALLED_APPS and "invoice" in settings.INSTALLED_APPS:
        #
        # restituisce True se tutte le Righe DDT collegate alle Righe Pratica sono fatturate
        #
        def is_ddtrows_allinvoiced(self):
            value = False
            for row in self.rows.all():
                for ddtrow in row.ddtrows.all():
                    if not ddtrow.ddt.is_allinvoiced() or not self.is_alldelivered():
                        value = False
                    else:
                        value = True
            return value

    if "invoice" in settings.INSTALLED_APPS:
        def is_allinvoiced(self):
            """
            restituisce True se la Pratica è stata fatturata o se tutte le Righe Pratica sono fatturate
            """
            if self.invoice:
                return True
            ### verifica se ci sono righe da fatturate
            rows = self.rows.filter(invoice=False, store_cause__invoice=True)
            if len(rows):
                return False
            return True


    if "invoice" in settings.INSTALLED_APPS:
        def set_allinvoiced(self):
            """
            Imposta a True se tutte le Righe Pratica sono state fatturate
            """
            if self.is_allinvoiced():
                self.invoice = True
                self.save()

    def set_tour (self, tour):
        """
        assegna o modifica giro della Pratica e del Beneficiario
        """
        pass

    def set_office (self, office):
        """
        assegna o modifica ufficio della Pratica e del Beneficiario
        """
        pass

    if "suspension" in settings.INSTALLED_APPS:
        def set_start(self, date_end = date.today()):
            #
            # crea una sospensione con data inizio = NULL e data fine = giorno precedente a quello passato
            # La Pratica è attiva dal giorno impostato
            #
            pass
            #from suspension.models import Suspension
            # TODO : Funzione errata. Usa la variabile cause che non e' definita
            #suspension = Suspension(procedure=self, disable_cause=cause, date_begin=None, date_end=date_end+timedelta(days=-1))
            #suspension.save()

    if "suspension" in settings.INSTALLED_APPS:
        def set_restart(self, date_end = date.today()):
            """
            attiva la Pratica disattivata da una sospensione modificando la
            data di fine sospensione inserendo il giorno precedente.
            La data passata deve essere compresa nell'intervallo di sospensione.
            """
            suspensions = self.get_stops(date_end)
            for suspension in suspensions:
                suspension.date_end = date_end+timedelta(days=-1)
                suspension.save()

    if "suspension" in settings.INSTALLED_APPS:
        def set_stop(self, cause, date_begin = date.today(), date_end = None):
            """
            disattiva la Pratica creando una sospensione dalla data di inizio
            alla data di fine.
            Se la data di fine è NULL la Pratica è sospesa definitivamente.
            Non verifica se c'è una sovrapposizione di sospensioni
            CONTROLLI:
            - La data di inizio deve essere <= alla data di fine
            - La data di inizio deve essere > della data di ultima consegna (get_last_date_delivery)
            """
            from suspension.models import Suspension
            suspension = Suspension(procedure=self, disable_cause=cause, date_begin=date_begin, date_end=date_end)
            suspension.save()

    if "ddt" in settings.INSTALLED_APPS:
        @transaction.commit_manually
        def create_or_append_to_ddt(self, ddt = None, date_d = date.today()):
            """
            crea o aggiunge ad un DDT le Righe Pratica
            """
            ## se l'Ordine è già stato consegnato esco
            if self.is_alldelivered():
                transaction.rollback()
                return False
            ## se l'Ordine non è stato accettato esco
            if not self.accepted:
                transaction.rollback()
                return False
            procedurerows = self.rows.filter(ddt=False, store_cause__ddt=True)
            if not ddt:
                from ddt.models import Ddt
                ddt = Ddt(
                    owner=self.owner, recipient=self.recipient,
                    payment_form=self.payment_form,
                    office=self.office, contact=self.contact,
                    store_cause=self.store_cause, date_d=date_d,
                )
                ddt.save()

            for row in procedurerows:
                row.add_to_ddt(ddt)

            if ddt.ddtrows.count() == 0:
                transaction.rollback()
                return False
            self.set_alldelivered()
            transaction.commit()
            return True

    if "invoice" in settings.INSTALLED_APPS:
        @transaction.commit_manually
        def create_or_append_to_invoice(self, invoice = None, date_i = date.today()):
            """
            crea o aggiunge ad una Fattura le Righe Pratica
            """
            ## se la Pratica è già stata fatturata esco
            if self.is_allinvoiced():
                transaction.rollback()
                return
            procedurerows = self.rows.filter(invoice=False, store_cause__ddt=False, store_cause__invoice=True)
            if not invoice:
                from invoice.models import Invoice
                invoice = Invoice(
                    owner=self.owner, recipient=self.recipient,
                    payment_form=self.payment_form,
                    office=self.office, contact=self.contact,
                    store_cause=self.store_cause, date_i=date_i,
                )
                invoice.save()

            for row in procedurerows:
                row.add_to_invoice(invoice)

            if invoice.invoicerows.count() == 0:
                transaction.rollback()
                return False
            self.set_allinvoiced()
            transaction.commit()
            return True

    class Meta:
        verbose_name = _("procedure")
        verbose_name_plural = _("procedures")
        permissions = (
            ('add_customer_procedure',          'Can ADD Customer Procedure'),
            ('add_supplier_procedure',          'Can ADD Supplier Procedure'),

            ('change_customer_procedure',       'Can CHANGE Customer Procedure'),
            ('change_supplier_procedure',       'Can CHANGE Supplier Procedure'),

            ('view_procedure',                  'Can VIEW Procedure'),
            ('view_customer_procedure',         'Can VIEW Customer Procedure'),
            ('view_supplier_procedure',         'Can VIEW Supplier Procedure'),

            ('list_all_procedures',             'Can LIST ALL Procedures'),
            ('list_group_procedures',           'Can LIST Group Procedures'),
            ('list_own_procedures',             'Can LIST Own Procedures'),
            ('list_contact_procedures',         'Can LIST Contact Procedures'),
            ('list_organization_procedures',    'Can LIST Organization Procedures'),
            ('list_customer_procedures',        'Can LIST Customer Procedures'),
            ('list_supplier_procedures',        'Can LIST Supplier Procedures'),
            ('list_accepted_procedures',        'Can LIST Accepted Procedures'),
        )

    def __unicode__(self):
        return u"%(date)s, %(number)s - total: %(total)s" % {'date'  : self.date_p,
                                                             'number': self.number,
                                                             'total' : number_format(self.totals['total'], 2)}

##################################
# RIGA PRATICA                   #
##################################
UNIT_TYPE = (
    ("pz", _("Pezzi")),
    ("ml", _("Metro lineare")),
    ("mq", _("Metro quadro")),
    ("mc", _("Metro cubo")),
    ("kg", _("Chili")),
    ("t", _("Tonnellate")),
    ("l", _("Litri")),
    ("cp", _("a corpo")),
    ("hh", _("ore")),
    ("gg", _("giorni")),
    ("pa", _("paio")),
)

class ProcedureRow (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    procedure = models.ForeignKey (
        Procedure,
        related_name = "rows",
        db_index = True,
        verbose_name = _("Procedure"),
    )
    product = models.ForeignKey (
        Product,
        related_name = "rows",
        db_index = True,
        verbose_name = _("Product"),
    )
    vat = models.ForeignKey (
        Vat,
        related_name = "rows",
        db_index = True,
        verbose_name = _("Vat"),
    )
    store_cause = models.ForeignKey (
        StoreCause,
        related_name = "rows",
        db_index = True,
        verbose_name = _("Store Cause"),
    )
    project = models.ForeignKey(
        Project,
        null = True, blank = True,
        db_index = True,
        related_name = "rows",
        verbose_name = _("Project"),
    )
    ## Campi
    # Codice che serve per tracciare altro che non sia il codice del prodotto
    # (ad esempio in thunder noi abbiamo dei prodotti generici (per evitare di
    #  gestire tt il magazzino) e usiamo questo codice per inserire il codice del
    #  prodotto generico)
    code = models.CharField (
        max_length = 50,
        db_index = True,
        blank = True, null = True,
        verbose_name = _("Code"),
    )
    quantity = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Quantity"),
    )
    m_unit = models.CharField (
        max_length = 2,
        choices = UNIT_TYPE,
        default = "pz",
        db_index = True,
        verbose_name = _("Unit"),
    )
    price = models.DecimalField (
        max_digits = 11,
        decimal_places = 4,
        default = 0,
        verbose_name = _("Unit Price"),
    )
    not_apply_discount = models.BooleanField(
        default = False,
        verbose_name = _("No Discount")
    )
    fee = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Fee"),
    )
    discount = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Discount"),
    )
    perc_discount1 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("Discount 1 %")
    )
    perc_discount2 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("Discount 2 %"),
    )
    ddt = models.BooleanField (
        db_index = True,
        verbose_name = _("DDT Comp."),
    )
    invoice = models.BooleanField (
        db_index = True,
        verbose_name = _("Inv. Comp."),
    )
    description = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Description")
    )
    long_description = models.TextField (
        blank = True, null = True,
        verbose_name = _("Long Description"),
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    to_print = models.BooleanField (
        db_index = True,
        default = True,
        verbose_name = _("To Print"),
    )
    # data prevista consegna
    date_delivery = models.DateField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Date Delivery"),
    )

    def save(self, force_insert=False, force_update=False):
        super(ProcedureRow, self).save(force_insert=force_insert, force_update=force_update)
        self.update_totals()

    def save_not_update(self, force_insert=False, force_update=False):
        super(ProcedureRow, self).save(force_insert=force_insert, force_update=force_update)

    def delete(self, using=None):
        super(ProcedureRow, self).delete(using=using)
        self.update_totals()

    def get_description(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or "%s %s" % (self.product.code, self.product.name)

    def get_description_wo_code(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or self.product.name

    def get_list_positions(self):
        """
        Resituisce la lista delle posizioni del prodotto nel magazzino selezionato
        """
        l = []
        for p in self.product.positions.filter(store=self.store_cause.store):
            l.append("%s" % p.position)
        return l


    def get_net_price(self):
        p = decimal.Decimal(self.price)

        # Se NOT_APPLY_DISCOUNT e' flaggato non calcolo gli sconti
        if not self.not_apply_discount and self.procedure.price_list:
            d1 = self.procedure.price_list.discount1
            d2 = self.procedure.price_list.discount2
            p = (self.price + self.fee - self.discount)# * (1 - d1/100) * (1 - d2/100)
            ## arrotondamento per difetto e per eccesso
            p += decimal.Decimal('0.00005')

        return p.quantize(decimal.Decimal('0.0001'), rounding='ROUND_DOWN')
    get_net_price.short_description = _("Net Price")
    net_price = property(get_net_price)


    #def get_income(self):
    #    return self.price + self.fee - self.discount


    def get_total_income(self):
        """
        Restituisce il totale imponibile calcolato sulla Riga: quantita x prezzo
        """
        p = self.quantity * self.price
        return p.quantize(decimal.Decimal('0.01'))
    get_total_income.short_description = _("Total Income")
    total_income = property(get_total_income)


    def get_total_fee(self):
        """
        Restituisce il totale spese calcolato sulla Riga: quantita x spesa accessoria
        """
        p = self.quantity * self.fee
        return p.quantize(decimal.Decimal('0.01'))
    get_total_fee.short_description = _("Total Fee")
    total_fee = property(get_total_fee)


    def get_total_discount(self):
        """
        Restituisce il totale sconto calcolato sulla Riga: quantita x sconto
        """
        # Nel calcolo dello sconto devo arrotondare tt a 2 decimali altrimenti
        # i totali dell'ordine e dei ddt sballano di qualche decimale
        #
        discount1 = self.price * self.perc_discount1 / 100
        discount2 = (self.price - discount1) * self.perc_discount2 / 100
        disc_tot = self.discount + discount1.quantize(decimal.Decimal('0.01')) + discount2.quantize(decimal.Decimal('0.01'))
        total_dis = self.quantity * disc_tot
        return total_dis.quantize(decimal.Decimal('0.01'))
    get_total_discount.short_description = _("Total Discount")
    total_discount = property(get_total_discount)

    def get_val_discount(self):
        """
        Restituisce il totale sconto calcolato sulla Riga: quantita x sconto
        """
        discount1 = self.price * self.perc_discount1 / 100
        discount2 = (self.price - discount1) * self.perc_discount2 / 100
        total_dis = self.discount + discount1.quantize(decimal.Decimal('0.01')) + discount2.quantize(decimal.Decimal('0.01'))
        return total_dis.quantize(decimal.Decimal('0.01'))
    get_val_discount.short_description = _("Discount Value")
    val_discount = property(get_val_discount)

    def get_total(self):
        """
        Restituisce il Totale generale calcolato sommando e sottrando tutti i totali
        """
        #price = self.total_income * (1 - (self.perc_discount1 / 100))
        #price = price * (1 - (self.perc_discount2 / 100))
        price = self.total_income + self.total_fee - self.total_discount
        return price.quantize(decimal.Decimal('0.01'))
    get_total.short_description = _("Total")
    total = property(get_total)


    def update_totals(self):
        """
        Aggiorna i totali dell'"Ordine" e delle righe relative all'"Iva Ordine"
        """
        procedure = self.procedure
        rows=self.procedure.rows.all()
        VatRow.objects.filter(procedure=procedure).delete()
        total = {'income': 0, 'fee': 0, 'discount': 0, 'total': 0, 'vat': 0}
        subtotal = {'income': {}, 'fee': {}, 'discount': {}, 'vat': {}, 'total': {}}

        for row in rows:
            subtotal['income'][row.vat]=subtotal['income'].get(row.vat, 0)
            subtotal['income'][row.vat]+=row.total_income

            subtotal['fee'][row.vat]=subtotal['fee'].get(row.vat, 0)
            subtotal['fee'][row.vat]+=row.total_fee

            subtotal['discount'][row.vat]=subtotal['discount'].get(row.vat, 0)
            subtotal['discount'][row.vat]+=row.total_discount

            subtotal['total'][row.vat]=subtotal['total'].get(row.vat, 0)
            subtotal['total'][row.vat]+=row.total

        for vat in subtotal['income'].keys():
            ## calcolo Totali Procedura
            total['income']+=subtotal['income'][vat]
            total['fee']+=subtotal['fee'][vat]
            total['discount']+=subtotal['discount'][vat]
            total['total']+=subtotal['total'][vat]
            ## calcolo Subtotale IVA per ogni aliquota
            subtotal['vat'][vat]=subtotal['total'][vat]*vat.aliquot_perc/100
            total['vat']+=subtotal['vat'][vat]
            ## creo nuovo oggetto VatRow
            vatrow = VatRow()
            vatrow.procedure = procedure
            vatrow.vat = vat
            vatrow.total_income = subtotal['income'][vat]
            vatrow.total_fee = subtotal['fee'][vat]
            vatrow.total_discount = subtotal['discount'][vat]
            vatrow.total_vat = subtotal['vat'][vat]
            vatrow.save()

        total['totals']=total['total']+total['vat']

        procedure.total_income = total['income']
        procedure.total_fee = total['fee']
        procedure.total_discount = total['discount']
        procedure.total_vat = total['vat']
        procedure.save()

    if "suspension" in settings.INSTALLED_APPS:
        def get_stops(self, date_s = date.today()):
            """
            restituisce tutte le sospensioni della Riga Pratica alla data specificata
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)
            return stop

    if "suspension" in settings.INSTALLED_APPS:
        def is_stopped(self, date_s = date.today()):
            """
            restituisce True o False se la Riga Pratica e' sospesa
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)[0]
            if stop: return True
            return False

    if "suspension" in settings.INSTALLED_APPS:
        def is_stopped_in(self, date_begin = date.today(), date_end = date.today()):
            """
            restituisce True o False se eiste già una sospensione della Riga Pratica nel periodo specificato.
            """
            stop = self.suspensions.filter(date_begin__lte=date_end, date_end__gte=date_begin)[0]
            if stop: return True
            return False

    if "suspension" in settings.INSTALLED_APPS:
        def is_forever_stopped(self, date_s = date.today()):
            """
            restituisce True o False se la Riga Pratica e' sospesa definitivamente:
            e' sospesa definitivamente se non esiste la data di fine
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__isnull=True)[0]
            if stop: return True
            return False

    if "suspension" in settings.INSTALLED_APPS:
        def set_start(self, date_end = date.today()):
            #
            # crea una Sospensione con data inizio = NULL e data fine = giorno precedente a quello passato
            # La Riga Pratica è attiva dal giorno impostato
            #
            pass
            #from suspension.models import Suspension
            #suspension = Suspension(procedurerow=self, disable_cause=cause, date_begin=None, date_end=date_end+timedelta(days=-1))
            #suspension.save()

    if "suspension" in settings.INSTALLED_APPS:
        def set_restart(self, date_end = date.today()):
            """
            attiva la Riga Pratica disattivata da una sospensione modificando la
            data di fine sospensione inserendo il giorno precedente.
            La data passata deve essere compresa nell'intervallo di sospensione.
            """
            suspensions = self.get_stops(date_end)
            for suspension in suspensions:
                suspension.date_end = date_end+timedelta(days=-1)
                suspension.save()

    if "suspension" in settings.INSTALLED_APPS:
        def set_stop(self, cause, date_begin = date.today(), date_end = None):
            """
            disattiva la Riga Pratica creando una sospensione dalla data di inizio
            alla data di fine.
            Se la data di fine è NULL la Riga Pratica è sospesa definitivamente.
            Non verifica se c'è una sovrapposizione di sospensioni
            CONTROLLI:
            - La data di inizio deve essere <= alla data di fine
            - La data di inizio deve essere > della data di ultima consegna (get_last_date_delivery)
            """
            from suspension.models import Suspension
            suspension = Suspension(procedurerow=self, disable_cause=cause, date_begin=date_begin, date_end=date_end)
            suspension.save()

    if "ddt" in settings.INSTALLED_APPS:
        def get_icon_ddt(self):
            img = "<img src='%simg/admin/icon-%%s.gif' />" % settings.ADMIN_MEDIA_PREFIX

            if self.ddt:
                return img % 'yes'
            else:
                return img % 'no'
        icon_ddt = property(get_icon_ddt)

    if "ddt" in settings.INSTALLED_APPS:
        def get_last_ddt_date(self):
            """
            restituisce l'ultima data
            """
            ddtrows = self.ddtrows.all()
            if ddtrows:
                return ddtrows[len(ddtrows)-1].ddt.date_d
            return None

    if "ddt" in settings.INSTALLED_APPS:
        def get_deliveredquantity(self):
            """
            restituisce la quantita totale consegnata
            """
            ddtrows = self.ddtrows.all()
            quantity = 0
            for row in ddtrows:
                quantity += row.quantity
            return quantity
        get_deliveredquantity.short_description = _('DQ')

        def get_deliveredvalue(self):
            """
            restituisce l'ammontare totale consegnata
            """
            ddtrows = self.ddtrows.all()
            tot_value = 0
            price = self.get_net_price()
            for row in ddtrows:
                tot_value += row.quantity * price
            return tot_value

    if "ddt" in settings.INSTALLED_APPS:
        def get_notdeliveredquantity(self):
            """
            restituisce la quantita ancora da consegnare
            """
            return self.quantity - self.get_deliveredquantity()
        get_notdeliveredquantity.short_description = _("ND Quantity")

        def get_notdeliveredquantity_format(self):
            """
            restituisce la quantita ancora da consegnare - stringa formattata
            """
            return number_format(get_notdeliveredquantity)
        get_notdeliveredquantity_format.short_description = _("ND Quantity")

    if "invoice" in settings.INSTALLED_APPS:
        def get_invoicedquantity(self):
            """
            restituisce la quantita totale fatturata
            """
            quantity = 0
            # se la riga ha una store_cause con ddt
            if self.store_cause.ddt:
                ddtrows = self.ddtrows.all()
                for row in ddtrows:
                    quantity += row.get_invoicedquantity()
            # se la riga ha una store_cause senza ddt
            else:
                invoicerows = self.invoicerows.all()
                for row in invoicerows:
                    quantity += row.quantity
            return quantity

    if "invoice" in settings.INSTALLED_APPS:
        def get_notinvoicedquantity(self):
            """
            restituisce la quantita totale non fatturata
            """
            return self.quantity - self.get_invoicedquantity()
        get_notinvoicedquantity.short_description = _("NI Quantity")

        def get_notinvoicedquantity_format(self):
            """
            restituisce la quantita ancora da consegnare - stringa non formattata
            """
            return number_format(self.get_notinvoicedquantity())
        get_notinvoicedquantity_format.short_description = _("NI Quantity")

    if (
        "invoice" in settings.INSTALLED_APPS or
        "ddt" in settings.INSTALLED_APPS
    ):
        def check_header_integrity(self, objmodel):
            ## verifica che i dati di intestazione dell'"Ordine" corrispondono a quelli dell'"Oggetto" passato ("DDT" o "Fattura")
            if (
                objmodel.owner == self.procedure.owner
                and objmodel.recipient == self.procedure.recipient
                and objmodel.payment_form == self.procedure.payment_form
            ):
                return True
            return False

    if "ddt" in settings.INSTALLED_APPS:
        def force_as_delivered(self):
            """
            chiude una riga ordine in quanto non più da consegnare
            """
            self.ddt = True
            self.quantity = self.quantity - self.get_notdeliveredquantity()
            self.save()

    if "ddt" in settings.INSTALLED_APPS:
        @transaction.commit_manually
        def create_ddt(self, date_d = None, quantity=None, check_quantity=True):
            """
            crea un DDT associando Riga Ordine
            """
            if not date_d:
                date_d = date.today()

            ## se l'ordine non è stato accettato esco
            if not self.procedure.accepted:
                transaction.rollback()
                return False, "Procedure must be accepted before"

            ## se la 'Riga Ordine' è già stata consegnata interamente esco
            if self.procedure.is_alldelivered():
                transaction.rollback()
                return False, "Procedure Row Already Delivered"

            ## creo ddt nuovo
            from ddt.models import Ddt
            ddt = Ddt(
                owner=self.procedure.owner, recipient=self.procedure.recipient,
                bank=self.procedure.bank, payment_form=self.procedure.payment_form,
                office=self.procedure.office, contact=self.procedure.contact,
                delivery=self.procedure.delivery, vector=self.procedure.vector,
                shipping_address=self.procedure.shipping_address,
                store_cause=self.procedure.store_cause, date_d=date_d,
            )
            ddt.save()
            res, error = self.add_to_ddt(ddt, quantity, check_quantity=check_quantity)

            if res:
                transaction.commit()
                return ddt, ""
            else:
                transaction.rollback()
                return False, error

    if "ddt" in settings.INSTALLED_APPS:
        def add_to_ddt(self, ddt, quantity=None, check_quantity=True):
            """
            aggiunge la "Riga Ordine" ad un DDT.
            Solo la quantita' passata viene inserita in DDT e consegnata.
            Se la quantita' non e' specificata l'intera quantita' da consegnare viene inserita nel DDT
            """
            notdeliveredquantity = self.get_notdeliveredquantity()

            ## verifica che la 'Riga Ordine' non sia già stata fatturata
            ## se già fatturata non è possibile consegnare ed esce
            if self.invoice:
                return False, "Procedure Row Already Invoiced"

            ## verifica se la quantita della 'Riga Ordine' da inserire nella 'Riga DDT' sia POSITIVA
            ## se <= 0 esce, non ci sono prodotti da consegnare
            if notdeliveredquantity <= 0:
                return False, "No product to delivery"

            ## se non viene specificata una 'quantita' viene inserita in 'Riga DDT' la quantita ancora da consegnare
            if not quantity:
                quantity = notdeliveredquantity
            else:
                quantity = decimal.Decimal(quantity)

            ## se viene passata una 'quantita' viene controllata che non sia maggiore di quella da consegnare
            ## se > allora esce
            if check_quantity and quantity > notdeliveredquantity:
                return False, "DDT Quantity greater then Procedure Row quantity"

            if not self.check_header_integrity(ddt):
            ## verifica che i dati delle intestazioni dell'"Ordine" e del "DDT" corrispondono
            ## se non c'è corrispondenza esce
                return False, "DDt Header and Procedure Header are different."

            from ddt.models import DdtRow
            ddtrow = DdtRow(
                ddt=ddt, procedurerow=self, product=self.product, vat=self.vat, store_cause=self.store_cause,
                quantity=quantity, price=self.price, fee=self.fee, discount=self.val_discount,
            )
            ddtrow.save(check_quantity=check_quantity)
            return True, ""

    if "invoice" in settings.INSTALLED_APPS:
        def create_or_append_to_invoice(self, invoice = None, date_i = date.today()):
            """
            crea o aggiunge ad una Fattura la Riga Pratica
            """
            ## se la Riga Pratica è già stata fatturata esco
            if self.is_allinvoiced():
                return
            if not invoice:
                from invoice.models import Invoice
                invoice = Invoice(
                    owner=self.procedure.owner, recipient=self.procedure.recipient,
                    payment_form=self.procedure.payment_form,
                    office=self.procedure.office, contact=self.procedure.contact,
                    store_cause=self.procedure.store_cause, date_i=date_i,
                )

            self.add_to_invoice(invoice)

            if invoice.invoicerows:
                invoice.save()
                return True

            return False

    if "invoice" in settings.INSTALLED_APPS:
        def add_to_invoice(self, invoice, quantity=None):
            notinvoicedquantity = self.get_notinvoicedquantity()
            if notinvoicedquantity <= 0:
                return False

            if not quantity:
                quantity = notinvoicedquantity

            if quantity > notinvoicedquantity:
                return False

            if not self.check_header_integrity(invoice):
                return False

            from invoice.models import InvoiceRow
            invoicerow = InvoiceRow(
                invoice=invoice, procedurerow=self, product=self.product, vat=self.vat, store_cause=self.store_cause,
                quantity=quantity, price=self.price, fee=self.fee, discount=self.discount,
            )
            invoicerow.save()

            ## se la quantita non ancora fatturata corrisponde alla quantita ordinata
            if quantity == notinvoicedquantity:
                self.invoice=True
                self.save()

    if "invoice" in settings.INSTALLED_APPS:
        def force_as_invoiced(self):
            """
            chiude una riga ordine in quanto non più da fatturare
            """
            self.invoice = True
            self.quantity = self.quantity - self.get_notinvoicedquantity()
            self.save()

    if "invoice" in settings.INSTALLED_APPS:
        @transaction.commit_manually
        def set_invoice(self, invoice = None, date_i = date.today(), quantity = None):
            """
            crea o aggiunge ad una Fattura la Riga Pratica
            """

            # TODO : la funzione e' sbagliata perche' usa ancora from invoice.utils import get_next_invoice_number
            # che non esiste
            pass
#            ## se non viene passata una quantita fatturo tutto quello che deve essere ancora fatturato
#            if not quantity:
#                quantity = self.get_notinvoicedquantity()
#
#            ## se non ci sono quantita da fatturare esco
#            if quantity <= 0:
#                transaction.rollback()
#                return None
#
#            from invoice.models import Invoice, InvoiceRow
#
#            ## se le Righe Pratica sono già fatturate esco
#            if self.procedure.is_allinvoiced():
#                transaction.rollback()
#                return None
#
#            from invoice.utils import get_next_invoice_number
#            try:
#                ## se viene passato una Fattura
#                if not (
#                    invoice
#                    and invoice.owner == self.procedure.owner
#                    and invoice.recipient == self.procedure.recipient
#                    and invoice.payment_form == self.procedure.payment_form
#                ):
#                    ## se l'anagrafica o la forma di pagamento non coincidono creo una nuova Fattura
#                    invoice = Invoice(
#                        owner=self.procedure.owner, recipient=self.procedure.recipient,
#                        payment_form=self.procedure.payment_form,
#                        office=self.procedure.office, contact=self.procedure.contact,
#                        store_cause=self.procedure.store_cause, number=get_next_invoice_number(date_i), date_i=date_i,
#                    )
#                    invoice.save()
#                invoicerow = InvoiceRow(
#                    invoice=invoice, procedurerow=self, product=self.product, vat=self.vat, store_cause=self.store_cause,
#                    quantity=quantity, price=self.price, fee=self.fee, discount=self.discount,
#                )
#                invoicerow.save()
#                ## se la quantita non ancora fatturata corrisponde alla quantita ordinata
#                if quantity >= self.get_notdeliveredquantity():
#                    self.invoice=True
#                    self.save()
#                ## verifico che tutte le righe della procedura siano state fatturate ed imposto self.procedure.invoice = true
#                if self.procedure.is_allinvoiced():
#                    self.procedure.invoice = True
#                    self.procedure.save()
#            except Exception, e:
#                transaction.rollback()
#            else:
#                transaction.commit()
#
#            ## ritornare invoice: l'oggetto è utilizzato da procedure.set_invoice
#            return invoice

    if "procedure_extensions" in settings.INSTALLED_APPS:
        def get_link_assistance_extension(self):
            if  self.store_cause.category == "cus" and \
                self.store_cause.stock == 0 and self.store_cause.busy == 0 and \
                self.store_cause.move == "not":
                    if self.assistance:
                        link = reverse('admin:procedure_extensions_assistance_change', args=[self.assistance.id])
                    else:
                        link = reverse('admin:procedure_extensions_assistance_add')
                    return """
                        <a class="iframe" href="%s?procedurerow=%s&_popup=1"><img src="%sadmin/images/a.png"></a>""" % (
                            link,
                            self.id,
                            settings.MEDIA_URL,
                        )
            return ""
        get_link_assistance_extension.allow_tags = True

    class Meta:
        verbose_name = _("procedure row")
        verbose_name_plural = _("procedure rows")
        permissions = (
            ('add_customer_procedurerow', 'Can ADD Customer Procedure Row'),
            ('view_customer_procedurerow', 'Can VIEW Customer Procedure Row'),
            ('add_supplier_procedurerow', 'Can ADD Supplier Procedure Row'),
            ('view_supplier_procedurerow', 'Can VIEW Supplier Procedure Row'),
        )
        ordering = ['-procedure__date_p', '-procedure__number', 'ordering', 'id']

    def __unicode__(self):
        return u"%(procedure)s, %(product)s" % {'procedure': self.procedure, 'product': self.product}

##################################
# RIGA IVA                       #
##################################
class VatRow (DateModel, UserModel, StatusModel):
    ## ForeignKey
    procedure = models.ForeignKey (
        Procedure,
        related_name = "vattotalrows",
        db_index = True,
        verbose_name = _("Procedure"),
    )
    vat = models.ForeignKey (
        Vat,
        related_name = "vattotalrows",
        db_index = True,
        verbose_name = _("Vat"),
    )
    total_income = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Income"),
    )
    total_fee = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Fee"),
    )
    total_discount = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Discount"),
    )
    total_vat = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Vat"),
    )

    def total(self):
        return self.total_income + self.total_fee - self.total_discount + self.total_vat

    def update_totals(self):
        procedure = self.procedure
        try:
            first_procedurerow=procedure.rows.all()[0]
            first_procedurerow.update_totals()
        except Exception:
            pass

    class Meta:
        verbose_name = _("vat row")
        verbose_name_plural = _("vat rows")
        ordering = ('vat__ordering', 'vat__code')

    def __unicode__(self):
        return u"%s %s %s %s" % (self.procedure, self.vat, self.total_income, self.total_vat)

class TotalOrganizationInvoiced(DateModel, UserModel):
    organization = models.ForeignKey(
        Organization,
        related_name = "total_invoiced",
        db_index = True,
        verbose_name = _("Organization"),
    )
    year = models.IntegerField(
        verbose_name=_("Year")
    )
    total = models.DecimalField (
        default = 0,
        max_digits = 11,
        decimal_places = 4,
        verbose_name = _("Total"),
    )

class Settings(JSettings):
    pass

from procedure import signals
