# -*- coding: utf-8 -*-
from django.contrib import admin, messages
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy as ugt
from django.conf.urls.defaults import *
from django.conf import settings
from django import forms
from django.db import transaction

from jumbo.forms.models import AdminModelForm
from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.forms import DateWidget
from jumbo.middleware.thread_local import get_request
from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete import AutocompleteModelChoiceField, AutocompleteModelMultiChoiceField, AutocompleteInlineCharField

from product.models import *
#from product import views as product_views

class SettingsAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('general admin'), {'fields': (('name', 'type'), 'value',
                                                           'description')}),
        )
    list_display = ('name', 'value', 'type', 'description')
    list_display_links = ('name', )
    list_per_page = 30

### FORM
class CategoryAdminForm(AdminModelForm):
    class Meta:
        model = ProductCategory

    def clean_parent(self):
        parent = self.cleaned_data['parent']
        slug = self.cleaned_data.get('slug', '')
        if parent and slug:
            if parent.slug == slug:
                raise ValidationError(_("You must not save a category in itself!"))

            for p in parent._recurse_for_parents(parent):
                if slug == p.slug:
                    raise ValidationError(_("You must not save a category in itself!"))

        return parent


class ProductAdminForm(AdminModelForm):
    class Meta:
        model = Product


    def __init__(self, *args, **kwargs):
        super(ProductAdminForm, self).__init__(*args, **kwargs)
        self.fields['categories'] = AutocompleteModelMultiChoiceField(
            'product_category', label=_("Categories"), required=False
        )


class InventoryAdminForm(AdminModelForm):
    class Meta:
        model = Inventory

    def __init__(self, *args, **kwargs):
        super(InventoryAdminForm, self).__init__(*args, **kwargs)
        self.fields['product'] = AutocompleteModelChoiceField('product_product', label=_("Product"))


class ProductCodeAdminForm(AdminModelForm):
    class Meta:
        model = ProductCode

    def __init__(self, *args, **kwargs):
        super(ProductCodeAdminForm, self).__init__(*args, **kwargs)
        self.fields['organization'] = AutocompleteModelChoiceField('product_organization', label=_("Organization"))


class OrganizationPriceListAdminForm(AdminModelForm):
    class Meta:
        model = OrganizationPriceList

    def __init__(self, *args, **kwargs):
        super(OrganizationPriceListAdminForm, self).__init__(*args, **kwargs)
        self.fields['organization'] = AutocompleteModelChoiceField('product_organization', label=_("Organization"))


### INLINE
class CategoryTranslationInline(admin.StackedInline):
    model = ProductCategoryTranslation
    extra = 0
    fieldsets = (
        (_('detail'), {
            'fields': (('language', 'name'), 'description'),
        }),
    )


class PriceListItemInline(admin.StackedInline):
    model = PriceListItem
    extra = 0
    fields = (
        'status', 'price_list', 'product', 'date_begin', 'date_end', 'price','price_tax_included', 'discount1','discount2','min_quantity','max_quantity',
    )

class ProductTranslationInline(admin.TabularInline):
    model = ProductTranslation
    extra = 0
    fields = ('language', 'name', 'short_description', 'full_description')


class ProductCodeInline(admin.TabularInline):
    model = ProductCode
    form = ProductCodeAdminForm
    extra = 5
    max_num = 5
    fields = ('organization', 'reserved_code', 'brand', 'draw_code', 'revision')


class PositionInline(admin.TabularInline):
    model = Position
    extra = 0
    fields = ('store', 'position')


### SEARCH
class ProductSearchForm(forms.ModelForm):
    class Meta:
        model = Product
        #fields = ('code', 'name', 'production_code')
        display_fields = (
            'status',
            ('name__icontains', 'code__icontains'),
            ('code__istartswith', 'production_code__icontains',),
            ('categories__in',),
        )


    def __init__(self, *args, **kws):
        super(ProductSearchForm, self).__init__(*args, **kws)
        STATUS = (('', '-'*9), ('1', _('Active')), ('0', _('Disactive')),)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), choices=STATUS, required=False)
        self.fields['categories__in'] = forms.ModelMultipleChoiceField(
            label=_("Categories"), queryset=ProductCategory.objects.all()
        )
        self.fields['name__icontains'] = forms.CharField(label=_('Name'), required=False)
        self.fields['code__icontains'] = forms.CharField(label=_('Code'), required=False)
        self.fields['code__istartswith'] = forms.CharField(label=_('Code starts with'), required=False)
        self.fields['production_code__icontains'] = forms.CharField(label=_('Production Code'), required=False)


class PriceListItemSearchForm(forms.ModelForm):
    class Meta:
        model = PriceListItem
        display_fields = ('price_list__in', ('status', 'product__name__icontains', 'product__code'))


    def __init__(self, *args, **kws):
        super(PriceListItemSearchForm, self).__init__(*args, **kws)
        STATUS = (('', '-'*9), ('1', _('Active')), ('0', _('Disactive')),)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), choices=STATUS, required=False)
        self.fields['price_list__in'] = forms.ModelMultipleChoiceField(
            label=_("Price List"), queryset=PriceList.objects.all()
        )
        self.fields['product__name__icontains'] = forms.CharField(label=_("Product Name"), required=False)
        self.fields['product__code'] = forms.CharField(label=_("Product Code"), required=False)


### GENERAL
class CategoryAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("product/css/admin_product.css",)}

    form = CategoryAdminForm
    list_display = ('get_status_icon', 'name', 'slug', 'get_edit_icon', 'get_delete_icon')
    inlines = [CategoryTranslationInline]
    filter_horizontal = ('related_categories',)
    search_fields = ('name',)
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'parent', ('name', 'slug'))
        }),
        (_('Description & Related Categories'), {
            'fields': ('description','related_categories'),
            'classes': ('collapse',)
        })
    )


class PriceListAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("product/css/admin_product.css",)}

    def get_link_name_detail(self, obj):
        return """<a href='%(reverse_url)s' title='%(title)s' class="overlay">%(name)s</a>""" % {
            'reverse_url': reverse('admin:product_pricelist_detail', obj.id),
            'title': _("View Detail"),
            'name': obj.name
        }
    get_link_name_detail.short_description = ugt("Name")
    get_link_name_detail.admin_order_field = 'name'
    get_link_name_detail.allow_tags = True

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',), 'classes': ('collapse',)}),
        (_('visualization admin'), {'fields': ('ordering',), 'classes': ('collapse',),}),
        (_('general admin'), {'fields': ('status','name','code','pricelist_type', 'currency'),}),
        (_('time admin'), {'fields': (('date_begin','date_end',),),}),
    )
#    filter_horizontal = ('organizations',)
#    inlines = (PriceListItemInline,)
    list_display = ('get_status_icon', 'code', 'get_link_name_detail', 'date_begin', 'date_end', 'ordering', 'get_edit_icon', 'get_delete_icon')
    search_fields = ('name', 'code')


class PriceListItemAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("product/css/admin_product.css",)}

    save_on_top = True
    search_form = PriceListItemSearchForm
    fieldsets = (
        (_('logs admin'), {'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',), 'classes': ('collapse',)}),
        (_('visualization admin'), {'fields': ('ordering',), 'classes': ('collapse',),}),
        (_('general admin'), {'fields': ('status', 'price_list', 'product', 'price', 'price_tax_included', 'discount1', 'discount2', 'min_quantity', 'max_quantity',),}),
        (_('time admin'), {'fields': (('date_begin','date_end',),),}),
    )
    list_display = ('get_status_icon', 'get_price_list', 'product', 'get_date_begin',
                    'get_date_end', 'price', 'price_tax_included', 'discount1',
                    'discount2', 'min_quantity', 'max_quantity', 'get_edit_icon',
                    'get_delete_icon')
    list_display_csv = (
        'status', 'get_price_list_csv', 'product', 'get_date_begin',
        'get_date_end', 'price', 'price_tax_included', 'discount1',
        'discount2', 'min_quantity', 'max_quantity'
    )
    search_fields = ('product__name', 'product__code')

    def get_price_list_csv(self, obj):
        return "%s - %s" % (
            obj.price_list.code, obj.price_list.name
        )
    get_price_list_csv.short_description = ugt("Price List")
    
    def get_price_list(self, obj):
        return "%s<br/><b>%s</b><br/>%s - %s" % (
            obj.price_list.code, obj.price_list.name, obj.price_list.date_begin, obj.price_list.date_end
        )
    get_price_list.short_description = ugt("Price List")
    get_price_list.admin_order_field = 'price_list.name'
    get_price_list.allow_tags = True


class ProductAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("product/css/admin_product.css",)}

    save_on_top   = True
    #form          = ProductAdminForm
    search_form   = ProductSearchForm

    # FIELDS GLOABLI : Devono esserci tutti i fields da rappresentare nei due
    # form dei prodotti (web e gestionale)
    #fields           = ['creator', 'date_create','last_modifier', 'date_last_modify',
    #                    'status','code', 'production_code', 'name','categories', 'catalog_categories',
    #                    'vat', 'm_unit', 'short_description','full_description','note',
    #                    'serialnumber','min_supply','store_cause', 'ordering']

    # FIELDSETS da visualizzare quando si vuole vedere il form dei prodotti x il gestionale
    fieldsets = (
                        (_('logs admin'), {'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',), 'classes': ('collapse',)}),
                        (_('general admin'), {'fields': ('status','code', 'production_code', 'name', 'categories'),}),
                        (_('price admin'), {'fields': ('vat', 'm_unit'),}),
                        (_('description admin'), {'fields': ('short_description','full_description','note',),}),
                        (_('code admin'), {'fields': ('serialnumber','min_supply','store_cause'),}),
                    )

    search_fields = ('name', 'code', 'production_code')
    ordering      = ('code', 'name',)
    list_display  = ('get_status_icon', 'get_list_categories', 'code', 'name', 'vat',
                     'm_unit', 'min_supply', 'serialnumber', 'get_image_icon', 'get_edit_icon', 'get_delete_icon')
    list_display_csv = ('status', 'get_list_categories', 'code', 'name', 'vat', 'm_unit', 'min_supply')
    filter_horizontal = ('categories',)
    #list_editable = ['ordering',]

    inlines       = (PriceListItemInline, ProductTranslationInline,
                     ProductCodeInline, PositionInline)
    actions = ['clone_product']

    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(ProductAdmin, self).get_urls()

        my_urls = patterns('',
            url(r'^(.+)/image/$',
                self._wrap(self.detail),
                {'template_name': 'image.html'},
                name=self._view_name('image')),
            )
        return my_urls + urls

    def clone_product(self, request, queryset):
        """ Clone product """
        rows_updated = 0
        for obj in queryset:
            try:
                self.clone(request, obj)
                rows_updated += 1
            except Exception, e:
                messages.error(request, "%s: %s" % (obj.name, e))
        if rows_updated == 0:
            messages.error(request, _("ATTENTION! Nothing was cloned"))
        elif rows_updated == 1:
            message_bit = _("1 object was")
            messages.info(request, _("%s successfully cloned.") % message_bit)
        else:
            message_bit = _("%s objects were") % rows_updated
            messages.info(request, _("%s successfully cloned.") % message_bit)
    clone_product.short_description = _("Clone Product")

    @transaction.commit_manually
    def clone(self, request, obj):
        try:
            old_obj = Product.objects.get(pk=obj.pk)
            obj.pk = None
            obj.code = "< da mofificare >"
            obj.save(force_insert=True)
            ## salvo many to many collegate
            for m2m in obj._meta.many_to_many:
                setattr(obj, m2m.name, getattr(old_obj, m2m.name).all())
            obj.save()
            transaction.commit()
            return ""
        except Exception, e:
            transaction.rollback()
            raise Exception(e)

    def get_image_icon(self, result):
        opts = self.model._meta
        # se non ci sono immagini da visualizzare non creo immagine link
        if result.images.count() == 0:
            return ""

        return """<a href=%(reverse_url)s class="overlay"><img src='%(url)sadmin/images/search.png' alt='%(window_title)s'/></a>""" % {
            'reverse_url': reverse('admin:%s_%s_image' % (opts.app_label, opts.object_name.lower()), result.id),
            'url': settings.MEDIA_URL,
            'window_title': _("View %s Photo")
        }
    get_image_icon.short_description = ugt("I")
    get_image_icon.allow_tags = True


class InventoryAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("product/css/admin_product.css",)}

    form = InventoryAdminForm
    fieldsets = (
        (_('logs admin'), {'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',), 'classes': ('collapse',)}),
        (_('general admin'), {'fields': (
            'status','product', 'store', 'year', 'initial_sold_value', 'stock', 'busy',
        ),}),
    )
    search_fields = ('year', 'product__code',)
    ordering = ('-year', 'store__name', 'product__name',)
    list_display = (
        'get_status_icon', 'product', 'store', 'year', 'initial_sold_value', 'stock', 'busy', 'date_last_modify', 'get_edit_icon', 'get_delete_icon'
    )


class OrganizationPriceListAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("product/css/admin_product.css",)}

    form = OrganizationPriceListAdminForm
    fieldsets = (
        (_('logs admin'), {'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',), 'classes': ('collapse',)}),
        (_('general admin'), {'fields': ('status','price_list', 'organization', ('discount1', 'discount2',),),}),
    )
    search_fields = ('organization__code', 'organization__name',)
    list_display = (
        'get_status_icon', 'get_organization_code', 'get_organization_name', 'get_price_list_code', 'get_price_list_name', 'discount1', 'discount2', 'get_edit_icon', 'get_delete_icon'
    )


    def get_organization_code(self, obj):
        return obj.organization.code
    get_organization_code.short_description = ugt("Organization Code")
    get_organization_code.admin_order_field = 'organization.code'
    get_organization_code.allow_tags = True


    def get_organization_name(self, obj):
        return obj.organization.name
    get_organization_name.short_description = ugt("Organization Name")
    get_organization_name.admin_order_field = 'organization.name'
    get_organization_name.allow_tags = True


    def get_price_list_code(self, obj):
        return obj.price_list.code
    get_price_list_code.short_description = ugt("Price List Code")
    get_price_list_code.admin_order_field = 'price_list.code'
    get_price_list_code.allow_tags = True


    def get_price_list_name(self, obj):
        return obj.price_list.name
    get_price_list_name.short_description = _("Price List Code")
    get_price_list_name.admin_order_field = 'price_list.name'
    get_price_list_name.allow_tags = True

admin.site.register(Settings, SettingsAdmin)
admin.site.register(Product, ProductAdmin)
admin.site.register(ProductCategory, CategoryAdmin)
admin.site.register(PriceList, PriceListAdmin)
admin.site.register(PriceListItem, PriceListItemAdmin)
admin.site.register(Inventory, InventoryAdmin)
admin.site.register(OrganizationPriceList, OrganizationPriceListAdmin)

########################################
# AUTOCOMPLETE REGISTER                #
########################################
def get_product_product(request, query):
    return Product.objects.filter(status=1).order_by('code')

def get_product_category(request, query):
    return ProductCategory.objects.filter(status=1)

def get_product_organization(request, query):
    results = []
    try:
        org_exact = Organization.objects.get(code=query)
        results.append([org_exact.id, "%s - %s" % (org_exact.code, org_exact.name)])
    except:
        pass

    q1 = Q(name__istartswith=query)
    q2 = Q(code__istartswith=query)
    for other_org in Organization.objects.exclude(code=query).filter(organization_type__category="cus").filter(status=1).filter(q1|q2)[:10]:
        results.append([other_org.id, "%s - %s" % (other_org.code, other_org.name)])
    return results

autocomplete.register('product_product', model = Product, views = get_product_product
                                       , search_fields=('name', 'code', 'other_codes__reserved_code__startswith')
                                       , limit=10)

autocomplete.register('product_category', model = ProductCategory, views = get_product_category, search_fields = ('name',), limit=10)
autocomplete.register('product_organization', model = Organization, views = get_product_organization)
