# -*- coding: utf-8 -*-
import datetime

from django.utils.translation import ugettext_lazy as _

from admin_tools.dashboard import modules
from product.models import Product
from product.statistics import year_topten_sold_products, month_topten_sold_products

CURRENT_YEAR = datetime.date.today().year
LAST_YEARS = [CURRENT_YEAR, CURRENT_YEAR-1, CURRENT_YEAR-2]

class ProductDashboard(modules.Group):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Products"),
            'display': "tabs",
            'children': [
                ProductModels(),
                ProductOtherModels(),
                ProductSettings(),
            ]
        }
        super(ProductDashboard, self).__init__(**kwargs)

class ProductModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Products"),
            'models':('product.models.Product','product.models.ProductCategory',),
        }
        super(ProductModels, self).__init__(**kwargs)

class ProductOtherModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Prices"),
            'models':('product.models.PriceList', 'product.models.PriceListItem', 'product.models.OrganizationPriceList'),
        }
        super(ProductOtherModels, self).__init__(**kwargs)

class ProductStoreModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Store"),
            'models':('product.models.Position', 'product.models.Inventory',),
            }
        super(ProductStoreModels, self).__init__(**kwargs)

class ProductSettings(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Settings"),
            'models':['product.models.Settings',],
            }
        super(ProductSettings, self).__init__(**kwargs)

class YearTopTenSoldProducts(modules.Group):
    def __init__(self, **kwargs):
        top_list = []
        for year in LAST_YEARS:
            top_list.append(Products(
                title=_('Invoiced %s') % year,
                children=year_topten_sold_products(years=year, limit=10),
                template='admin_tools/dashboard/year_topten_sold_products.html'
            ))
        kwargs =  {
            'title': _("Top Ten Sold Products Per Year"),
            'children': top_list,
        }
        super(YearTopTenSoldProducts, self).__init__(**kwargs)

class YearMonthTopTenSoldProducts(modules.Group):
    def __init__(self, **kwargs):
        top_list = []
        for year in LAST_YEARS:
            top_list.append(MonthTopTenSoldProducts(
                title=_('Top Ten Sold %s') % year,
                year=year
            ))
        kwargs =  {
            'title': _("Top Ten Sold Products Per Month"),
            'children': top_list,
        }
        super(YearMonthTopTenSoldProducts, self).__init__(**kwargs)

class MonthTopTenSoldProducts(modules.Group):
    def __init__(self, **kwargs):
        year = kwargs.get('year', CURRENT_YEAR)
        top_list = []
        for month in range(1,13):
            top_list.append(Products(
                title=_('Sold Products Month %s') % month,
                children=month_topten_sold_products(years=year, months=month, limit=10),
                template='admin_tools/dashboard/month_topten_sold_products.html'
            ))
        kwargs =  {
            'title': _("Year %s") % year,
            'children': top_list,
        }
        super(MonthTopTenSoldProducts, self).__init__(**kwargs)

class Products(modules.DashboardModule):
    def __init__(
        self, title, template='admin_tools/dashboard/products.html',
        order_by=None, limit=10, qs=Product.objects.all(), extra_filters={},
        children=None, **kwargs
    ):
        self.order_by = order_by
        self.template = template
        self.qs = qs
        self.extra_filters = extra_filters
        self.limit = limit
        self.custom_children = children

        kwargs =  {
            'title': title,
            'template': self.template,
            'children': self.custom_children
        }
        super(Products, self).__init__(**kwargs)

    def init_with_context(self, context):
        request = context['request']
        if self.custom_children is None:
            qs = self.qs
            if self.extra_filters: qs = qs.filter(**self.extra_filters)
            if self.order_by: qs = qs.order_by(self.order_by)
            if self.limit: qs = qs[:self.limit]
            self.children = qs
