# -*- coding: utf-8 -*-

from datetime import date, timedelta
import decimal

from django.conf import settings
from django.db import models
from django.db.models import Sum
from django.db.models import Q, connection
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify
from django.contrib.contenttypes import generic
from django.core import urlresolvers
from django.utils.formats import number_format
from django.utils.translation import get_language

from jumbo.db.fields import HTMLField
from jumbo.middleware.thread_local import get_current_user
from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel, PeriodicityModel
from jumbo.db.models import Settings as JSettings
from jumbo.apps.i18n.models import TranslationModel, I18NModel
from jumbo.utils.functions import get_settings, slugify
from jumbo.apps.i18n.templatetags.transdb import transdb

from store.models import Store, StoreCause
from organization.models import Organization, OrganizationType, Vat, Currency

import mptt


##################################
# CATEGORIA                      #
##################################
class ProductCategory(DateModel, UserModel, StatusModel, OrderedModel, I18NModel):
    name = models.CharField(
        max_length = 200,
        db_index=True,
        verbose_name = _("Name"),
    )
    slug = models.SlugField(
        blank = True, null = True,
        db_index=True,
        verbose_name = _("Slug"),
        help_text = _("Used for URLs, auto-generated from name if blank"),
    )
    parent = models.ForeignKey(
        'self',
        blank = True, null = True,
        related_name = 'children',
    )
    description = HTMLField(
        blank = True, null = True,
        verbose_name = _("Description"),
        help_text = _("Description"),
    )
    related_categories = models.ManyToManyField(
        'self',
        blank = True, null = True,
        symmetrical = False,
        related_name = '%(class)s_related_categories',
        verbose_name = _('Related Categories'),
    )
    class Meta:
        verbose_name = _("Product Category")
        verbose_name_plural = _("Product Categories")
        permissions = (
            ("view_productcategory"    , "Can view product category"),
        )


    def products_by_ordering(self):
        ap = self.active_products()
        return ap.order_by('ordering')


    def active_products(self, include_children=False, **kwargs):
        #if not include_children:
        qry = self.products.all()
        #else:
        #    cats = self.get_all_children(include_self=True)
        #    qry = Product.objects.filter(category__in=cats)
        return qry.filter(status=True)


    def get_absolute_url(self):
        parents = self.get_ancestors()
        slug_list = [cat.slug for cat in parents]
        if slug_list:
            slug_list = "/".join(slug_list) + "/"
        else:
            slug_list = ""
        return urlresolvers.reverse('ecommerce-productcategory',
            kwargs={'parent_slugs' : slug_list, 'slug' : self.slug})


    def get_separator(self):
        return ">>>"


    def get_tree_name(self):
        return self.__unicode__()
    get_tree_name.short_description = "Tree Name"
    tree_name = property(get_tree_name)


    def __unicode__(self):
        name_list = [ c.name for c in self.get_ancestors() ]
        name_list.append(self.name)
        return self.get_separator().join(name_list)


    def save(self, force_insert=False, force_update=False):
        if not self.slug:
            self.slug = slugify(self.name, instance=self)

        super(ProductCategory, self).save(force_insert=force_insert, force_update=force_update)


    class Meta:
        ordering = ['parent__id', 'ordering', 'name']
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")
        permissions = (
            ("view_productcategory"    , "Can view Product Category"),
        )
mptt.register(ProductCategory, order_insertion_by=['name',])


class ProductCategoryTranslation(TranslationModel):
    parent = models.ForeignKey(
        ProductCategory,
        db_index = True,
        related_name = 'translations',
        verbose_name = _('parent'),
    )
    name = models.CharField(
        max_length = 100,
        verbose_name = _('Name'),
    )
    description = HTMLField(
        null = True, blank = True,
        verbose_name = _('Description'),
    )

    class Meta:
        verbose_name = _('Category Translation')
        verbose_name_plural = _('Category Translations')
        unique_together = (('parent','language',),)

    def __unicode__(self):
        return u'%s' % self.name[:15]


##############################
# PRODOTTO                   #
##############################
UNIT_TYPE = (
    ("pz", _("Pezzi")),
    ("ml", _("Metro lineare")),
    ("mq", _("Metro quadro")),
    ("mc", _("Metro cubo")),
    ("kg", _("Chili")), #
    ("t", _("Tonnellate")), #
    ("l", _("Litri")), #
    ("cp", _("a corpo")), #
    ("hh", _("ore")), #
    ("gg", _("giorni")), #
    ("pa", _("paio")), #
    ("rt", _("rotoli")),
    ("pc", _("pacchi")),
    ("bn", _("bancali")),
    ("sc", _("scatole")),
    ("ls", _("lastre")),
    ("pn", _("pannelli")),
)

class Product(DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    categories = models.ManyToManyField(
        ProductCategory,
        related_name="products",
        verbose_name=_("Categories"),
        blank = True, null = True
    )
    vat = models.ForeignKey(
        Vat,
        verbose_name=_("vat"),
        help_text=_('Default vat')
    )
    store_cause = models.ForeignKey(
        StoreCause,
        blank=True,
        null=True,
        verbose_name=_("Store cause")
    )
    ## Campi Modello
    code = models.CharField(
        max_length=25,
        db_index=True,
        unique=True,
        verbose_name=_('Code'),
    )
    production_code = models.CharField(
        max_length=25,
        blank=True, null=True,
        db_index=True,
        #unique=True,
        verbose_name=_('Production Code'),
    )
    vendor = models.CharField(
        max_length=200,
        blank=True, null=True,
        db_index=True,
        verbose_name=_('Vendor'),
    )
    name = models.CharField(
        db_index=True,
        max_length=100,
        verbose_name=_('Name'),
    )
    m_unit = models.CharField (
        max_length = 2,
        choices = UNIT_TYPE,
        default = "pz",
        db_index = True,
        verbose_name = _("Unit"),
    )
    short_description = models.CharField(
        blank=True,
        null=True,
        max_length=100,
        verbose_name=_("Short description"),
    )
    full_description = HTMLField(
        blank=True,
        null=True,
        verbose_name=_("Full description"),
    )
    min_supply = models.IntegerField(
        default=0,
        verbose_name=_("Min supply"),
    )
    serialnumber= models.BooleanField(
        verbose_name=_("Serial number"),
    )
    note = models.TextField(
        blank=True,
        null=True,
        verbose_name=_("Notes"),
    )

    class Meta:
        verbose_name = _("Product")
        verbose_name_plural = _("Products")
        ordering = ['code','name']
        permissions = (
            ("view_product"    , "Can view product"),
        )

    def __unicode__(self):
        total_description = u""
        if self.vendor:
            total_description += self.vendor + " - "
        if self.production_code:
            total_description += self.production_code + " - "
        total_description += "%s - %s" % (self.code, self.name)
        return total_description

    def save(self, *args, **kwargs):
        try:
            self.store_cause
        except Exception:
            self.store_cause = StoreCause.objects.get_default()
        super(Product, self).save(*args, **kwargs)

    def get_description(self):
        if self.full_description and len(self.full_description) > 0:
            return transdb(self, 'full_description')
        if self.short_description and len(self.short_description) > 0:
            return transdb(self, 'short_description')
        return ""
    description = property(get_description)

    def get_short_description(self):
        if self.short_description and len(self.short_description) > 0:
            return self.short_description
        return self.name

    def get_full_description(self):
        if self.full_description and len(self.full_description) > 0:
            return self.full_description
        return self.name

    def get_list_categories(self):
        c = [ x.name for x in self.categories.all() ]
        return ', '.join(c)
    get_list_categories.short_description = _("Categories")


    def get_list_positions(self):
        pos_list = []
        for pos in self.positions.filter(status=1):
            pos_list.append("%s %s" % (pos.store.name, pos.position))
        return pos_list

    def _filter(self, user=None, date_pr=None, **kw):
        if not user:
            user = get_current_user()

        date_pr = date_pr or date.today()

        ## prezzi ATTIVI di listini ATTIVI
        q1 = Q(status=1, price_list__status=1)

        ## listini e tipi listini non associati a nessuna ORGANIZZAZIONE
        q2 = Q(price_list__organizations__isnull=True)&Q(price_list__organization_types__isnull=True)

        organization = user.contact.organization
        ## se esiste ORGANIZZAZIONE perfeziono condizione Q2
        if organization:
            organization_type = organization.organization_type
            q2 = q2|Q(price_list__organizations=organization)|Q(price_list__organization_types=organization_type)

        q3 = Q(date_begin__lte=date_pr, date_end__gte=date_pr)|Q(
            date_begin__isnull=True, date_end__isnull=True,
            price_list__date_begin__lte=date_pr, price_list__date_end__gte=date_pr
        )


        #q1 = Q(status=1, price_list__status=1)
        ##q2 = Q(user=get_current_user())|Q(user__isnull=True)
        ## FIXME organizations__id=1

        #organization = user.contact.organization
        #organization_type = organization.organization_type
        #q2 = Q(price_list__organizations=organization)
            #|Q(price_list__organization_types=organization_type)
            #|Q(price_list__organizations__isnull=True)&Q(price_list__organization_types__isnull=True)
        #q3 = Q(date_begin__lte=now, date_end__gte=now)|Q(
            #date_begin__isnull=True, date_end__isnull=True,
            #price_list__date_begin__lte=now, price_list__date_end__gte=now
        #)
        if kw:
            q = q1 | q2 | q3 | Q(**kw)
        else:
            q = q1 | q2 | q3
        return self.prices.filter(q)

#    def get_first_price(self, user=None):
#        return self._filter(user).order_by('ordering')[0]
#    first_price = property(get_first_price)
#
#    def get_last_price(self, user=None):
#        return self._filter(user).order_by('-ordering')[0]
#    last_price = property(get_last_price)
#
#    def get_worse_price(self, user=None):
#        return self._filter(user).order_by('-price')[0]
#    worse_price = property(get_worse_price)
#
#    def get_best_price(self, user=None):
#        return self._filter(user).order_by('price')[0]
#    best_price = property(get_best_price)

#    def get_current_prices(self,  user=None, date_l=None, quantity=None):
#        date_l = date_l or date.today()
#        prices = self._filter(user, date_l)
#        if quantity:
#            prices = prices.filter(min_quantity__lte=quantity, max_quantity__gte=quantity)
#        prices = prices.order_by('min_quantity')
#        return prices

#    def get_price(self):
#        return self.best_price
#    price = property(get_price)

    def get_user_active_pricelistitem(self, user=None):
        if not user:
            user = get_current_user()
        try:
            organization = user.contact.organization
        except Exception:
            organization = None

        return self.get_organization_active_pricelistitem(organization=organization)


    def get_organization_active_pricelistitem(self, organization):
        # Recupero i listini attivi associati all'anagrafica
        pricelist = None
        organization_price_lists = organization.price_lists.all()
        if not organization_price_lists:
            organizationtype_price_lists = organization.organization_type.price_lists.all()
            if organizationtype_price_lists:
                pricelist = organizationtype_price_lists[0].price_list
        else:
            pricelist = organization_price_lists[0].price_list

        return self.get_active_pricelistitem(pricelist=pricelist)


    def get_active_pricelistitem(self, pricelist=None, pricelist_code=None, pricelist_type=None):
        today = date.today()
        price_list_item = self.prices_lists.filter(status = 1)
        q1 = Q(date_begin__isnull=True, date_end__isnull=True)
        q2 = Q(date_begin__lte=today, date_end__gte=today)
        q3 = Q(date_begin__lte=today, date_end__isnull=True)
        q4 = Q(date_begin__isnull=True, date_end__gte=today)
        price_list_item = price_list_item.filter(q1|q2|q3|q4)

        if not pricelist:
            street_price_lists = PriceList.objects.filter(pricelist_type="SP", prices_lists__product=self).distinct()
            if street_price_lists:
                pricelist = street_price_lists[0]

        price_list_item = price_list_item.filter(price_list=pricelist)

        if pricelist_code:
            price_list_item = price_list_item.filter(price_list__code=pricelist_code)

        if pricelist_type:
            price_list_item = price_list_item.filter(price_list__pricelist_type=pricelist_type)

        return price_list_item[0] if price_list_item else None


    def get_qty_active_pricelistitem(self, qty, user=None, organization=None, pricelist=None):
        # Recupero tutti i listini attivi di quel prodotto
        active_price_listitem = None

        if pricelist:
            active_price_listitem = self.get_active_pricelistitem(pricelist=pricelist)
        elif user:
            active_price_listitem = self.get_user_active_pricelistitem(user=user)
        elif organization:
            active_price_listitem = self.get_organization_active_pricelistitem(organization=organization)

        #q1 = Q(min_quantity__isnull=True)&Q(max_quantity__isnull=True)
        #q2 = Q(min_quantity__lte=qty)&Q(max_quantity__isnull=True)
        #q3 = Q(min_quantity__lte=qty)&Q(max_quantity__gte=qty)
        #q4 = Q(min_quantity__isnull=True)&Q(max_quantity__gte=qty)

        #qty_price_listitem = active_price_listitem.filter(q1|q2|q3|q4)
        return active_price_listitem
        #return qty_price_listitem


    def get_user_price(self):
        user = get_current_user()
        try:
            organization = user.contact.organization
        except Exception:
            organization = None

        return self.get_organization_price(organization=organization)

    def get_organization_price(self, organization, qty=None, with_discount=True):
        if qty:
            pricelist_item = self.get_qty_active_pricelistitem(qty=qty, organization=organization)
        else:
            pricelist_item = self.get_organization_active_pricelistitem(organization=organization)

        opl = OrganizationPriceList.objects.filter(organization=organization,
                                                   price_list = pricelist_item.price_list)
        if not opl:
            return pricelist_item.price

        if not with_discount:
            return pricelist_item.price

        price = pricelist_item.price * (1 - (opl[0].discount1 / 100))
        price = price * (1 - (opl[0].discount2 / 100))
        return price

    def get_organization_price_without_discount(self, organization, qty=None):
        return self.get_organization_price(organization=organization, qty=qty, with_discount=False)

    #price = property(get_price)

#    def _get_price_without_vat(self, list_type):
#        # recupero il listino in funzione del tipo. Nel caso ci fossero
#        # piu listini dello stesso tipo (cosa che non dovrebbe
#        # succedere) prendo solo il primo
#        #
#        p = self.prices_lists.filter(price_list__pricelist_type=list_type)
#        if not p:
#            return 0
#        p = p[0]
#        price = p.price
#        if p.price_tax_included:
#            price = p.price / (1 + self.vat.aliquot_perc / 100)
#        return price
#
#    def _get_price_with_vat(self, list_type):
#        # recupero il listino in funzione del tipo. Nel caso ci fossero
#        # piu listini dello stesso tipo (cosa che non dovrebbe
#        # succedere) prendo solo il primo
#        #
#        p = self.prices_lists.filter(price_list__pricelist_type=list_type)
#        if not p:
#            return 0
#        p = p[0]
#        price = p.price
#        if not p.price_tax_included:
#            price = p.price * (1 + self.vat.aliquot_perc / 100)
#        return price


    def is_available(self, store=None):
        """
        vero o falso in funzione della disponibilita del prodotto
        """
        availability = self.get_availability(store)
        if availability > 0:
            return True
        else:
            return False

    def is_available_serialnumber(self, store=None):
        """
        vero o falso in funzione della disponibilita del prodotto
        """
        available_sn = self.get_available_serialnumber(store)
        if available_sn:
            return True
        else:
            return False

    def get_availability(self, store=None):
        """
        restituisce il numero dei pezzi disponibili in magazzino
        """
        cursor = connection.cursor()
        condition = ""
        if store:
            condition = "AND sc.store_id = %s" % store.id
        sql = """
            SELECT
                SUM(pr.quantity * sc.stock) AS availability
            FROM
                procedure_procedurerow pr
                JOIN store_storecause sc ON sc.id = pr.store_cause_id
            WHERE
                pr.product_id = %s
                %s
            GROUP BY
                pr.product_id
        """ % (self.id, condition)

        cursor.execute(sql)
        row = cursor.fetchone()
        if not row:
            return 0
        return row[0]


    def is_in_stock(self, store=None):
        """
        restituisce in funzione della presenza in magazzino del prodotto
        """
        stock_quantity = self.get_stock_quantity(store)
        if stock_quantity > 0:
            return True
        else:
            return False

    def get_stock_quantity(self, store=None, year=None):
        from store.models import Store

        if not store:
            store = Store.objects.default_store()

        current_year = date.today().year
        if year:
            current_year = year

        last_year = current_year - 1

        try:
            stock = self.get_inventory(store=store, year=last_year).stock
        except Exception, e:
            stock = 0

        for ddtrow in self.ddtrows.filter(ddt__date_d__year=current_year, store_cause__store=store):
            stock = stock + (ddtrow.quantity * ddtrow.store_cause.stock)
        return stock
    get_stock_quantity.short_description = _("Q. Disp.")


    def get_busy_quantity(self, store=None):
        from store.models import Store

        if not store:
            store = Store.objects.default_store()

        last_year = date.today().year - 1
        current_year = date.today().year

        try:
            busy = Inventory.objects.get(product=self, store=store, year=last_year).busy
        except Exception, e:
            busy = 0

        return busy


    def get_available_serialnumber(self, store=None, quantity=None):
        """
        restituisce seriali disponibili
        """
        if not self.serialnumber:
            return None
        cursor = connection.cursor()
        condition = ""
        if store:
            condition = "AND sc.store_id = %s" % store.id

        limit = ""
        if quantity:
            limit = "LIMIT %s" % quantity

        sql = """
            SELECT
                sn.id,
                sn.serial_number,
                SUM(sc.stock) AS availability
            FROM
                procedure_procedurerow pr
                JOIN store_storecause sc ON sc.id = pr.store_cause_id
                JOIN trace_serialnumber sn ON sn.procedurerow_id = pr.id
            WHERE
                pr.product_id = %s
                %s
            GROUP BY
                sn.serial_number
            HAVING
                availability = 1
            %s
        """ % (self.id, condition, limit)

        cursor.execute(sql)
        row = cursor.fetchall()
        if not row:
            return None
        return row

    def get_availability_serialnumber(self, store=None):
        available = self.get_available_serialnumber(store)
        if available:
            return len(available)
        else:
            return 0

###    def is_in_office_hours(self):
###        pass
###
    def get_inventory(self, store=None, year=None):
        from ddt.models import DdtRow
        from store.models import Store

        if not store:
            store = Store.objects.default_store()

        if not year:
            year = date.today().year - 1

        try:
            inventory = self.inventories.get(store=store, year=year)
        except Inventory.DoesNotExist:
            return None

        return inventory

    def get_year_inventory(self, num_year=4):
        l = []
        first_year = date.today().year - num_year
        last_year = date.today().year

        for y in reversed(range(first_year, last_year)):
            l.append(self.get_inventory(year=y))
        return l

    def get_sold_quantity(self, store=None, year=None):
        inventory = self.get_inventory(store=store, year=year)
        if inventory:
            return inventory.initial_sold_value
        return 0

    if "suspension" in settings.INSTALLED_APPS:
        def get_stops(self, date_s = date.today()):
            """
            restituisce tutte le sospensioni del Prodotto alla data specificata
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)
            return stop

    if "suspension" in settings.INSTALLED_APPS:
        def is_stopped(self, date_s = date.today()):
            """
            restituisce True o False se il Prodotto e' sospeso
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)[0]
            if stop: return True
            return False

    if "suspension" in settings.INSTALLED_APPS:
        def is_stopped_in(self, date_begin = date.today(), date_end = date.today()):
            """
            restituisce True o False se eiste già una sospensione del Prodotto nel periodo specificato.
            """
            stop = self.suspensions.filter(date_begin__lte=date_end, date_end__gte=date_begin)[0]
            if stop: return True
            return False

    if "suspension" in settings.INSTALLED_APPS:
        def is_forever_stopped(self, date_s = date.today()):
            """
            restituisce True o False se il Prodotto e' sospeso definitivamente:
            e' sospeso definitivamente se non esiste la data di fine
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__isnull=True)[0]
            if stop: return True
            return False

    if "suspension" in settings.INSTALLED_APPS:
        def set_start(self, date_end = date.today()):
            """
            crea una Sospensione con data inizio = NULL e data fine = giorno precedente a quello passato
            Il Prodotto è attivo dal giorno impostato
            """
            from suspension.models import Suspension
            suspension = Suspension(product=self, disable_cause=cause, date_begin=None, date_end=date_end+timedelta(days=-1))
            suspension.save()

    if "suspension" in settings.INSTALLED_APPS:
        def set_restart(self, date_end = date.today()):
            """
            attiva il Prodotto disattivato da una sospensione modificando la
            data di fine sospensione inserendo il giorno precedente.
            La data passata deve essere compresa nell'intervallo di sospensione.
            """
            suspensions = self.get_stops(date_end)
            for suspension in suspensions:
                suspension.date_end = date_end+timedelta(days=-1)
                suspension.save()

    if "suspension" in settings.INSTALLED_APPS:
        def set_stop(self, cause, date_begin = date.today(), date_end = None):
            """
            disattiva il prodotto creando una sospensione dalla data di inizio
            alla data di fine.
            Se la data di fine è NULL il Prodotto è sospeso definitivamente.
            Non verifica se c'è una sovrapposizione di sospensioni
            CONTROLLI:
            - La data di inizio deve essere <= alla data di fine
            - La data di inizio deve essere > della data di ultima consegna (get_last_date_delivery)
            """
            from suspension.models import Suspension
            suspension = Suspension(product=self, disable_cause=cause, date_begin=date_begin, date_end=date_end)
            suspension.save()

    if "procedure" in settings.INSTALLED_APPS:
        def set_series_procedure(self, date_s = date.today()):
            """
            Aggiunge il Prodotto alle Pratiche che hanno la Collana
            Il Prodotto deve necessariamente appartenere ad una Collana
            """
            parent = self.parent
            if parent:
                ## il Prodotto è un "Figlio" di una "Collana"
                rows = self.parent.rows.all()
                for row in rows:
                    procedure = row.procedure
                    ## creare una Riga Pratica e relativa Sospensione: la sospensione serve per determinare il giorno di consegna. Da FIXARE

    if "procedure" in settings.INSTALLED_APPS:
        def set_name(self):
            """
            modifica il nome del prodotto nel Righe Pratiche
            """
            pass

    if "procedure" in settings.INSTALLED_APPS:
        def set_short_description(self):
            """
            modifica la descrizione del Prodotto nelle Righe Pratiche
            """
            pass

    if "procedure" in settings.INSTALLED_APPS:
        def set_price(self):
            """
            modifica il prezzo del prodotto nelle Righe Pratiche
            """
            rows = self.rows.all()
            for row in rows:
                row.price = self.name
                row.save()

class ProductTranslation(TranslationModel):
    parent = models.ForeignKey(
        Product,
        db_index = True,
        related_name = 'translations',
        verbose_name = _('parent'),
    )
    name = models.CharField(
        max_length = 100,
        verbose_name = _('Name'),
    )
    short_description = models.CharField(
        blank=True, null=True,
        max_length=50,
        verbose_name=_("Short description"),
    )
    full_description = HTMLField(
        null = True, blank = True,
        verbose_name = _('Full Description'),
    )

    class Meta:
        verbose_name = _('Product Translation')
        verbose_name_plural = _('Product Translations')
        unique_together = (('parent','language',),)

    def __unicode__(self):
        return u'%s' % self.name[:15]


#####################################
# PERIODICITA' PRODOTTO             #
#####################################
class Periodicity(DateModel, UserModel, PeriodicityModel):
    product = models.OneToOneField(
        Product,
        related_name="periodicity",
        verbose_name=_("Product"),
    )

    def __unicode__(self):
        return u'%s' % self.product

    class Meta:
        verbose_name = _("Product Periodicity")
        verbose_name_plural = _("Product Periodicities")

#################################
# LISTINO                       #
#################################
PRICELIST_TYPE = (
    ('CP', 'COST PRICE'),
    ('RP', 'RESERVED PRICE'),
    ('SP', 'STREET PRICE'),
)

class PriceList(DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    ## Campi
    date_begin = models.DateField(
        db_index=True,
        verbose_name = _("date begin"),
        help_text = _('when this pricelist begins'),
    )
    date_end = models.DateField(
        db_index=True,
        verbose_name = _("date end"),
        help_text = _('when this pricelist ends'),
    )
    code = models.CharField(
        unique = True,
        max_length = 15,
        verbose_name = _("code"),
    )
    organizations = models.ManyToManyField(
        Organization,
        #related_name='price_lists',
        null = True, blank=True,
        verbose_name=_('organizations'),
        through = "OrganizationPriceList"
    )
    currency = models.ForeignKey (
        Currency,
        blank = True, null = True,
        related_name = "pricelists",
        db_index = True,
        verbose_name = _("Currency"),
    )
    ## campi modello
    pricelist_type = models.CharField (
        choices = PRICELIST_TYPE,
        max_length = 2,
        verbose_name = _("Price List Type"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        verbose_name=_("name"),
    )

    class Meta:
        verbose_name = _("price list")
        verbose_name_plural = _("price lists")
        ordering = ('ordering',)
        permissions = (
            ("view_pricelist"    , "Can view Price List"),
        )

    def __unicode__(self):
        return u"%s %s" % (self.code, self.name)

#############################
# RIGA LISTINO              #
#############################
class PriceListItem(DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    price_list = models.ForeignKey (
        PriceList,
        related_name='prices_lists',
        verbose_name=_("price list"),
    )
    product = models.ForeignKey (
        Product,
        blank=True,
        null=True,
        related_name='prices_lists',
        verbose_name=_("product"),
    )
    ## Campi
    date_begin = models.DateField(
        null=True,
        blank=True,
        verbose_name=_("date begin"),
    )
    date_end = models.DateField(
        null=True,
        blank=True,
        verbose_name=_("date end"),
    )
    price = models.DecimalField(
        max_digits=11,
        decimal_places=4,
        default=0,
        verbose_name=_("price"),
    )
    price_tax_included = models.BooleanField(
        verbose_name=_("price tax included"),
    )
    discount1 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("discount 1 %")
    )
    discount2 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("discount 2 %"),
    )
    recharge = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("recharge %"),
    )
    min_quantity = models.IntegerField(
        blank=True,
        null=True,
        verbose_name=_("min quantity"),
    )
    max_quantity = models.IntegerField(
        blank=True,
        null=True,
        verbose_name=_("max quantity"),
    )
    note = models.TextField (
        blank=True,
        null=True,
        verbose_name=_("note"),
    )

    class Meta:
        verbose_name = _("product price list item")
        verbose_name_plural = _("price list items")
        ordering = ('ordering',)
        permissions = (
            ("view_pricelistitem"    , "Can view Price List Item"),
        )

    def __unicode__(self):
        return u"%(ls_name)s - %(name)s (%(date_begin)s - %(date_end)s): %(price)s" % {'ls_name': self.price_list.name, 'name': self.price_list.name, 'date_begin': self.date_begin, 'date_end': self.date_end, 'price': self.price}

    def get_date_begin(self):
        return self.date_begin or self.price_list.date_begin
    get_date_begin.short_description = _("Date Begin")

    def get_date_end(self):
        return self.date_end or self.price_list.date_end
    get_date_end.short_description = _("Date End")

    def get_price_vat_included(self):
        if self.price_tax_included:
            return self.price
        else:
            return self.price * (1 + (self.product.vat.aliquot_perc)/100)
    price_vat_included = property(get_price_vat_included)

    def get_price_vat_excluded(self):
        if self.price_tax_included:
            return self.price / (1 + (self.product.vat.aliquot_perc)/100)
        else:
            return self.price
    price_vat_excluded = property(get_price_vat_excluded)

    def get_last_price(self):
        last_price = ((self.price_vat_excluded * (1 - self.discount1)) * (1 - self.discount1)) * (1 + self.recharge/100)
        return last_price #.quantize(decimal.Decimal('.0001'), rounding='ROUND_UP')
    last_price = property(get_last_price)



#############################
# LISTINO SCONTI            #
#############################
class CustomPriceList(DateModel, UserModel, StatusModel, OrderedModel):
    class Meta:
        abstract = True

    ## ForeignKey
    price_list = models.ForeignKey(
        PriceList,
        blank=True, null=True,
        related_name='%(class)s_price_lists',
        verbose_name=_("Price List"),
    )
    ## Campi
    discount1 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("Discount 1 %"),
    )
    discount2 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("Discount 2 %"),
    )


class OrganizationPriceList(CustomPriceList):
    organization = models.ForeignKey(
        Organization,
        related_name='price_lists',
        verbose_name=_("Organization"),
    )

    class Meta:
        verbose_name = _("Organization Price List")
        verbose_name_plural = _("Organization Prices Lists")
        ordering = ('ordering',)
        unique_together = (("price_list", "organization",),)

    def __unicode__(self):
        price_list_name = ""
        if self.price_list:
            price_list_name = self.price_list.name
        return u"%s %s%% + %s%%" % (price_list_name,
                                  number_format(self.discount1, 1),
                                  number_format(self.discount2, 1))


class OrganizationTypePriceList(CustomPriceList):
    organization_type = models.ForeignKey(
        OrganizationType,
        related_name='price_lists',
        verbose_name=_("Organization Type"),
    )

    class Meta:
        verbose_name = _("Organization Type Price List")
        verbose_name_plural = _("Organization Type Prices Lists")
        ordering = ('ordering',)
        unique_together = (("price_list", "organization_type",),)

    def __unicode__(self):
        price_list_name = ""
        if self.price_list:
            price_list_name = self.price_list.name
        return u"%s %s%% + %s%%" % (price_list_name,
                                  number_format(self.discount1, 1),
                                  number_format(self.discount2, 1))


#############################
# EXTRA CODICI              #
#############################
class ProductCode(DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    organization = models.ForeignKey (
        Organization,
        related_name='product_codes',
        verbose_name=_("organization"),
    )
    product = models.ForeignKey (
        Product,
        related_name='other_codes',
        verbose_name=_("product"),
    )
    ## Campi
    reserved_code = models.CharField(
        max_length=20,
        db_index=True,
        blank=True, null=True,
        verbose_name=_("Reserved Code"),
    )
    draw_code  = models.CharField(
        max_length=20,
        db_index=True,
        blank=True, null=True,
        verbose_name=_("Draw Code"),
    )
    revision = models.CharField(
        max_length=20,
        db_index=True,
        blank=True, null=True,
        verbose_name=_("Revision"),
    )
    brand = models.CharField(
        max_length=100,
        db_index=True,
        blank=True, null=True,
        verbose_name=_("Brand"),
    )

    class Meta:
        verbose_name = _('Product code')
        verbose_name_plural = _('Product codes')
        unique_together = (('organization','product'),)

    def __unicode__(self):
        return u'%s %s %s' % (self.reserved_code, self.brand, self.organization)

#############################
# POSIZIONE MAGAZZINO       #
#############################
class Position(DateModel, UserModel, StatusModel, OrderedModel):
    product = models.ForeignKey(
        Product,
        related_name = "positions",
        verbose_name = _("Product")
    )
    store = models.ForeignKey(
        Store,
        related_name = "positions",
        verbose_name = _("Store")
    )
    position = models.CharField(
        max_length = 10,
        db_index = True,
        verbose_name = _("Position"),
    )

    class Meta:
        verbose_name = _('Position')
        verbose_name_plural = _('Positions')

    def __unicode__(self):
        return u'%s' % self.position

#############################
# INVENTARIO                #
#############################
class Inventory(DateModel, UserModel, StatusModel):
    ## ForeignKey
    product = models.ForeignKey (
        Product,
        related_name = "inventories",
        db_index = True,
        verbose_name = _("Product"),
    )
    store = models.ForeignKey (
        Store,
        related_name = "Invetories",
        db_index = True,
        verbose_name = _("store"),
    )
    ## Campi
    year = models.IntegerField (
        db_index = True,
        verbose_name = _("year"),
    )
    stock = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Stock Quantity"),
    )
    busy = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Busy Quantity"),
    )
    initial_sold_value = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Initial Sold Value"),
    )

    class Meta:
        verbose_name = _('Inventory')
        verbose_name_plural = _('Inventories')
        unique_together = (('product', 'store', 'year'),)

    def __unicode__(self):
        return u'%s' % self.product

class StoreMovement(DateModel, UserModel, StatusModel):
    ## ForeignKey
    product = models.ForeignKey (
        Product,
        related_name = "movements",
        db_index = True,
        verbose_name = _("Product"),
    )
    store = models.ForeignKey (
        Store,
        related_name = "movements",
        db_index = True,
        verbose_name = _("store"),
    )
    ## Campi
    year = models.IntegerField (
        db_index = True,
        verbose_name = _("year"),
    )
    quantity = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Quantity"),
    )
    value = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Value"),
    )

    class Meta:
        verbose_name = _('Store Movement')
        verbose_name_plural = _('Store Movements')
        unique_together = (('product', 'store', 'year'),)

    def __unicode__(self):
        return u'%s' % self.product

class Settings(JSettings):
    pass