from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.db.models import Q

from jumbo.admin.sites import root_site
from jumbo.forms.models import AdminModelForm
from jumbo.admin.options import ExtendibleModelAdmin
from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete import AutocompleteModelChoiceField, AutocompleteModelMultiChoiceField, AutocompleteInlineCharField
from jumbo.admin.reverseadmin import ReverseModelAdmin

from shop.models import TermOfSale, Cart, CartItem, BuyoutMail, ShopProduct, ShopCategory, Discount, ShopProductTranslation
from shop.models import ShopCategoryTranslation, ShopCategoryImage, ShopCategoryFile, Reservation
from shop.models import ShopProductFile, ShopProductImage

from product.models import Product

# IMPORTO I SEGNALI DELLO SHOP
from shop import signals

class ShopCategoryAdminForm(AdminModelForm):
    class Meta:
        model = ShopCategory

    def clean_parent(self):
        parent = self.cleaned_data['parent']
        slug = self.cleaned_data.get('slug', '')
        if parent and slug:
            if parent.slug == slug:
                raise ValidationError(_("You must not save a category in itself!"))

            for p in parent._recurse_for_parents(parent):
                if slug == p.slug:
                    raise ValidationError(_("You must not save a category in itself!"))

        return parent


class ShopCategoryTranslationInline(admin.StackedInline):
    model = ShopCategoryTranslation
    extra = 0
    fieldsets = (
        (_('detail'), {
            'fields': (('language', 'name'), 'description'),
        }),
    )

class ShopCategoryImageInline(admin.TabularInline):
    model = ShopCategoryImage
    extra = 0
    fieldsets = (
        (_('detail'), {
            'fields': (('status', 'picture', 'big_picture', 'ordering'),),
            'classes': ('collapse',)
        }),
    )

class ShopCategoryFileInline(admin.TabularInline):
    model = ShopCategoryFile
    extra = 0
    fieldsets = (
        (_('detail'), {
            'fields': (('status', 'title', 'name'), ('file', 'ordering'),),
            'classes': ('collapse',)
        }),
    )

class ShopCategoryAdmin(ExtendibleModelAdmin):
    #class Media:
    #    css = {"all": ("product/css/admin_product.css",)}

    form = ShopCategoryAdminForm
    list_display = ('get_status_icon', 'ordering', 'tree_name', 'slug',
                    'get_edit_icon', 'get_delete_icon')
    inlines = [ShopCategoryImageInline, ShopCategoryFileInline,
               ShopCategoryTranslationInline]
    filter_horizontal = ('related_categories',)
    ordering = ('tree_id', 'level', 'ordering')
    search_fields = ('name',)

    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'parent', ('name', 'slug'), 'ordering')
        }),
        (_('Description & Related Categories'), {
            'fields': ('code', 'description', 'long_description', 'related_categories'),
            'classes': ('collapse',)
        })
    )

    list_editable = ['ordering',]

class ShopProductTranslationInline(admin.StackedInline):
    model = ShopProductTranslation
    extra = 1
    fields = ('language', 'name', 'print_info', 'popup_info')


class ShopProductFileInline(admin.TabularInline):
    model     = ShopProductFile
    extra     = 0
    fields    =  ('language', 'code', 'name', 'file')


class ShopProductImageInline(admin.StackedInline):
    model     = ShopProductImage
    extra     = 0
    fields    = ('big_picture', 'small_picture', 'title', 'description', 'ordering')


### SEARCH
class ShopProductSearchForm(forms.ModelForm):
    class Meta:
        model = ShopProduct
        #fields = ('code', 'name', 'production_code')
        display_fields = (
            'status',
            'product__name__icontains',
            ('product__code__icontains', 'product__code__startswith'),
            ('shop_categories__in',),
        )

    def __init__(self, *args, **kws):
        super(ShopProductSearchForm, self).__init__(*args, **kws)
        STATUS = (('', '-'*9), ('1', _('Active')), ('0', _('Disactive')),)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), choices=STATUS, required=False)
        self.fields['shop_categories__in'] = forms.ModelMultipleChoiceField(
            label=_("Shop Categories"), queryset=ShopCategory.objects.all()
        )
        self.fields['product__name__icontains'] = forms.CharField(label=_('Name'), required=False)
        self.fields['product__code__icontains'] = forms.CharField(label=_('Code'), required=False)
        self.fields['product__code__startswith'] = forms.CharField(label=_('Code Starts With'), required=False)


class ShopProductAdminForm(AdminModelForm):
    class Meta:
        model = ShopProduct

    def __init__(self, *args, **kwargs):
        super(ShopProductAdminForm, self).__init__(*args, **kwargs)
        self.fields['product'] = AutocompleteModelChoiceField(
            'shop_get_products', label=_("Product")
        )
        self.fields['related_shopproducts'] = AutocompleteModelMultiChoiceField(
            'shop_related_shopproducts', label=_("Related Products"), required=False
        )


class ShopProductAdmin(ExtendibleModelAdmin):
    form              = ShopProductAdminForm
    search_fields     = ('product__name', 'product__code',)
    search_form       = ShopProductSearchForm
    list_display      = ('get_status_icon', 'product', 'name', 'ordering', 'get_list_shop_categories', 'min_quantity',
                         'max_quantity', 'mult_quantity', 'availability', 'get_edit_icon', 'get_delete_icon')
    filter_horizontal = ('shop_categories', 'visible_by')
    list_editable = ['availability', 'ordering',]
    inlines       = (ShopProductImageInline, ShopProductFileInline, ShopProductTranslationInline,)
    #inline_type = 'stacked'

    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'status', 'product', 'name', 'shop_categories', 'short_description',
                'full_description', 'related_shopproducts', 'visible_by',
                ('min_quantity', 'max_quantity', 'mult_quantity'),
                'fill_basket', 'print_info', 'popup_info', 'icon'
            )
        }),
    )

class DiscountAdmin(ExtendibleModelAdmin):
    list_display = (
        'get_status_icon', 'ordering', 'code', 'description', 'qty', 'startDate', 'endDate', 'get_edit_icon', 'get_delete_icon'
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify', 'ordering'),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'status', 'code', 'description', 'long_description', 'qty',
                ('amount', 'percentage', 'extra_discount'),
                ('allowedUses', 'numUses'),
                'minOrder',
                'valid_products',
                'valid_categories',
                ('startDate', 'endDate',),
                'allValid'
            )
        }),
    )
    list_editable = ['ordering',]

class ReservationAdmin(ExtendibleModelAdmin):
    list_display = ('creator', 'product', 'discount', 'reserved_at', 'reservation_expire', 'get_edit_icon', 'get_delete_icon')
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'last_modifier'),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('product', 'discount', 'reserved_at', 'reservation_expire',)
        }),
    )

class CartAdmin(ExtendibleModelAdmin):
    list_display = ('creator', 'organization', 'customer', 'agent', 'get_edit_icon', 'get_delete_icon')

class CartItemAdmin(ExtendibleModelAdmin):
    list_display = ('creator', 'product', 'get_edit_icon', 'get_delete_icon')
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify'),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('product',)
        }),
    )


class TermOfSaleAdmin(ExtendibleModelAdmin):
    #class Media:
    #    css = {"all": ("product/css/admin_product.css",)}

    list_display = ('status', 'ordering', 'title', 'text', 'required', 'get_edit_icon')
    list_editable = ('ordering',)

admin.site.register(Cart, CartAdmin)
admin.site.register(CartItem, CartItemAdmin)
admin.site.register(ShopProduct, ShopProductAdmin)
admin.site.register(ShopCategory, ShopCategoryAdmin)
admin.site.register(Discount, DiscountAdmin)
admin.site.register(Reservation, ReservationAdmin)
admin.site.register(TermOfSale, TermOfSaleAdmin)

root_site.register(Cart)
root_site.register(CartItem)
root_site.register(ShopProduct)
root_site.register(ShopCategory)
root_site.register(Discount)

########################################
# AUTOCOMPLETE REGISTER                #
########################################
def get_autocomplete_product(request, query):
    results = []
    try:
        prod_exact = Product.objects.get(code=query)
        results.append([prod_exact.id, "%s - %s" % (prod_exact.code, prod_exact.name)])
    except:
        pass

    q1 = Q(name__istartswith=query)
    q2 = Q(code__istartswith=query)
    for other_product in Product.objects.exclude(code=query).filter(status=1).filter(q1|q2)[:10]:
        results.append([other_product.id, "%s - %s" % (other_product.code, other_product.name)])
    return results

def get_shop_product(request, query):
    return ShopProduct.objects.filter(status=1)

def get_shop_category(request, query):
    return ShopCategory.objects.filter(status=1)

autocomplete.register('shop_get_products', model = Product, views=get_autocomplete_product, limit=10)
autocomplete.register('shop_related_shopproducts', model = ShopProduct, views=get_shop_product, search_fields = ('product__name__istartswith','product__code__istartswith'), limit=10)
autocomplete.register('shop_shop_category', model = ShopCategory, views=get_shop_category, search_fields = ('name',), limit=10)
