# -*- coding: utf-8 -*-

import datetime
from decimal import Decimal

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from django.conf import settings
from django.db.models import Q, Sum
from django.core import urlresolvers
from django.utils.translation import get_language
from django.contrib import messages
from django.contrib.auth.models import User
from django.core.mail import EmailMultiAlternatives
from django.contrib.sites.models import Site, RequestSite

from jumbo.db.fields import HTMLField
from jumbo.db import models as jumbo_models
from jumbo.middleware.thread_local import get_request, get_current_user
from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.apps.i18n.models import TranslationModel, I18NModel
from jumbo.utils.functions import get_settings, slugify
from jumbo.shortcuts import render_to_string

from product.models import Product, Vat
from organization.models import Contact, Organization, Agent, Vat
from procedure.models import Procedure, ProcedureRow
from store.models import StoreCause
import mptt

##################################
# CONDIZIONI DI VENDITA          #
##################################

class TermOfSale(StatusModel, OrderedModel):
    members = models.ManyToManyField(
        Organization,
        through='TermOfSaleOrganization'
    )
    title = models.CharField(
        blank = True, null = True,
        max_length = 100,
        db_index=True,
        verbose_name = _("Title"),
    )
    text = models.TextField(
        blank = True, null = True,
        verbose_name = _("Text"),
    )
    required_text = models.CharField(
        blank = True, null = True,
        max_length = 200,
        db_index=True,
        verbose_name = _("Required Text"),
    )
    required = models.BooleanField(
        default = True,
        verbose_name = _("Required"),
    )
    slug = models.SlugField(
        blank = True, null = True,
        max_length = 100,
        db_index=True,
        verbose_name = _("Slug"),
    )
    def save(self):
        if not self.slug:
            self.slug = slugify(self.title, instance=self)

        super(TermOfSale, self).save()


class TermOfSaleOrganization(StatusModel):
    organization = models.ForeignKey(
        Organization,
        db_index = True,
        related_name = 'terms_of_sale_organizations',
        verbose_name = _('organizations'),
    )
    term_of_sale = models.ForeignKey(
        TermOfSale,
        db_index = True,
        related_name = 'organizations_terms_of_sale',
        verbose_name = _('terms_of_sale'),
    )
    accepted = models.BooleanField(
        default = False,
        verbose_name = _("Accepted"),
    )
    acceptance_date = models.DateField(
        auto_now_add = True,
        verbose_name = _("Acceptance Date"),
    )


##################################
# CATEGORIA CATALOGO             #
##################################
class ShopCategory(DateModel, UserModel, StatusModel, OrderedModel, I18NModel):
    name = models.CharField(
        max_length = 200,
        db_index=True,
        verbose_name = _("Name"),
    )
    code = models.CharField(
        blank = True, null = True,
        max_length = 20,
        db_index=True,
        verbose_name = _("Code"),
    )
    slug = models.SlugField(
        blank = True, null = True,
        max_length = 200,
        db_index=True,
        verbose_name = _("Slug"),
        help_text = _("Used for URLs, auto-generated from name if blank"),
    )
    parent = models.ForeignKey(
        'self',
        blank = True, null = True,
        related_name = 'children',
    )
    description = HTMLField(
        blank = True, null = True,
        verbose_name = _("Description"),
        help_text = _("Description"),
    )
    long_description = HTMLField(
        blank = True, null = True,
        verbose_name = _("Long description"),
        help_text = _("Long description"),
    )
    related_categories = models.ManyToManyField(
        'self',
        blank = True, null = True,
        symmetrical = False,
        related_name = '%(class)s_related_shop_categories',
        verbose_name = _('Related Shop Categories'),
    )

    def products_by_ordering(self):
        ap = self.active_products()
        return ap.order_by('ordering')

    def active_products(self, include_children=False, **kwargs):
        #if not include_children:
        qry = self.shop_products.all()
        #else:
        #    cats = self.get_all_children(include_self=True)
        #    qry = Product.objects.filter(category__in=cats)
        return qry.filter(status=True)

    def get_absolute_url(self):
        parents = self.get_ancestors()
        slug_list = [cat.slug for cat in parents]
        if slug_list:
            slug_list = "/".join(slug_list) + "/"
        else:
            slug_list = ""
        return urlresolvers.reverse('shop_shopcategory_detail',
            kwargs={'parent_slugs' : slug_list, 'slug' : self.slug})

    def recursive_get_code(self):
        categ = self
        while categ:
            if categ.code:
                return categ.code
            categ = categ.parent
        return ""

    def get_separator(self):
        return ">>>"

    def get_tree_name(self):
        return self.__unicode__()
    get_tree_name.short_description = "Tree Name"
    tree_name = property(get_tree_name)

    def get_children(self):
        return self.get_descendants().order_by('ordering')
    get_children.short_description = "Children"
    children = property(get_children)

    def __unicode__(self):
        name_list = [ c.name for c in self.get_ancestors() ]
        name_list.append(self.name)
        return self.get_separator().join(name_list)

    def save(self, force_insert=False, force_update=False):
        if not self.slug:
            self.slug = slugify(self.name, instance=self)
        super(ShopCategory, self).save(force_insert=force_insert, force_update=force_update)

    def _get_image(self, idx):
        images = self.images.order_by('ordering')
        try:
            return images[idx]
        except IndexError:
            return None

    def get_default_small_image(self):
        return self._get_image(0).picture
    default_small_image = property(get_default_small_image)

    def get_default_big_image(self):
        return self._get_image(0).big_picture
    default_big_image = property(get_default_big_image)

    def get_alternative_small_image(self):
        return self._get_image(1).picture
    alternative_small_image = property(get_alternative_small_image)

    def get_alternative_big_image(self):
        return self._get_image(1).big_picture
    alternative_big_image = property(get_alternative_big_image)


    def recursive_default_big_image(self):
        i = self.get_default_big_image()
        if not i:
            i = self.parent.recursive_default_big_image()
        return i


    def get_first_product(self, small = False):
        products = self.active_products().order_by('ordering')[:1]
        if products:
            return products[0]
        return None
    first_product = property(get_first_product)


    class Meta:
        ordering = ['tree_id', 'level', 'ordering']
        verbose_name = _("Shop Category")
        verbose_name_plural = _("Shop Categories")
        permissions = (
            ("view_shopcategory"    , "Can view Shop Category"),
        )
mptt.register(ShopCategory, order_insertion_by=['ordering', 'name',])


class ShopCategoryTranslation(TranslationModel):
    parent = models.ForeignKey(
        ShopCategory,
        db_index = True,
        related_name = 'translations',
        verbose_name = _('parent'),
    )
    name = models.CharField(
        max_length = 100,
        verbose_name = _('Name'),
    )
    description = HTMLField(
        null = True, blank = True,
        verbose_name = _('Description'),
    )

    class Meta:
        verbose_name = _('Shop Category Translation')
        verbose_name_plural = _('Shop Category Translations')
        unique_together = (('parent','language',),)

    def __unicode__(self):
        return u'%s' % self.name[:15]


class ShopCategoryImage(DateModel, UserModel, StatusModel, OrderedModel):
    category = models.ForeignKey(
        ShopCategory,
        related_name = "images",
        verbose_name = _("Shop Category"),
    )
    picture = models.ImageField(
        max_length = 200,
        upload_to = get_settings("shop.settings", "SHOP_CATEGORY_IMAGES"),
        verbose_name=_('Small Picture'),
        blank = True, null = True
    )
    big_picture = models.ImageField(
        max_length = 200,
        upload_to = get_settings("shop.settings", "SHOP_CATEGORY_IMAGES"),
        verbose_name=_('Big Picture'),
        blank = True, null = True
    )

    def __unicode__(self):
        return u'%s' % self.category

    class Meta:
        verbose_name = _('Shop Category Image')
        verbose_name_plural = _('Shop Category Images')


class ShopCategoryFile(DateModel, UserModel, StatusModel, OrderedModel):
    category = models.ForeignKey(
        ShopCategory,
        related_name = "files",
        verbose_name = _("Shop Category"),
    )
    title = models.CharField(
        max_length = 100,
        db_index = True,
        verbose_name = _('Title'),
    )
    name = models.CharField(
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )
    file = models.FileField(
        max_length = 200,
        upload_to = get_settings("shop.settings", "SHOP_CATEGORY_FILES"),
        verbose_name=_('File'),
    )

    def __unicode__(self):
        return u'%s' % self.title

    class Meta:
        verbose_name = _('Shop Category File')
        verbose_name_plural = _('Shop Category Files')
        unique_together = (('category', 'title','name',),)
        ordering = ('category', 'ordering', 'title', 'name')

class ShopProductManager(models.Manager):
    def get_query_set(self):
        qs = super(ShopProductManager, self).get_query_set()
        #request = get_request()
        user = get_current_user() #request.user
        if not user.is_superuser and user.is_authenticated():
            qs = qs.filter(Q(visible_by=None)|Q(visible_by=user))
        ## TODO: permettere al responsabile di vederli tutti

        return qs

class ShopProduct(DateModel, UserModel, StatusModel, OrderedModel, I18NModel):
    ## Foreignkey
    product = models.OneToOneField(
        Product,
        db_index = True,
        related_name = 'shop_info',
        verbose_name = _("product"),
    )
    shop_categories = models.ManyToManyField(
        ShopCategory,
        blank = True, null = True,
        related_name="shop_products",
        verbose_name=_("Shop Categories"),
    )
    related_shopproducts = models.ManyToManyField(
        'self',
        blank = True, null = True,
        symmetrical = False,
        related_name = '%(class)s_related_shop_products',
        verbose_name = _('Related Shop Products'),
    )
    visible_by = models.ManyToManyField(
        User,
        blank = True, null = True,
        related_name = 'visible_shop_products',
        verbose_name = _('Visible by'),
    )
    ## Campi Modello
    name = models.CharField(
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("Shop Product Name"),
        help_text = _("If set, overwrite the product name.")
    )
    short_description = models.CharField(
        blank=True, null=True,
        max_length=150,
        verbose_name=_("Short description"),
    )
    full_description = HTMLField(
        blank=True, null=True,
        verbose_name=_("Full description"),
    )
    min_quantity = models.IntegerField(
        default = 1,
        db_index = True,
        verbose_name = _("min quantity"),
    )
    max_quantity = models.IntegerField(
        default = 0,
        db_index = True,
        verbose_name = _("max quantity"),
    )
    mult_quantity = models.IntegerField(
        default = 1,
        db_index = True,
        verbose_name = _("mult quantity"),
    )
    fill_basket = models.BooleanField(
        db_index = True,
        verbose_name = _("fill basket"),
    )
    extra_form_name = models.CharField(
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("extra form name"),
        help_text = _("Name of the form class for this product.")
    )
    office_time = models.BooleanField(
        default = False,
        db_index = True,
        verbose_name = _("office time"),
    )
    print_info = jumbo_models.HTMLField(
        null = True, blank = True,
        verbose_name = _("print info"),
    )
    popup_info = jumbo_models.HTMLField(
        null = True, blank = True,
        verbose_name = _("popup info"),
    )
    icon = models.ImageField(
        upload_to = "uploads/products/images/",
        blank = True, null = True,
        verbose_name = _("icon"),
    )
    availability = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Available Quantity"),
    )
    objects = ShopProductManager()


    class Meta:
        verbose_name = _('Shop Product')
        verbose_name_plural = _('Shop Products')
        ordering = ('ordering',)
        permissions = (
            ("view_shopproduct"    , "Can view Shop Product"),
        )


    def __unicode__(self):
        return u"%s - %s" % (self.product.code, self.product.name)


    def get_first_image(self, small=False):
        images =self.images.order_by('ordering')[:1]
        if images:
            #if small:
            #    return images[0].small_picture
            return images[0]
        return None
    first_image = property(get_first_image)


    def get_current_language_files(self):
        lang = get_language()[0:2]
        return self.files.filter(language__code=lang)


    def _get_ecm_name(self):
        return self.product.description or self.product.name
    ecm_name = property(_get_ecm_name)


    def render_widget(self):
        if self.min_quantity == 1 and self.max_quantity == 1:
            return """<input class="inputDx" type="checkbox" value="1" size="1" maxlength="2" name="prodotto_%s"/>""" % self.id
        else:
            return """<input class="inputDx" type="text" onfocus="this.value = '';" value="0" size="1" maxlength="2" name="prodotto_%s"/>""" % self.id


    def get_list_shop_categories(self):
        c = [ x.name for x in self.shop_categories.all() ]
        return ', '.join(c)
    get_list_shop_categories.short_description = _("Shop Categories")


    def get_booked_quantity(self):
        total_booked_quantity = CartItem.objects.filter(product=self).aggregate(Sum('quantity'))
        return total_booked_quantity['quantity__sum'] or 0


    def get_reserved_quantity(self):
        reserved_quantity = Reservation.objects.filter(product=self).aggregate(Sum('quantity'))
        return reserved_quantity['quantity__sum'] or 0


class ShopProductFile(StatusModel, TranslationModel):
    product = models.ForeignKey(
        ShopProduct,
        db_index=True,
        related_name='files',
        verbose_name=_('parent'),
    )
    code = models.CharField(
        max_length=20,
        db_index=True,
        verbose_name=_('Code'),
    )
    name = models.CharField(
        max_length=50,
        db_index=True,
        verbose_name=_('Name'),
    )
    file = models.FileField(
        upload_to='uploads/products/docs/',
        db_index=True,
        verbose_name=_('File'),
    )

    class Meta:
        verbose_name = _('Product file')
        verbose_name_plural = _('Product files')
        unique_together = (('product','language','code',),)

    def __unicode__(self):
        return u'%(product)s %(name)s' % {'name': self.name, 'product': self.product}


class ShopProductImage(DateModel, UserModel, StatusModel, OrderedModel):
    product = models.ForeignKey(
        ShopProduct,
        related_name = "images",
        verbose_name = _("Product"),
    )
    big_picture = models.ImageField(
        max_length = 200,
        upload_to = get_settings("shop.settings", "SHOP_PRODUCT_IMAGES_BIG"),
        verbose_name=_('Big Picture'),
    )
    small_picture = models.ImageField(
        max_length = 200,
        upload_to = get_settings("shop.settings", "SHOP_PRODUCT_IMAGES_SMALL"),
        verbose_name=_('Small Picture'),
        blank = True, null = True
    )
    title = models.CharField(
        max_length = 100,
        db_index = True,
        blank = True, null = True,
        verbose_name = _("Title")
    )
    description = HTMLField(
        null = True, blank = True,
        verbose_name = _('Description'),
    )

    def __unicode__(self):
        return u'%s - %s' % (self.product, self.title)

    class Meta:
        verbose_name = _('Product Image')
        verbose_name_plural = _('Product Images')
        ordering = ('ordering', 'title')


class ShopProductTranslation(TranslationModel):
    parent = models.ForeignKey(
        ShopProduct,
        db_index = True,
        related_name = 'translations',
        verbose_name = _('parent'),
    )
    name = models.CharField(
        max_length = 100,
        null = True, blank = True,
        verbose_name = _('Shop Product Name'),
    )
    short_description = models.CharField(
        blank=True, null=True,
        max_length=50,
        verbose_name=_("Short description"),
    )
    full_description = HTMLField(
        null = True, blank = True,
        verbose_name = _('Full Description'),
    )
    print_info = jumbo_models.HTMLField(
        null = True, blank = True,
        verbose_name = _("print info"),
    )
    popup_info = jumbo_models.HTMLField(
        null = True, blank = True,
        verbose_name = _("popup info"),
    )

    class Meta:
        verbose_name = _('Shop Product Translation')
        verbose_name_plural = _('Shop Product Translations')
        unique_together = (('parent','language',),)

    def __unicode__(self):
        return u'%s' % self.parent.product.name[:15]


class ActiveDiscountManager(models.Manager):
    def get_query_set(self):
        """Get the current 'sale' discount."""
        qs = super(ActiveDiscountManager, self).get_query_set()

        today = datetime.date.today()
        return qs.filter(
            status=True,
            startDate__lte=today,
            endDate__gt=today
        )


class Discount(DateModel, UserModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=20,
        unique=True,
        verbose_name=_("Discount Code"),
        help_text=_("Coupon Code"),
    )
    valid_products = models.ManyToManyField(
        ShopProduct,
        blank=True, null=True,
        verbose_name=_("Valid Products"),
        related_name="discounts",
    )
    valid_categories = models.ManyToManyField(
        ShopCategory,
        blank=True, null=True,
        verbose_name=_("Valid Categories"),
    )
    description = HTMLField(
        verbose_name=_("Short Description"),
    )
    long_description = HTMLField(
        verbose_name=_("Long Description"),
        blank=True, null=True,
    )
    qty = models.IntegerField(
        blank=True, null=True,
        verbose_name=_("Quantity"),
        help_text=_("Number of items that have the discount."),
    )
    amount = models.DecimalField(
        verbose_name=_("Discount Amount"),
        decimal_places=2,
        max_digits=8,
        blank=True, null=True,
        help_text=_("Enter absolute discount amount."),
    )
    percentage = models.DecimalField(
        decimal_places=2,
        max_digits=5,
        blank=True, null=True,
        verbose_name=_("Discount Percentage"),
        help_text=_("Enter absolute percentage. Percents are given in whole numbers, and can be up to 100%."),
    )
    extra_discount = HTMLField(
        blank=True, null=True,
        verbose_name=_("Extra Discount"),
        help_text=_("Use this field to show an extra particular discount like '-20% of your reserved price'."),
    )
    allowedUses = models.IntegerField(
        blank=True, null=True,
        verbose_name=_("Number of allowed uses"),
        help_text=_('Set this to a number greater than 0 to have the discount expire after that many uses.'),
    )
    numUses = models.IntegerField(
        blank=True, null=True,
        verbose_name=_("Number of times already used"),
    )
    minOrder = models.DecimalField(
        decimal_places=2,
        max_digits=8,
        blank=True, null=True,
        verbose_name=_("Minimum order value"),
    )
    startDate = models.DateField(
        verbose_name=_("Start Date"),
    )
    endDate = models.DateField(
        verbose_name=_("End Date"),
    )
    allValid = models.BooleanField(
        verbose_name=_("All products?"), default=False,
        help_text=_('Apply this discount to all discountable products? If this is false you must select products below in the "Valid Products" section.'),
    )

    objects = models.Manager()
    active = ActiveDiscountManager()

    def __init__(self, *args, **kwargs):
        self._calculated = False
        super(Discount, self).__init__(*args, **kwargs)

    def __unicode__(self):
        return self.description

    def is_bad_discount(self, quantity=None, cart=None, contact=None):
        """
        Make sure this discount still has available uses and is in the current date range.
        If a cart has been populated, validate that it does apply to the products we have selected.
        If this is a "FREECHEAP" discount, then error if the cheapest shipping hasn't been chosen.
        """
        if not self.status:
            return (True, ugettext('This coupon is disabled.'))
        if self.startDate > datetime.date.today():
            return (True, ugettext('This coupon is not active yet.'))
        if self.endDate < datetime.date.today():
            return (True, ugettext('This coupon has expired.'))
        if self.numUses and self.allowedUses and self.allowedUses > 0 and self.numUses >= self.allowedUses:
            return (True, ugettext('This discount has exceeded the number of allowed uses.'))
        if not self.qty:
            return (True, ugettext("Requested discount has no more items."))

        if quantity and self.qty:
            if type(quantity) == type('') and not quantity.isdigit():
                return (True, ugettext('Insert a valid quantity.'))

            if int(quantity) > self.qty:
                return (True, ugettext('Quantity is greater than the available discount quantity.'))

        #if cart:
            #minOrder = self.minOrder or 0
            #if cart.total < minOrder:
            #    return (True, ugettext('This discount only applies to orders of at least %s.' % moneyfmt(minOrder)))
            #
            #if not (self.allValid or (len(self._valid_products(cart.items.all())) > 0)):
            #    return (True, ugettext('This discount cannot be applied to the products in your cart.'))

        # last minute check to make sure discount is valid
        success = {'is_bad' : False, 'message': ugettext('Valid.')}
        return (success['is_bad'], success['message'])

    def _valid_products_in_categories(self):
        slugs = set()
        for cat in ShopCategory.objects.filter(id__in=self.valid_categories.values_list('id', flat=True)):
            slugs.update([p.slug for p in cat.active_products()])
        return slugs

    def _percentage_text(self):
        """Get the human readable form of the sale percentage."""
        return "%d%%" % self.percentage

    percentage_text = property(_percentage_text)


    class Meta:
        verbose_name = _("Discount")
        verbose_name_plural = _("Discounts")
        ordering = ['ordering',]


class CartManager(models.Manager):
#    def quantity(self, user):
#        return self.get_query_set().filter(contact__user=user).aggregate(quantity=Sum('quantity'))['quantity']
    def from_request(self, request):
        cart = None
        contact = None
        agent = None
        ## verifico che l'utente abbia un CONTATTO associato
        try:
            contact = Contact.objects.get(user=request.user)
        except Contact.DoesNotExist:
            pass

        ## verifico che l'utente abbia un AGENTE associato
        if not contact:
            try:
                agent = Agent.objects.get(user=request.user)
            except Agent.DoesNotExist:
                pass

            if not agent:
                raise Exception("You don't have any Contact or Agent associated with your User.")

        if contact:
            try:
                cart = Cart.objects.get(customer=contact)
            except Cart.DoesNotExist:
                cart = Cart(customer=contact)
                cart.save()
        elif agent:
            try:
                cart = Cart.objects.get(agent=agent)
            except Cart.DoesNotExist:
                cart = Cart(agent=agent)
                cart.save()

        return cart

class Cart(DateModel, UserModel, StatusModel):
    organization = models.OneToOneField(
        Organization,
        blank=True, null=True,
        related_name='cart',
        verbose_name=_('Organization'),
    )
    customer = models.OneToOneField(
        Contact,
        blank=True, null=True,
        related_name='cart',
        verbose_name=_('Contact'),
    )
    agent = models.OneToOneField(
        Agent,
        blank=True, null=True,
        related_name='cart',
        verbose_name=_('agent'),
    )
    total_income = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Total Income"),
    )
    total_fee = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Total Fee"),
    )
    total_discount = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Total Discount"),
    )
    total_vat = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Total VAT"),
    )
    objects = CartManager()

    def __iter__(self):
        return iter(self.items.all())

#    def _get_totals(self):
#        totals = {}
#        totals['income'] = self.total_income
#        totals['fee'] = self.total_fee
#        totals['discount'] = self.total_discount
#        totals['vat'] = self.total_vat
#        totals['total'] = self.total_income+self.total_fee-self.total_discount+self.total_vat
#        return totals
#    totals = property(_get_totals)

    def _get_totals(self):
        totals = {}
        totals['total_with_vat'] = self.get_total_price_with_vat()
        totals['total_vat_free'] = self.get_total_price_without_vat()
        return totals
    totals = property(_get_totals)

    def get_total_price_without_vat(self):
        user = get_current_user()
        total_price = 0
        for item in self.items.all():
            #pricelist_item = item.product.product.get_qty_active_pricelistitem(item.quantity, user=user)
            #if pricelist_item:
            total_price += item.total
        return total_price

    def get_total_price_with_vat(self):
        user = get_current_user()
        total_price = 0
        for item in self.items.all():
            #pricelist_item = item.product.product.get_qty_active_pricelistitem(item.quantity, user=user)
            vat_item = item.product.product.vat
            #if pricelist_item:
            #    total_price += pricelist_item.price * item.quantity
            total_price += item.total
            if vat_item:
                total_price += item.total * (vat_item.aliquot_perc/100)
        return total_price

    def remove_all_items(self):
        """Delete all items belonging to this order."""
        for item in self.items.all():
            item.delete()
        self.save()

    def is_all_available(self):
        for cart_item in self.items.all():
            if cart_item.quantity > cart_item.product.availability:
                return False
        return True

    def is_bad_cart(self, shop_product, qty, is_checkout = False,
                    special_discount=None, reservation=None):
        errors = False
        request = get_request()
        product_name = shop_product.product.name

        if special_discount:
            is_bad_discount, error = special_discount.is_bad_discount(quantity=qty)
            if is_bad_discount:
                messages.error(request, error)
                return True

        try:
            qty = int(qty)
        except:
            messages.error(request,
                        ugettext("ATTENTION!!! Quantity must be an integer."))
            errors = True

        # Se non sto facendo il checkout del carrello faccio anche il
        # controllo che il prodotto non sia gia presente nel carrello
        #if not is_checkout:
            #alreadyInCart = False
            #cartitem = self.items.filter(product = shop_product)
            #if cartitem:
                #messages.error(request,
                               #ugettext("ATTENTION!!! Product %s already in the cart." % product_name)
                        #)
                #errors = True

        ## verifico quantita minima
        if shop_product.min_quantity > qty:
            messages.error(request,
                           ugettext("ATTENTION!!! Product %(product_name)s, minimum quantity : %(min_qty)s." % {'product_name':product_name, 'min_qty':shop_product.min_quantity})
                        )
            errors = True

        ## verifico quantita massima
        if shop_product.max_quantity > 0 and shop_product.max_quantity < qty:
            messages.error(request,
                           ugettext("ATTENTION!!! Product %(product_name)s, maximum quantity : %(max_qty)s." % {'product_name':product_name, 'max_qty':shop_product.max_quantity})
                        )
            errors = True

        ## verifico quantita multipla
        if shop_product.mult_quantity > 0 and (qty % shop_product.mult_quantity) != 0:
            messages.error(request,
                           ugettext("ATTENZIONE!!! Prodotto %(product_name)s, la quantita' deve essere un multiplo di %(mult_qty)s." % {'product_name':product_name, 'mult_qty':shop_product.mult_quantity})
                        )
            errors = True

        # Escludo l'item stesso in modo che i controlli all'atto dell'acquisto
        # dell'ordine funzionino correttamente
        cartitems = self.items.exclude(product = shop_product)

        # Controllo se ci sono item che riempiono il basket e quindi
        # non permettono altre operazioni sul carrello
        cart_fill_basket = cartitems.filter(product__fill_basket = True)
        if cart_fill_basket:
            messages.error(request,
                           ugettext("ATTENZIONE!!! Il prodotto precedentemente prenotato non permette altre prenotazioni.")
                        )
            errors = True

        # Se sto cercando di aggiungere un prodotto che riempie il carrello, ma nel carrello
        # ne sono gia presenti altri, do un errore che non si puo' fare
        if cartitems.count() and shop_product.fill_basket:
            messages.error(request,
                           ugettext("ATTENZIONE!!! Il prodotto %s non puo' essere ordinato con altri prodotti." % product_name)
                        )

            errors = True

        return errors

    def add_reservation(self, shop_product, qty, discount, reservation):
        cart_item = self.add_item(shop_product, qty, discount, reservation)
        try:
            cartitem_reservation = CartItemReservation.objects.get(
                reservation=reservation, cart_item=cart_item
            )
            cartitem_reservation.quantity += qty
        except:
            cartitem_reservation = CartItemReservation()
            cartitem_reservation.quantity = qty
            cartitem_reservation.reservation = reservation
            cartitem_reservation.cart_item = cart_item
        cartitem_reservation.save()

    def add_item(self, shop_product, qty, special_discount=None, reservation=None):
        if self.is_bad_cart(shop_product, qty, special_discount=special_discount,):
            return None

        # Aggiungo il prodotto scelto al carrello

        cart_items = self.items.filter(product = shop_product)
        ## se il prodotto è già nel carrello
        if cart_items:
            cart_item = cart_items[0]
            cart_item.quantity += Decimal(qty)
        ## se il prodotto non è nel carrello
        else:
            cart_item = CartItem(cart=self, product=shop_product)
            cart_item.quantity = qty

        #if special_discount:
        #    cart_item.special_discount = special_discount

        if reservation:
            cart_item.reservation = reservation
        cart_item.save()
        return cart_item


    def remove_item(self, item_id, qty_to_remove=None):
        item_to_modify = self.items.get(id = item_id)

        if not qty_to_remove:
            item_to_modify.delete()
        else:
            item_to_modify.quantity -= qty_to_remove
            item_to_modify.save()

            if item_to_modify.quantity <= 0:
                item_to_modify.delete()


    def _get_quantity_items(self):
        items_qty = 0
        for item in self.items.all():
            items_qty += item.quantity
        return (items_qty)
    quantity_items = property(_get_quantity_items)


    def send_order_detail_via_mail(self, procedure,
                                   template_subject="shop/email/order_subject.txt",
                                   template_body_html="shop/email/order_detail.html",
                                   template_body_txt="shop/email/order_detail.txt", **kwargs):
        subject = render_to_string(template_subject, procedure = procedure)
        body_msg = render_to_string(template_body_txt, procedure = procedure)

        if Site._meta.installed:
            site = Site.objects.get_current()
        else:
            site = RequestSite(request)

        kwargs.update({'site':site})

        html_body_msg = render_to_string(template_body_html, procedure = procedure, **kwargs)
        from_mail = get_settings('shop.settings', 'SHOP_FROM_MAIL_ORDER')
        always_send_mail = get_settings('shop.settings', 'SHOP_EMAIL_TO_ALWAYS_SEND_ORDER')
        always_send_mail_bcc = get_settings('shop.settings', 'SHOP_EMAIL_TO_ALWAYS_SEND_ORDER_BCC')

        emails = set()
        if procedure.contact and procedure.contact.email:
            emails.add(procedure.contact.email)

        if procedure.agent and procedure.agent.email:
            emails.add(procedure.agent.email)

        for email in always_send_mail:
            emails.add(email)

        msg = EmailMultiAlternatives(subject, body_msg,
                                    from_mail, emails, always_send_mail_bcc)
        msg.attach_alternative(html_body_msg, "text/html")
        msg.send()


    def checkout(self, send_mail=False, contact=None, organization=None, agent=None, store_cause_code=None):
        cart_items = self.items.all()
        if not cart_items:
            return None

        for item in cart_items:
            # Faccio il controllo degli errori per ogni item del carrello
            if self.is_bad_cart(item.product, item.quantity, is_checkout=True):
                return

        request = get_request()
        # Creo l'ordine
        rows = []
        user = request.user

        ## assegno il CONTATTO se esiste
        if not contact:
            try:
                contact = user.contact
            except:
                pass

        ## assegno ORGANIZATION se esiste
        if not organization and contact:
            organization = contact.organization

        ## assegno LISTINO applicato
        org_price_list = None
        if organization:
            try:
                org_price_list = organization.price_lists.all()[0]
            except:
                pass

        ## assegno AGENT se esiste
        if not agent:
            try:
                agent = user.agent
            except:
                pass

        if not agent and organization:
            ## TODO: se un'organization ha piu agenti bisognerebbe metter
            ## quello di default (che pero' ora non e' gestito quindi prendo il primo)
            agent = organization.agent.filter(status=1)
            if agent:
                agent = agent[0]

        ## TODO: congruenza dati
        if not organization:
            raise Exception("Organization is required")

        try:
            if not store_cause_code:
                store_cause = StoreCause.objects.get(default_for_shop=True)
            else:
                try:
                    store_cause = StoreCause.objects.get(code=store_cause_code)
                except:
                    raise Exception("Store Cause %s Does Not Exists." % store_cause_code)
        except:
            raise Exception("Default Store Cause for the Shop is not set.")

        p = Procedure()
        p.owner = organization
        p.recipient = organization
        p.price_list = org_price_list
        p.contact = contact
        if agent:
            p.agent = agent
        p.description = "Ordine"
        p.date_p = datetime.date.today()
        p.store_cause = store_cause
        p.save()

        # Creo le righe ordine
        for item in cart_items:
            pr = ProcedureRow()
            pr.procedure = p
            pr.product = item.product.product
            pr.vat = item.product.product.vat
            pr.store_cause = store_cause #item.product.product.store_cause
            pr.quantity = item.quantity
            pr.price = item.price_without_discount
            pr.perc_discount1 = org_price_list.discount1
            pr.perc_discount2 = org_price_list.discount2
            #pr.fee = item.fee
            #pr.discount = item.discount
            #if item.special_discount:
            #    pr.long_description = item.special_discount.description+item.special_discount.extra_discount
            pr.ddt = False
            pr.invoice = False
            pr.save()

        # Cancello il carrello e i relativi cart item
        self.delete()

        if send_mail:
            self.send_order_detail_via_mail(p)

        return p


class CartItem(DateModel, UserModel):
    cart = models.ForeignKey(
        Cart,
        related_name="items",
        verbose_name=_("Cart")
    )
    product = models.ForeignKey(
        ShopProduct,
        related_name='items',
        verbose_name=_('shop_items'),
    )
    reservations = models.ManyToManyField(
        "Reservation",
        through='CartItemReservation',
        related_name='cart_items',
        verbose_name=_("Reservations"),
        blank = True, null = True
    )
    quantity = models.DecimalField(
        max_digits = 9,
        decimal_places = 2,
        verbose_name=_("quantity"),
    )
    short_description = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Short Description")
    )

    class Meta:
        verbose_name = _("Cart Item")
        verbose_name_plural = _("Cart Items")
        ordering = ('id',)

    def __unicode__(self):
        return u'%s ' % (self.description)

    def get_price(self):
        """Get the price for for each unit before any detail modifications"""
        user = get_current_user()
        price = self.product.product.get_organization_price(organization=user.contact.organization, qty=self.quantity)
        return price if price else 0
    price = property(get_price)

    def get_price_without_discount(self):
        """Get the price for for each unit before any detail modifications"""
        user = get_current_user()
        price = self.product.product.get_organization_price_without_discount(organization=user.contact.organization, qty=self.quantity)
        return price if price else 0
    price_without_discount = property(get_price_without_discount)

    def _get_description(self):
        return self.short_description or self.product.product.name
    description = property(_get_description)

#    def _get_totals_income(self):
#        t = {
#            'cost_price' : self.quantity * self.cost_price,
#            'reserved_price': self.quantity * self.reserved_price,
#            'street_price' : self.quantity * self.street_price
#            }
#        return t
#    totals_income = property(_get_totals_income)

#    def _get_total_fee(self):
#        return self.quantity * self.fee
#    total_fee = property(_get_total_fee)
#
#    def _get_total_discount(self):
#        return self.quantity * self.discount
#    total_discount = property(_get_total_discount)

    def _get_total(self):
        return self.get_price() * self.quantity
    total = property(_get_total)


    def _is_bad_item(self):
        ## verifico quantita minima
        if self.product.min_quantity > self.quantity:
            return _("ATTENTION!!! Product error minimum quantity.")

        ## verifico quantita massima
        if self.product.max_quantity > 0 and self.shop.max_quantity < self.quantity:
            return _("ATTENTION!!! Product error maximum quantity.")

        ## verifico quantita multipla
        if self.product.mult_quantity > 0 and (self.quantity % self.product.mult_quantity) != 0:
            return _("ATTENTION!!! Product error multiple quantity.")

        ## verifico carrello non vuoto
        if self.product.fill_basket and self.cart.quantity_items > 0 or self.cart.is_full:
            return _("ATTENTION!!! Cart is full.")

        ## verifico richiesta form aggiuntiva e completezza eventuale
        if self.product.shop.request_form:
            pass

        ## verifico orario di ufficio
        if self.product.office_time:
            now = datetime.datetime.now()
            weekday = now.weekday()
            office_time = settings.OFFICE_TIME or ""
            day_time = office_time.get(weekday, None)
            if day_time and office_time:
                for interval in day_time:
                    if now.time() <= interval[0] or now.time() >= interval[1]:
                        return _("ATTENTION!!! Product Time OUT of Range TIME.")

        return False
    is_bad_item = property(_is_bad_item)


    def get_booked_quantity(self):
        total_booked_quantity = CartItem.objects.filter(product=self.product, date_create__lte=self.date_create).aggregate(Sum('quantity'))
        return total_booked_quantity['quantity__sum']


    def get_conditional_quantity(self):
        qty = self.get_booked_quantity() - self.special_discount.qty
        if qty < 0:
            return 0
        return qty

    def get_order_quantity(self):
        qty = self.quantity - self.get_conditional_quantity()
        return qty

    #def is_available(self):
    #    total_booked_quantity = self.get_booked_quantity
    #    if total_booked_quantity >


class BuyoutMail(DateModel, UserModel):
    number = models.IntegerField(
        db_index=True,
        verbose_name=_("Number"),
        help_text=_('Mail number, refers to procedure number.'),
    )
    contact = models.ForeignKey(
        Contact,
        related_name='buyouts',
        verbose_name=_('Contact'),
    )
    procedure = models.ForeignKey(
        Procedure,
        related_name='buyout_mails',
        verbose_name=_('procedure'),
        help_text=_('procedure for this buyout.'),
    )
    message = models.TextField(
        verbose_name=_('Message'),
        help_text=_('Mail message'),
    )

    class Meta:
        verbose_name=_('Buyout mail')
        verbose_name_plural = _('Buyout mails')
        ordering = ('-date_create',)

    def __unicode__(self):
        return u'Buyout #%(id)s by %(contact)s' % {'id': self.procedure.number, 'contact': self.contact}

    def save(self, user=None, force_insert=False, force_update=False):
        super(BuyoutMail, self).save(force_insert=force_insert, force_update=force_update)

    @models.permalink
    def get_absolute_url(self):
        return ('admin_shop_buyoutmail_change', (str(self.id),))


class ReservationManager(models.Manager):
    def for_user(self):
        qs = super(ReservationManager, self).get_query_set()
        request = get_request()

        return qs.filter(creator=request.user)


class Reservation(UserModel):
    product = models.ForeignKey(
        ShopProduct,
        related_name='reservations',
        verbose_name=_('Product'),
    )
    discount = models.ForeignKey(
        Discount,
        verbose_name = _("Discount"),
        related_name = 'reservations'
    )
    quantity = models.DecimalField(
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Quantity")
    )
    reserved_at = models.DateTimeField(
        verbose_name = _("Reserved At")
    )
    reservation_expire = models.DateTimeField(
        verbose_name = _("Reservation Expire")
    )
    objects = ReservationManager()

    #def is_bad_reservation(self, is_update=False):
    #    errors = False
    #    request = get_request()
    #
    #    is_bad_discount, error = self.discount.is_bad_discount(quantity=self.quantity)
    #    if is_bad_discount:
    #        messages.error(request, error)
    #        return True
    #
    #    # is_update viene settato a True quando si compra solo un parte
    #    # della quantita della reservation e, dato che deve venire aggiornata
    #    # la quantita, non faccio il controllo dell'esistenza della reservation
    #    #if not is_update:
    #    #    already_reserved = Reservation.objects.for_user().filter(product=self.product, discount=self.discount)
    #
    #    #    if already_reserved:
    #    #        messages.error(request,
    #    #                       "Attention!!! This product was already reserved.")
    #    #        errors = True
    #
    #    return errors


    def get_reserved_quantity(self):
        """
        quantità totale prenotata per un singolo prodotto
        """
        total_reserved_quantity = Reservation.objects.filter(product=self.product, reserved_at__lte=self.reserved_at).aggregate(Sum('quantity'))
        return total_reserved_quantity['quantity__sum']


    def get_conditional_quantity(self):
        """
        quantità totale prenotata ma non ordinabile
        """
        qty = self.get_reserved_quantity() - self.discount.qty
        if qty < 0:
            return 0
        if qty > self.quantity:
            return self.quantity
        return qty


    def get_order_quantity(self):
        """
        quantità ordinabile
        """
        qty = self.quantity - self.get_conditional_quantity()
        if qty < 0:
            return 0
        return qty

    #def is_available(self):
    #    total_booked_quantity = self.get_booked_quantity
    #    if total_booked_quantity >

    def get_cart_quantity(self):
        """
        restitutisce la quantita prenotata gia inserita nel carrello corrente
        """
        qty_sum = self.cart_reservations.all().aggregate(
            Sum('quantity')
        )['quantity__sum'] or 0
        return qty_sum

    def get_valid_quantity(self):
        """
        restituisce la quantita ancora ordinabile.
        La quantita ordinabile è la differenza della quantita prenotata meno la quantita gia inserita nel carrello.
        Se è maggiore della quantità dispononibile allora è la quantità disponibile
        """
        return self.get_order_quantity() - self.get_cart_quantity()

    def save(self, is_update=False, *args, **kws):
        is_bad_discount, error = self.discount.is_bad_discount(quantity=self.quantity)
        if is_bad_discount:
            #messages.error(request, error)
            return None

        #request = get_request()
        #if not self.controllo_promozione():
        #    messages.error(request, "La promozione non e' piu' valida.")
        #    return

        if not self.reservation_expire:
            self.reservation_expire = self.reserved_at + datetime.timedelta(days=1)
        super(Reservation, self).save(*args, **kws)


    def __unicode__(self):
        return "%s - %s - %s" % (self.discount, self.quantity, self.reserved_at)


    class Meta:
        verbose_name = _("Reservation")
        verbose_name_plural = _("Reservations")
        ordering = ['reserved_at']

class CartItemReservation(models.Model):
    reservation = models.ForeignKey(
        Reservation,
        related_name='cart_reservations'
    )
    quantity = models.DecimalField(
        max_digits = 9,
        decimal_places = 2,
    )
    cart_item = models.ForeignKey(
        CartItem,
        related_name='cart_reservations',
    )
    class Meta:
        unique_together=(('reservation', 'quantity'),)
