from django.template import Library, Node, TemplateSyntaxError
from shop.models import ShopCategory, ShopProduct

try:
    from xml.etree.ElementTree import Element, SubElement, tostring
except ImportError:
    from elementtree.ElementTree import Element, SubElement, tostring

register = Library()

def recurse_for_children(current_node, parent_node, active_cat, level=1,
                         parent_slugs=[], max_level = None, show_empty=True,
                         exclude_root_node=False):
    children = current_node.get_children()
    child_count = len(children)

    if show_empty or child_count > 0: #or current_node.product_set.count() > 0:
        attrs = {}
        if current_node == active_cat or current_node.slug in parent_slugs:
            attrs["class"] = "current"
        if not current_node.is_child_node():
            if attrs.has_key("class"):
                attrs["class"] = attrs["class"] + " first"
            else:
                attrs["class"] = "first"
        else:
            if attrs.has_key("class"):
                attrs["class"] = attrs["class"] + " child"
            else:
                attrs["class"] = "child"
        temp_parent = SubElement(parent_node, 'li', attrs)
        attrs = {'href': current_node.get_absolute_url()}

        link = SubElement(temp_parent, 'a', attrs)
        span = SubElement(link, 'span')
        #span.text = current_node.translated_name()
        if exclude_root_node:
            span.text = ""
        else:
            span.text = current_node.name

        clear = SubElement(temp_parent, 'div', {'class':'clear'})
        clear.text = ' '
        if max_level and level >= max_level:
            return

        if child_count > 0:
            new_parent = SubElement(temp_parent, 'ul')
            #children = current_node.child.all()
            for child in children:
                recurse_for_children(child, new_parent, active_cat, level=level+1, max_level=max_level)

def shop_category_tree(id=None, parent_slugs=[], max_level = None, exclude_root_node = False, root_cat_id=None):
    """
    Creates an unnumbered list of the categories.  
    
    Example::
        <ul>
            <li>Books
                <ul>
                <li>Science Fiction
                    <ul>
                    <li>Space stories</li>
                    <li>Robot stories</li>
                    </ul>
                </li>
                <li>Non-fiction</li>
                </ul>
        </ul>
    """
    active_cat = None
    if id:
        active_cat = ShopCategory.objects.filter(id=id)

    root = Element("ul", {'id':'root_menu'})

    if root_cat_id:
        shop_category = ShopCategory.objects.filter(id=root_cat_id)
    else:
        shop_category = ShopCategory.tree.root_nodes()
    shop_category = shop_category.order_by('ordering','name')

    for node in shop_category:
        recurse_for_children(node, root, active_cat,
                             parent_slugs=parent_slugs,
                             exclude_root_node = exclude_root_node,
                             max_level=max_level)
    return tostring(root, 'utf-8')
register.simple_tag(shop_category_tree)


class ShopCategoryListNode(Node):
    """Template Node tag which pushes the category list into the context"""
    def __init__(self, slug, var):
        self.var = var
        self.slug = slug
        #self.nodelist = nodelist

    def render(self, context):
        if self.slug:
            try:
                cat = ShopCategory.objects.get(slug__iexact=self.slug)
                cats = cat.children.all()
            except ShopCategory.DoesNotExist:
                log.warn("No shop category found for slug: %s", self.slug)
                cats = []

        else:
            cats = ShopCategory.tree.root_nodes().order_by('ordering')

        context[self.var] = cats
        return ''

def do_shop_categorylistnode(parser, token):
    """Push the shop category list into the context using the given variable name.

    Sample usage::

        {% shop_category_list slug as var %}
        or
        {% shop_category_list as var %}
        

    """
    args = token.split_contents()
    ct = len(args)
    if not ct in (3,4):
        raise TemplateSyntaxError("%r tag expecting '[slug] as varname', got: %s" % (args[0], args))
    
    if ct == 3:
        slug = None
        var = args[2]
    else:
        slug = args[1]
        var = args[3]
    #nodelist = parser.parse(('endcatalog_category_list',))
    #parser.delete_first_token()

    return ShopCategoryListNode(slug, var)
register.tag('shop_category_list', do_shop_categorylistnode)


class ShopProductFromRequestNode(Node):
    """Template Node tag which pushes the category list into the context"""
    def __init__(self, var):
        self.var = var

    def render(self, context):
        # try to find "prod" in request GET and return the related product
        # otherwise return the first category product
        prod = None
        request = context['request']
        id_prod = request.GET.get('id_prod', None)

        if not id_prod:
            prod =  context['shop_category'].get_first_product()
        else:
            try:
                prod = ShopProduct.objects.get(pk=id_prod)
            except:
                pass

        context[self.var] = prod
        return ''

def do_get_shopproduct_from_request(parser, token):
    """Try to find "prod" in request GET and put the found product into the context using the given variable name.
       If any request get variable was found return the first catalog product

    Sample usage::

        {% get_shopproduct_from_request as my_product %}
    """
    args = token.split_contents()
    var = args[2]

    return ShopProductFromRequestNode(var)
register.tag('get_shopproduct_from_request', do_get_shopproduct_from_request)



class ShopCategoryProductsNode(Node):
    """Template Node tag which pushes the products list of a specify category into the context"""
    def __init__(self, slug, var):
        self.var = var
        self.slug = slug

    def render(self, context):
        try:
            cat = ShopCategory.objects.get(slug__iexact=self.slug)
            products = cat.shop_products.all()
        except ShopCategory.DoesNotExist:
            #log.warn("No shop category found for slug: %s", self.slug)
            products = []

        context[self.var] = products
        return ''

def do_shop_product_listnode(parser, token):
    """Push the shop product list into the context using the given variable name.

    Sample usage::

        {% shop_product_list slug as products %}

    """
    args = token.split_contents()
    ct = len(args)
    if not ct in (3,4):
        raise TemplateSyntaxError("%r tag expecting '[slug] as varname', got: %s" % (args[0], args))
    
    if ct == 3:
        raise TemplateSyntaxError("%r tag expecting '[slug] parameter', got: %s" % (args[0], args))
    else:
        slug = args[1]
        var = args[3]

    return ShopCategoryProductsNode(slug, var)
register.tag('shop_product_list', do_shop_product_listnode)