# -*- coding: utf-8 -*-

import decimal
import datetime

import django
from django.utils.translation import ugettext as _
from django.conf import settings
from django.core.mail import EmailMessage
from django.http import HttpResponseRedirect
from django.db import transaction
from django.core.urlresolvers import reverse
from django.utils.http import urlquote
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.paginator import Paginator, InvalidPage, EmptyPage
from django.template import RequestContext

from jumbo.shortcuts import render_to_response, render_to_string, get_queryset_or_404, get_object_or_404
from jumbo.utils.functions import get_settings

from shop.models import ShopCategory, Cart, CartItem, ShopProduct, Discount, Reservation
from procedure.models import Procedure, ProcedureRow

ACTION_STREAM = False
try:
    from actstream import action
    ACTION_STREAM = True
except:
    pass

def catalog(request, template_name='shop/catalog.html'):
    try:
        paginator = Paginator(ShopCategory.objects.all(), 25) # Show 25 contacts per page
    except ShopCategory.DoesNotExist:
        raise Exception("There isnt't any Shop Category")

    # Make sure page request is an int. If not, deliver first page.
    try:
        page = int(request.GET.get('page', '1'))
    except ValueError:
        page = 1

    # If page request (9999) is out of range, deliver last page of results.
    try:
        shop_category = paginator.page(page)
    except:
        shop_category = paginator.page(paginator.num_pages)

    ctx = {
        'shop_category': shop_category,
    }
    return render_to_response(template_name,
        shop_category=shop_category)


def shopcategory_detail(request, slug, parent_slugs='', template_name='shop/category_detail.html'):
    """Display the shop category, its child categories, and its shop products.

    Parameters:
     - slug: slug of category
     - parent_slugs: ignored
    """
    try:
        category  =  ShopCategory.objects.get(slug=slug)
        paginator = Paginator(category.shop_products.all(), 25) # Show 25 contacts per page
        paginator_cat = Paginator(category.get_children().filter(status=1).order_by("tree_id", "level", "ordering"), 25)
    except ShopCategory.DoesNotExist:
        return render_to_response('shop/category_does_not_exist.html')
        #raise Exception("There isnt't any Shop Category")

    # Make sure page request is an int. If not, deliver first page.
    try:
        page = int(request.GET.get('page', '1'))
        page_cat = int(request.GET.get('page_cat', '1'))
    except ValueError:
        page = 1
        page_cat = 1

    # If page request (9999) is out of range, deliver last page of results.
    try:
        products = paginator.page(page)
    except:
        products = paginator.page(paginator.num_pages)

    try:
        child_categories = paginator_cat.page(page_cat)
    except (EmptyPage, InvalidPage):
        child_categories = paginator_cat.page(paginator_cat.num_pages)

    # level-based template
    level = 0
    categ = category
    while categ and level < 1000:
        categ = categ.parent
        level += 1

    # try level & code
    if category.recursive_get_code():
        template_level = 'shop/category_detail%s.html' % "_%s_level%s" % (category.recursive_get_code(), level)
        try:
            django.template.loader.get_template(template_level)
            template_name = template_level
        except django.template.TemplateDoesNotExist: pass
    else:
        # try level
        template_level = 'shop/category_detail%s.html' % "_level%s" % level
        try:
            django.template.loader.get_template(template_level)
            template_name = template_level
        except django.template.TemplateDoesNotExist: pass

    ctx = {
        'shop_category': category,
        'child_categories': child_categories,
        'shop_products': products,
    }
    return render_to_response(template_name, shop_category=category,
                              child_categories=child_categories, shop_products=products)


@login_required
def order_buy(request, template_name='shop/order_buy.html'):
    user_cart = Cart.objects.from_request(request)
    send_mail = get_settings('shop.settings', 'SHOP_SEND_ORDER_DETAIL_VIA_MAIL')
    procedure = user_cart.checkout(send_mail=send_mail, organization=organization)

    if ACTION_STREAM:
        action.send(request.user, verb='Acquisto Ordine', action_object=procedure)

    #procedure = user_cart.checkout()
    return render_to_response(template_name, procedure=procedure)

@login_required
def order_confirm(request, template_name='shop/order_confirm.html'):
    return render_to_response(template_name)

@login_required
def order_detail(request, order_id, template_name='shop/order_detail.html'):
    procedure = Procedure.objects.get(id=order_id)
    return render_to_response(template_name, procedure=procedure)

@login_required
def add_to_cart(request, shop_product_id, redirect_url=None, redirect_to_referer=False):
    referer = request.META['HTTP_REFERER']

    if not redirect_url:
        if redirect_to_referer:
            redirect_url = referer
        else:
            redirect_url = reverse('shop_cart_list')

    qty = 1

    if request.POST.has_key('shopproduct_mod_qty') and not request.POST['shopproduct_mod_qty'].isdigit():
        messages.error(request, _("Insert a valid quantity."))
        return HttpResponseRedirect(referer)
    if request.POST.has_key('shopproduct_mod_qty'):
        qty = request.POST.get('shopproduct_mod_qty')

    try:
        shop_product = ShopProduct.objects.get(pk=shop_product_id)
    except:
        messages.error(request, _("The requested product does not exists."))
        return HttpResponseRedirect(referer)

    user_cart = Cart.objects.from_request(request)
    item = user_cart.add_item(shop_product, int(qty))
    if not item:
        messages.error(request, _("The requested product does not exists."))
        return HttpResponseRedirect(referer)

    messages.info(request, _("The product %(product_name)s was successfully added to cart.")  %
                                                                {'product_name':shop_product.name})
    return HttpResponseRedirect(redirect_url)

@login_required
def add_to_cart_from_request(request, redirect_url=None, redirect_to_referer=False):
    referer = request.META['HTTP_REFERER']
    if not redirect_url:
        if redirect_to_referer:
            redirect_url = referer
        else:
            redirect_url = reverse('shop_cart_list')

    post = request.POST.copy()
    if post.has_key('next'):
        redirect_url = post['next']

    user_cart = Cart.objects.from_request(request)

    is_shop_product = post.get('shopproduct_id', False)
    is_discount =  post.get('discount_id', False)
    is_reservation =  post.get('reservation_id', False)

    if not is_shop_product and not is_discount:
        messages.error(request, _("Invalid Request. Can add this item to the cart."))
        return HttpResponseRedirect(referer)

    if is_discount:
        try:
            discount = Discount.objects.get(pk=post['discount_id'])
        except:
            messages.error(request, _("The requested discount product does not exists."))
            return HttpResponseRedirect(referer)

    if is_reservation:
        try:
            reservation = Reservation.objects.get(pk=post['reservation_id'])
        except:
            messages.error(request, _("The requested reservation does not exists."))
            return HttpResponseRedirect(referer)

    try:
        shop_product = ShopProduct.objects.get(pk=post['shopproduct_id'])
    except:
        messages.error(request, _("The requested product does not exists."))
        return HttpResponseRedirect(referer)

    if not post['shopproduct_qty'].isdigit():
        messages.error(request, _("Insert a valid quantity."))
        return HttpResponseRedirect(referer)

    if is_discount:
        if is_reservation:
            item = user_cart.add_reservation(
                shop_product, post['shopproduct_qty'],
                discount=discount,
                reservation=reservation
            )
        else:
            item = user_cart.add_item(shop_product, post['shopproduct_qty'],
                                      special_discount=discount)
    else:
        item = user_cart.add_item(shop_product, post['shopproduct_qty'])

    if not item:
        return HttpResponseRedirect(referer)

    return HttpResponseRedirect(redirect_url)


@login_required
def empty_cart(request, redirect_url=None):
    user_cart = Cart.objects.from_request(request)
    user_cart.remove_all_items()

    if not redirect_url:
        redirect_url = request.META['HTTP_REFERER']

    messages.info(request, _("Your cart is now empty."))

    return HttpResponseRedirect(redirect_url)

@login_required
def remove_from_cart(request, item_id, redirect_url=None, redirect_to_referer=False):
    referer = request.META['HTTP_REFERER']
    if not redirect_url:
        if redirect_to_referer:
            redirect_url = referer
        else:
            redirect_url = reverse('shop_cart_list')

    try:
        user_cart = Cart.objects.from_request(request)
        user_cart.remove_item(item_id)
    except:
        messages.error(request, _("There was an error while deleting items. Please contact the administrator."))
        return HttpResponseRedirect(referer)

    messages.info(request, _("The product was successfully removed from cart."))

    return HttpResponseRedirect(redirect_url)

@login_required
def change_cartitem_quantity(request, cartitem_id):
    post = request.POST.copy()
    referer = request.META['HTTP_REFERER']

    if not post.has_key('shopproduct_mod_qty') or not post['shopproduct_mod_qty'].isdigit():
        messages.error(request, _("Insert a valid quantity."))
        return HttpResponseRedirect(referer)

    user_cart = Cart.objects.from_request(request)

    try:
        cartitem = user_cart.items.get(id=cartitem_id)
    except:
        messages.error(request, _("Cart Item does not exists."))
        return HttpResponseRedirect(referer)

    if user_cart.is_bad_cart(cartitem.product, post['shopproduct_mod_qty']):
        return HttpResponseRedirect(referer)

    cartitem.quantity = post['shopproduct_mod_qty']
    cartitem.save()

    return HttpResponseRedirect(referer)


@login_required
def list_discount(request, template_name='shop/list_discount.html'):
    discounts = Discount.active.all()
    return render_to_response(template_name, discounts=discounts)


@login_required
def list_reservation(request, discount_id, template_name="shop/list_reservation.html"):
    discount = get_object_or_404(Discount, pk=discount_id)
    return render_to_response(template_name, discount=discount)

def list_all_reservations(request, template_name="shop/list_all_reservations.html"):
    reservations = Reservation.objects.all().order_by('product', 'reserved_at')
    return render_to_response(template_name, reservations=reservations)

@login_required
def add_reservation(request):
    post = request.POST.copy()
    referer = request.META['HTTP_REFERER']
    next = referer
    if post.has_key('next'):
        next = post['next']

    is_discount =  post.get('discount_id', False)

    if not is_discount:
        messages.error(request, _("The requested discount product does not exists."))
        return HttpResponseRedirect(referer)

    try:
        discount = Discount.objects.get(pk=post['discount_id'])
    except:
        messages.error(request, _("The requested discount product does not exists."))
        return HttpResponseRedirect(referer)

    try:
        shop_product = ShopProduct.objects.get(pk=post['shopproduct_id'])
    except:
        messages.error(request, _("The requested product does not exists."))
        return HttpResponseRedirect(referer)

    if not post['shopproduct_qty'].isdigit():
        messages.error(request, _("Insert a valid quantity."))
        return HttpResponseRedirect(referer)

    is_bad_discount, error = discount.is_bad_discount(quantity=post['shopproduct_qty'])
    if is_bad_discount:
        messages.error(request, error)
        return HttpResponseRedirect(referer)

    # inserisco la prenotazione
    p = Reservation()
    p.product = shop_product
    p.discount = discount
    p.quantity = post['shopproduct_qty']
    p.reserved_at = datetime.datetime.today()
    p.save()

    if ACTION_STREAM:
        action.send(request.user, verb='Effettuata Prenotazione', action_object=p)

    return HttpResponseRedirect(next)


@login_required
def delete_reservation(request, reservation_id):
    referer = request.META['HTTP_REFERER']

    try:
        reservation = Reservation.objects.get(id=reservation_id)
        reservation.delete()
    except:
        messages.error(request, _("There was an error while deleting items. Please contact the administrator.."))
        return HttpResponseRedirect(referer)
    return HttpResponseRedirect(referer)
