from django.test import TestCase
from django.db import IntegrityError
from store.models import Store, StoreCause, StoreDocument

class StoreTest(TestCase):
    def setUp(self):
        pass
    def tearDown(self):
        pass

    def test_address_can_be_null(self):
        l = Store.objects.count()
        self.assertEquals(l, 0)
        store = Store(
            code='NO1',
            name='Test1',
            description='Test1',
        )
        store.save()
        self.assertEquals(Store.objects.count(), l+1)
    def test_name_must_be_unique(self):
        store = Store(
            code='CODE1',
            name='test',
            description='',
        )
        store.save()
        store = Store(
            code='CODE2',
            name='test',
            description='',
        )
        self.assertRaises(IntegrityError, store.save)
    def test_code_must_be_unique(self):
        store = Store(
            code='CODE',
            name='test1',
            description='',
        )
        store.save()
        store = Store(
            code='CODE',
            name='test2',
            description='',
        )
        self.assertRaises(IntegrityError, store.save)
    def test_cannot_insert_two_identical_stores(self):
        store = Store(
            code='CODE',
            name='test',
            description='',
        )
        store.save()
        store = Store(
            code='CODE',
            name='test',
            description='',
        )
        self.assertRaises(IntegrityError, store.save)

class StoreDocumentTest(TestCase):
    def setUp(self):
        pass
    def tearDown(self):
        pass

    def test_name_must_be_unique(self):
        doc = StoreDocument(
            code='CODE1',
            name='test',
            description='',
        )
        doc.save()
        doc = StoreDocument(
            code='CODE2',
            name='test',
            description='',
        )
        self.assertRaises(IntegrityError, doc.save)
    def test_code_must_be_unique(self):
        doc = StoreDocument(
            code='CODE',
            name='test1',
            description='',
        )
        doc.save()
        doc = StoreDocument(
            code='CODE',
            name='test2',
            description='',
        )
        self.assertRaises(IntegrityError, doc.save)
    def test_cannot_insert_two_identical_documents(self):
        doc = StoreDocument(
            code='CODE',
            name='test',
            description='',
        )
        doc.save()
        doc = StoreDocument(
            code='CODE',
            name='test',
            description='',
        )
        self.assertRaises(IntegrityError, doc.save)

class StoreCauseTest(TestCase):
    def setUp(self):
        self.store = Store(
            code='CODE',
            name='test',
            description='',
        )
        self.store.save()
    def tearDown(self):
        pass

    def test_fail_if_store_not_provided(self):
        cause = StoreCause(
            code='hippo',
            name='test',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        self.assertRaises(IntegrityError, cause.save)
    def test_fail_if_code_not_provided(self):
        cause = StoreCause(
            store=self.store,
            code=None,
            name='test',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        self.assertRaises(IntegrityError, cause.save)
    def test_fail_if_name_not_provided(self):
        cause = StoreCause(
            store=self.store,
            code='CODE',
            name=None,
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        self.assertRaises(IntegrityError, cause.save)
    def test_fail_if_move_not_provided(self):
        cause = StoreCause(
            store=self.store,
            code='CODE',
            name='test',
            description='',
            move=None,
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        self.assertRaises(IntegrityError, cause.save)
    def test_fail_if_ddt_not_provided(self):
        cause = StoreCause(
            store=self.store,
            code='CODE',
            name='test',
            description='',
            move='not',
            invoice=False,
            ddt=None,
            stock=0,
            busy=0,
        )
        self.assertRaises(IntegrityError, cause.save)
    def test_fail_if_stock_not_provided(self):
        cause = StoreCause(
            store=self.store,
            code='CODE',
            name='test',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            busy=0,
        )
        self.assertRaises(IntegrityError, cause.save)
    def test_fail_if_busy_not_provided(self):
        cause = StoreCause(
            store=self.store,
            code='CODE',
            name='test',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
        )
        self.assertRaises(IntegrityError, cause.save)
    def test_name_must_be_unique(self):
        cause = StoreCause(
            store=self.store,
            code='CODE1',
            name='test_unique',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        cause.save()
        cause = StoreCause(
            store=self.store,
            code='CODE2',
            name='test_unique',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        self.assertRaises(IntegrityError, cause.save)
    def test_code_must_be_unique(self):
        cause = StoreCause(
            store=self.store,
            code='CODE_UNIQUE',
            name='test1',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        cause.save()
        cause = StoreCause(
            store=self.store,
            code='CODE_UNIQUE',
            name='test2',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        self.assertRaises(IntegrityError, cause.save)
    def test_cannot_insert_two_identical_cause_stores(self):
        cause = StoreCause(
            store=self.store,
            code='CODE',
            name='test',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        cause.save()
        cause = StoreCause(
            store=self.store,
            code='CODE',
            name='test',
            description='',
            move='not',
            ddt=False,
            invoice=False,
            stock=0,
            busy=0,
        )
        self.assertRaises(IntegrityError, cause.save)

